/*
 * hfcl_timer_recovery.c
 * Copyright (C) 2007, 2012, Hitachi, Ltd.
 * Author(s): Yoshihiro Toyohara <yoshihiro.toyohara.qs@hitachi.com>
 */

char timer_rcsid[] = "$Id: hfcl_timer_recovery.c,v 1.85.2.25.2.32.2.6.2.20.2.7 2012/05/25 17:51:54 toyo Exp $";

/*--------------------------------------------------------------------------*/
/*    hfc_handler()                                                         */
/*        +--hfc_hwerr_int()                                                */
/*        +--hfc_mb_resp()                                                  */
/*        |      +--hfc_issue_relogin()                                     */
/*        |      +--hfc_cancel_scsi_cmd()                                   */
/*        |      +--hfc_issue_pdisc()                                       */
/*        +--hfc_mb_intreq()                                                */
/*        |      +--hfc_cancel_scsi_cmd()                                   */
/*        +--hfc_xrb_resp()                                                 */
/*        |      +--hfc_abend()                                             */
/*        |      +--hfc_link_chk()                                          */
/*        |      +--hfc_task_mgm_chk()                                      */
/*        |      |      +--hfc_deque_we_que()                               */
/*        |      |      +--hfc_iov_update()                                 */
/*        |      |      +--hfc_dma_unmap()                                  */
/*        |      |      +--hfc_start()                                      */
/*        |      |      +--hfc_cancel_weque()                               */
/*        |      +--hfc_scsi_chk()                                          */
/*        |             +--hfc_enque_wr_que()                               */
/*        |             +--hfc_deque_we_que()                               */
/*        |             +--hfc_iov_update()                                 */
/*        |             +--hfc_dma_unmap()                                  */
/*        |                                                                 */
/*--------------------------------------------------------------------------*/


#include "hfcldd.h"
#include "hfcl_tbol.h"
#include "hfcl_strategy.h"
#include "hfcl_handler.h"
#include "hfcl_stra_trace.h"
#include "hfcl_timer_recovery.h"
#include "hfcl_top.h"
#include "hfcl_hand_timer_trace.h"
#include "hfcl_detect.h"
#include "hfcl_ioctl.h"
#include "hfcl_mlpf.h"
#include "hfcldd_conf.h"

#ifndef _HFC_NO_RASLOG
#include "hraslog.h"
#endif

#if defined(__x86_64)  /* FCLNX-0398 */ /* FCLNX-0629 */
extern void touch_nmi_watchdog(void);
#endif					/* FCLNX-0398 */

const struct errlog_t errlog_info[] = {
	{ ERRID_HFCP_ERR1 ,		"HFC_ERR1 Permanent FC Adapter Hardware error" },
	{ ERRID_HFCP_ERR2 ,		"HFC_ERR2 Temporary FC Adapter Hardware error" },
	{ ERRID_HFCP_ERR3 ,		"HFC_ERR3 Permanent FC Adapter Firmware error" },
	{ ERRID_HFCP_ERR4 ,		"HFC_ERR4 Temporary FC Adapter Firmware error" },
	{ ERRID_HFCP_ERR5 ,		"HFC_ERR5 Permanent FC Link error" },
	{ ERRID_HFCP_ERR6 ,		"HFC_ERR6 Temporary FC Link error" },
	{ ERRID_HFCP_ERR7 ,		"HFC_ERR7 Permanent FC Adapter Unknown error" },
	{ ERRID_HFCP_ERR8 ,		"HFC_ERR8 Temporary FC Adapter Unknown error" },
	{ ERRID_HFCP_ERR9 ,		"HFC_ERR9 FC Adapter Driver error" },
	{ ERRID_HFCP_ERRA ,		"HFC_ERRA FC Adapter Interrupt time-out" },
	{ ERRID_HFCP_ERRB ,		"HFC_ERRB FC Adapter Link Down" },
	{ ERRID_HFCP_ERRC ,		"HFC_ERRC FC Adapter Diagnostics error" },
	{ ERRID_HFCP_ERRD ,		"HFC_ERRD FC Adapter PCI error" },
	{ ERRID_HFCP_ERRE ,		"HFC_ERRE FC Adapter I/O Unavailable" },
	{ ERRID_HFCP_ERRF ,		"HFC_ERRF FC Adapter Initialize error" },
	{ ERRID_HFCP_EVNT1,		"HFC_EVNT1 FC Adapter Link Up" },
	{ ERRID_HFCP_EVNT2,		"HFC_EVNT2 FC Adapter Link Changed" },
	{ ERRID_HFCP_EVNT3,		"HFC_EVNT3 FC Adapter Driver Warning Event" },
	{ ERRID_HFCP_EVNT4,		"HFC_EVNT4 FC Adapter Driver Request Log" },
	{ ERRID_HFCP_OPTERR0,	"HFC_OPTERR0 Invalid Optical Module install" },
	{ 0xFFFFFFFF      ,	"" },
};

const struct hraslog_t hraslog_info[] = {
	{ ERRID_HFCP_ERR1 ,	"E1", "R1" },
	{ ERRID_HFCP_ERR2 ,	"E2", "R2" },
	{ ERRID_HFCP_ERR3 ,	"E3", "R3" },
	{ ERRID_HFCP_ERR4 ,	"E4", "R4" },
	{ ERRID_HFCP_ERR5 ,	"E5", "R5" },
	{ ERRID_HFCP_ERR6 ,	"E6", "R6" },
	{ ERRID_HFCP_ERR7 ,	"E7", "R7" },
	{ ERRID_HFCP_ERR8 ,	"E8", "R8" },
	{ ERRID_HFCP_ERR9 ,	"E9", "R9" },
	{ ERRID_HFCP_ERRA ,	"EA", "RA" },
	{ ERRID_HFCP_ERRB ,	"EB", "RB" },
	{ ERRID_HFCP_ERRC ,	"EC", "RC" },
	{ ERRID_HFCP_ERRD ,	"ED", "RD" },
	{ ERRID_HFCP_ERRE ,	"EE", "RE" },
	{ ERRID_HFCP_ERRF ,	"EF", "RF" },
	{ ERRID_HFCP_EVNT1,	"I1", "N1" },
	{ ERRID_HFCP_EVNT2,	"I2", "N2" },
	{ ERRID_HFCP_EVNT3,	"I3", "N3" },
	{ ERRID_HFCP_EVNT4,	"I4", "N4" },
	{ ERRID_HFCP_OPTERR0, "P0", "T0" },
	{ 0xFFFFFFFF      ,	 "" ,  ""  },
};

#define HFC_MFC			512
#define HFC_MODELNAME	528
#define	HFC_FWVERSION	560
#define	HFC_DRVVERSION	576
#define	HFC_PARTSNUM	592
#define	HFC_LOCATION	608
#define	HFC_LOGSEQNUM	624
#define	HFC_IRQNUM		628
#define	HFC_EC			632
#define	HFC_ADAPWWPN	640
#define	HFC_DEVICEID	656
#define HFC_WARNING		660
#define	HFC_SFP_INFO	760								/* FCLNX-GPL-526 */

#ifdef _HFC_FCGW										/* FC-GW */
const struct errlog_t errlog_disk[] = {					/* FC-GW */
	{ 0 ,				"HFC_DISKERR1 Array Disk System Hardware error" },
	{ 0xFFFFFFFF      ,	"" },
};

/* FC-GW : SIM,SNMP Report MAP */
struct errsim_t {
	ushort	err_no;
	uchar	sim_report;
	uchar	level;										/* FCLNX-0405 */
	
	uchar	snmp_report;
	uchar	snmp_trap;									/* FCLNX-0405 */
	
	uchar	rsv[2];										/* FCLNX-0405 */
};

const struct errsim_t errsim_map[] = {					/* FCLNX-0405 */
    /* err_no,SIM  ,level,SNMP, trap   */
    
	{ 0x0005, TRUE , 5 , FALSE, FALSE },
	{ 0x0008, TRUE , 5 , FALSE, FALSE },
	{ 0x000B, FALSE, 4 , FALSE, FALSE },
	{ 0x000C, TRUE , 2 , FALSE, FALSE },
	{ 0x000D, TRUE , 2 , FALSE, FALSE },
	{ 0x000E, TRUE , 2 , TRUE , TRUE  },
	                                    
	{ 0x0014, TRUE , 2 , TRUE , TRUE  },
	{ 0x0015, FALSE, 4 , FALSE, FALSE },
	{ 0x0016, FALSE, 4 , TRUE , TRUE  },
	{ 0x0017, FALSE, 4 , TRUE , TRUE  },
	{ 0x0018, FALSE, 4 , FALSE, FALSE },
	{ 0x001C, FALSE, 4 , FALSE, FALSE },
	{ 0x001D, TRUE , 4 , FALSE, FALSE },
	                                    
	{ 0x0020, TRUE , 2 , FALSE, FALSE },
	{ 0x0021, TRUE , 2 , FALSE, FALSE },
	{ 0x0022, TRUE , 2 , FALSE, FALSE },
	{ 0x0024, TRUE , 2 , FALSE, FALSE },
	{ 0x0026, TRUE , 2 , FALSE, FALSE },
	{ 0x002A, TRUE , 4 , FALSE, FALSE },
	{ 0x002B, TRUE , 6 , FALSE, FALSE },
	{ 0x002C, TRUE , 6 , FALSE, FALSE },
	{ 0x002D, TRUE , 6 , FALSE, FALSE },
	                                    
	{ 0x0031, TRUE , 6 , FALSE, FALSE },
	{ 0x0032, TRUE , 1 , FALSE, FALSE },
	{ 0x0033, TRUE , 1 , FALSE, FALSE },
	{ 0x0034, TRUE , 1 , FALSE, FALSE },
	{ 0x0035, TRUE , 1 , FALSE, FALSE },
	{ 0x0036, TRUE , 1 , FALSE, FALSE },
	{ 0x0037, TRUE , 5 , FALSE, FALSE },
	{ 0x0038, TRUE , 5 , FALSE, FALSE },
	{ 0x0039, TRUE , 5 , FALSE, FALSE },
	{ 0x003A, TRUE , 5 , FALSE, FALSE },
	{ 0x003B, TRUE , 5 , FALSE, FALSE },
	{ 0x003C, TRUE , 5 , FALSE, FALSE },
	{ 0x003D, TRUE , 5 , FALSE, FALSE },
	{ 0x003E, TRUE , 5 , FALSE, FALSE },
	                                    
	{ 0x004F, TRUE , 5 , FALSE, FALSE },
	                                    
	{ 0x0051, FALSE, 4 , FALSE, FALSE },
	{ 0x0052, TRUE , 2 , FALSE, FALSE },
	{ 0x0055, TRUE , 5 , FALSE, FALSE },
	{ 0x0056, TRUE , 5 , FALSE, FALSE },
	{ 0x0058, FALSE, 4 , TRUE , FALSE },
	{ 0x0059, TRUE , 2 , FALSE, FALSE },
	{ 0x005A, TRUE , 5 , FALSE, FALSE },
	{ 0x005B, TRUE , 5 , FALSE, FALSE },
	{ 0x005E, FALSE, 0 , TRUE , FALSE },
	{ 0x005F, FALSE, 0 , TRUE , FALSE },
	                                    
	{ 0x0060, FALSE, 0 , TRUE , FALSE },
	{ 0x0061, FALSE, 0 , TRUE , FALSE },
	{ 0x0062, FALSE, 0 , TRUE , FALSE },
	{ 0x0063, FALSE, 0 , TRUE , FALSE },
	{ 0x0064, FALSE, 0 , TRUE , FALSE },
	{ 0x0065, FALSE, 0 , FALSE, FALSE },
	{ 0x0066, FALSE, 0 , FALSE, FALSE },
	{ 0x0067, FALSE, 0 , FALSE, FALSE },
	{ 0x0068, FALSE, 5 , FALSE, FALSE },
	                                    
	{ 0x0071, TRUE , 1 , FALSE, FALSE },
	{ 0x0072, TRUE , 1 , FALSE, FALSE },
	{ 0x0073, TRUE , 1 , FALSE, FALSE },
	{ 0x0074, TRUE , 1 , FALSE, FALSE },
	{ 0x0075, TRUE , 1 , FALSE, FALSE },
	{ 0x0077, FALSE, 0 , FALSE, FALSE },
	{ 0x0078, FALSE, 0 , FALSE, FALSE },
	{ 0x007B, TRUE , 2 , FALSE, FALSE },
	{ 0x007C, TRUE , 2 , FALSE, FALSE },
	{ 0x007D, TRUE , 2 , FALSE, FALSE },
	{ 0x007E, TRUE , 2 , FALSE, FALSE },
	{ 0x007F, TRUE , 2 , FALSE, FALSE },
	                                    
	{ 0x0080, FALSE, 4 , FALSE, FALSE },
	{ 0x0081, TRUE , 2 , FALSE, FALSE },
	{ 0x0082, TRUE , 2 , FALSE, FALSE },
	{ 0x0083, TRUE , 2 , TRUE , TRUE  },
	{ 0x0084, TRUE , 2 , FALSE, FALSE },
	{ 0x0085, TRUE , 2 , FALSE, FALSE },
	{ 0x0086, TRUE , 2 , FALSE, FALSE },
	{ 0x0088, TRUE , 2 , FALSE, FALSE },
	{ 0x0089, TRUE , 2 , FALSE, FALSE },
	{ 0x008A, TRUE , 2 , FALSE, FALSE },
	{ 0x008D, TRUE , 4 , FALSE, FALSE },
	                                    
	{ 0x0098, TRUE , 5 , FALSE, FALSE },
	{ 0x0099, TRUE , 5 , FALSE, FALSE },
	{ 0x009A, TRUE , 5 , FALSE, FALSE },
	{ 0x009B, TRUE , 5 , FALSE, FALSE },
	{ 0x009C, TRUE , 8 , FALSE, FALSE },
	{ 0x009D, TRUE , 6 , FALSE, FALSE },
	{ 0x009E, TRUE , 9 , FALSE, FALSE },
	{ 0x009F, TRUE , 10, FALSE, FALSE },
	                                    
	{ 0x00A0, FALSE, 4 , FALSE, FALSE },
	                                    
	{ 0x00F0, FALSE, 2 , FALSE, FALSE },
	{ 0xFFFF, FALSE, 0 , FALSE, FALSE }	/* END */
}; 
#endif /* _HFC_FCGW */							/* FC-GW */

static void hfc_errsave( struct adap_info *,
				  int,	
				  uint,		
				  struct hfc_err_rec *,
				  uint,
				  uint );
static uint hfc_raslog( struct adap_info *,
				  struct hfc_err_rec *,
				  char	 *,
				  char	 *,
				  char	 *,
				  struct hfc_pkt		*hfcp,
				  struct scsi_cmnd		*cmnd,
				  uchar);
void hfc_hand2_trace(
	uchar               id,
	uchar               sub_id,
	struct adap_info    *ap,
	struct target_info  *target,
	struct hfc_pkt		*hfcp,
	uint64_t            etc1,
	uint64_t            etc2,
	uint64_t            etc3);
static void hfc_timeout_by_reset(
	struct adap_info        *ap,
	struct target_info      *target,
	struct hfc_pkt     		*hfcp);

static void hfc_timeout_by_scnlinkup(
	struct adap_info        *ap,
	struct target_info       *target);

static void hfc_timeout_by_restart(		/* FCLNX-0500 */
	struct adap_info        *ap,
	struct target_info 		*target,
	struct hfc_pkt			*hfcp);

				  
static uchar	logdata[16] ;

/*
 * Function:    hfc_hwerr_int
 *
 * Purpose:     Process Hardware error
 *
 * Arguments:   
 *  ap          - 
 *  int_a_reg   - 
 *  status_reg0 - 
 *  status_reg1 - 
 *  detail_reg  - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_hwerr_int(struct adap_info *ap,
					uint int_a_reg,
					uint status_reg0,
					uint status_reg1,
					uint detail_reg)
{
	uchar		  abend_code=0 ;
	uint64_t      status_reg;

	status_reg = ((uint64_t)status_reg0 << 32) & 0xffffffff00000000ULL;
	status_reg |= (uint64_t)status_reg1 & 0x00000000ffffffffULL;
	
	/* Align detail_reg with FPP format if package type is FIVE */
	if( ap->pkg.type != HFC_PKTYPE_FPP)                                 /* FCWIN-0240 */
		detail_reg <<= 8;

	HFC_DBGPRT(" hfcldd%d : hfc_hwerr_int \n",ap->dev_minor);

	hfc_hand2_trace(HFC_TRC_HWERR, 0x00, ap, NULL, NULL, 
			(uint64_t)int_a_reg, status_reg, (uint64_t)detail_reg);

	if( int_a_reg & HFC_HWERR_MCK ) {
		if( test_bit( HFC_WAIT_T3, (ulong *)&ap->status ) ) /* FCLNX-0269 */ /* FCLNX-GPL-061,083 */
		{
			abend_code = HFC_ABEND_T3 ;
			clear_bit( HFC_WAIT_T3, (ulong *)&ap->status );
		}
		else if( status_reg0 & HFC_HWERR_MPCHK )
		{
			abend_code = HFC_ABEND_MPCHK ;
		}
		else
		{
			abend_code = HFC_ABEND_MCK_INT ;
		}
	}
	else if( int_a_reg & HFC_HWERR_PCI )
	{
		if( int_a_reg & HFC_HWERR_SERR )
			abend_code = HFC_ABEND_SERR ;
		if( int_a_reg & HFC_HWERR_PERR )
			abend_code = HFC_ABEND_PERR ;
		if( int_a_reg & HFC_HWERR_SPERR )
			abend_code = HFC_ABEND_SPERR ;
	}
	hfc_abend(ap,abend_code);

	HFC_DBGPRT(" hfcldd : hfc_hwerr_int - end\n");
	hfc_hand2_trace(HFC_TRC_HWERR, 0x10, ap, NULL, NULL, 0, 0, 0);

}


/*
 * Function:    hfc_errlog
 *
 * Purpose:     Collect error log data 
 *
 * Arguments:   
 *  ap         - Pointer to adap_info 
 *  target     - Pointer to target_info 
 *  hfcp       - Pointer to hfc_pkt 
 *  type       - Error log type
 *  err_id     - Error name
 *  err_num    - error_no (Error log)
 *  data       - Output data format
 *  data_len   - Output data length
 *
 * Returns:     
 *
 * context:     Interruption level/process level
 *
 * Notes:       
 */
void hfc_errlog( struct adap_info *ap,
				struct target_info	*target,
				struct hfc_pkt		*hfcp,
				uchar				type,
				uint				err_id,
				uint				err_num,
				uchar				*data,
				ushort				data_len )
{
	struct mp_adap_info     *mpap=NULL;
	struct hfc_err_rec		*err_rec=NULL;	/* FCLNX_GPL-0151 	*//* FCLNX_GPL-0225 */
	uchar	ssn = 0;						/* slog page No 	*/
	uchar	son = 0;						/* slog offset No	*/
	ushort	sbc = 0;						/* slog length		*/
	ushort	sbc_wk = 0;	
	uint	slog_in_page=0;
	uint	slog_adr;
	uint	errlog_size;
	Typ_dderrlog	*dderrlog;
	uint	trc_num;
	int 	errlog_info_p=0;	
	int		lun=0, page = 1;
	uint	i;
	ushort	slog_len;       
	ushort	slog_num;
	ushort	wk_slog_len;
	ushort	wk_slog_num;   
	uint	scsi_id;
	ushort	we_que_cnt, xob_no;
	uchar	dd_log=FALSE;
	char	wkbuf[VPD_PN_LEN+13];  /* +13=NULL */	/* FCLNX-0337 *//* FCLNX-0368 */
	char	ras_error_id1[9], ras_error_id2[9];
	char	resource_name[16] ;
	uint	raslog=2;
	struct	scsi_cmnd *cmnd=NULL;
	char	sfp_type_name[HFC_SFP_TYPE_NAME_LEN+1];	/* FCLNX-GPL-526 */
	char	sfp_serial_no[HFC_SFP_SERIAL_NO_LEN+1];	/* FCLNX-GPL-526 */
	char	sfp_date_code[HFC_SFP_DATE_CODE_LEN+1];	/* FCLNX-GPL-526 */

	HFC_DBGPRT(" hfcldd%d : hfc_errlog - start\n",ap->dev_minor);

	errlog_info_p = 0;
	while( errlog_info[errlog_info_p].type != 0xFFFFFFFF ){
		if( errlog_info[errlog_info_p].type == err_id ){
			break;
		}
		errlog_info_p++;
	}


	memset(ras_error_id1,0,sizeof(ras_error_id1));
	if( err_num != 0x7f ){
		sprintf(&ras_error_id1[strlen(ras_error_id1)], "KALB%c%c%02X",
			hraslog_info[errlog_info_p].alart1[0], hraslog_info[errlog_info_p].alart1[1], err_num);
	}
	else{
		sprintf(&ras_error_id1[strlen(ras_error_id1)], "KALB%c%c%02X",
			hraslog_info[errlog_info_p].alart2[0], hraslog_info[errlog_info_p].alart2[1], err_num);
	}

	memset(ras_error_id2,0,sizeof(ras_error_id2));
	sprintf(&ras_error_id2[strlen(ras_error_id2)], "KALB%c%c%02X",
		hraslog_info[errlog_info_p].alart2[0], hraslog_info[errlog_info_p].alart2[1], err_num);

	if( hfcp != NULL ){
		cmnd = hfcp->cmd_pkt;
	}

	memset(resource_name,0,sizeof(resource_name));
	if( ap != NULL){
		sprintf(&resource_name[strlen(resource_name)], "hfcldd%d", ap->dev_minor);
	}
	else if( ap == NULL){
		sprintf(&resource_name[strlen(resource_name)], "hfcldd");
	}

	if(	( ap == NULL )               ||					/* FCLNX_GPL-0225 */
		 (ap->mp_adap_info == NULL ) || 
		 (ap->pci_cfginf == NULL )   ||
		 (ap->mem_base_addr == 0) 		){
		
		HFC_ERRPRT("hfcldd : %s (ErrNo:0x%02x) \n",errlog_info[errlog_info_p].errmsg, err_num); 
		if( ( data != NULL ) && ( data_len != 0 ) ){
			if( data_len > 16 )
				data_len = 16;
			HFC_ERRPRT("      [%02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x]\n",
				data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7],
				data[8], data[9], data[10], data[11], data[12], data[13], data[14], data[15]);	
		}
		return;
	}													/* FCLNX-GPL-225 */

	err_rec = &ap->err_rec;

	if( type == HFC_ERRLOG_TYPE_RASLOG_EXIST ){		/* raslog output	*/
		raslog = hfc_raslog( ap, err_rec, (char *)&ras_error_id1,
					 (char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
	}
	else{
		switch( type ){
			case HFC_ERRLOG_TYPE_IODONE_WARN1 :
			case HFC_ERRLOG_TYPE_IODONE_WARN2 :
			case HFC_ERRLOG_TYPE_IODONE_WARN3 :
			case HFC_ERRLOG_TYPE_IODONE_WARN4 :
			case HFC_ERRLOG_TYPE_IODONE_WARN5 :
				if( !ap->raslog_install ){
					raslog = hfc_raslog( ap, err_rec, (char *)&ras_error_id2,
					 	(char *)&ras_error_id2,(char *)&resource_name, hfcp,cmnd,type);
				}
				break;
			default :
				break;
		}
	}

	if( raslog == 0 ) return;
	else if( ( raslog == 2 ) || ( raslog == 1) ){		/* syslog output	*/
		if( ap == NULL ){
			HFC_ERRPRT("hfcldd : %s (ErrNo:0x%02x) \n",errlog_info[errlog_info_p].errmsg, err_num); 
			return;
		}
		else if( ( ap->raslog_install ) && ( data_len == 0) && ( err_num != 0x7f ) ){
			HFC_ERRPRT("hfcldd%d: %s (ErrNo:0x%02x) \n",ap->dev_minor, errlog_info[errlog_info_p].errmsg, err_num); 
			if( err_num == 0x93 ){  	/* write command failed */
				if( ( hfcp != NULL ) && ( cmnd != NULL ) ){
					if( type == HFC_ERRLOG_TYPE_IODONE_WARN1 ){
						HFC_INFPRT(" cdb=0x%02x allowed =%d for host%d:channel%d:id%d:lun%d.(instance%d)\n",
														cmnd->cmnd[0],cmnd->allowed,
														CMND_HOST_NO(cmnd),
														CMND_CHANNEL(cmnd),
														CMND_TARGET(cmnd),
														CMND_LUN(cmnd),
														ap->instance);
					}
					else if( type == HFC_ERRLOG_TYPE_IODONE_WARN2 ){
						HFC_WRNPRT("hfcldd%d: WARNING - Incomplete data write command. \n",ap->dev_minor);
						HFC_WRNPRT(" cdb=0x%02x for host%d:channel%d:id%d:lun%d.(instance%d)\n",
														cmnd->cmnd[0],
														CMND_HOST_NO(cmnd),
														CMND_CHANNEL(cmnd),
														CMND_TARGET(cmnd),
														CMND_LUN(cmnd),
														ap->instance);
					}
					else if( type == HFC_ERRLOG_TYPE_IODONE_WARN3 ){
						HFC_WRNPRT(" retries:-1 allowed:-1 result=0x%08x adap_status=0x%08x. \n",
														cmnd->result,
														hfcp->adap_status);
					}
					else if( type == HFC_ERRLOG_TYPE_IODONE_WARN4 ){
						HFC_WRNPRT(" retries:%d allowed:%d  result=0x%08x  adap_status=0x%08x. \n",
														hfcp->fail,
														cmnd->allowed,
														cmnd->result,
														hfcp->adap_status);
					}
					else if( type == HFC_ERRLOG_TYPE_IODONE_WARN5 ){
						HFC_WRNPRT(" retries:%d allowed:%d  result=0x%08x adap_status=0x%08x. \n",
														cmnd->retries,
														cmnd->allowed,
														cmnd->result,
														hfcp->adap_status);
					}
				}
			}
			return;
		}
		if( ( ap->raslog_install ) && ( err_num == 0x18 ) ){		/* FCLNX-486	*/
			HFC_INFPRT("hfcldd%d: %s (ErrNo:0x%02x)\n",
			 	ap->dev_minor, errlog_info[errlog_info_p].errmsg, err_num );	
			return;
		}
	}

	if ( hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info && (hfcp != NULL) ) {
		if (hfc_manage_info.npubp->hfc_errlog_mp(hfcp)) return;
	}
	
	HFC_BZERO( (char*)err_rec, sizeof( struct hfc_err_rec ) );
	errlog_size = sizeof( struct hfc_err_head );

	ap->raslog_cnt = 0;
	if( ap->raslog_install ){
		HFC_ERRPRT("hfcldd%d: Firmware version %06x, Driver version %s, device %02x:%02x.%02x IRQ %d\n",
			ap->dev_minor,ap->mp_adap_info->sys_rev, hfc_manage_info.package_ver,
			ap->pci_cfginf->bus->number,
			PCI_SLOT(ap->pci_cfginf->devfn),
			PCI_FUNC(ap->pci_cfginf->devfn),
			ap->irq);

		HFC_ERRPRT("hfcldd%d: Adapter wwpn : %llx \n", ap->dev_minor,(unsigned long long)ap->ww_name);
	}
	memset(wkbuf,0,sizeof(wkbuf));									/* FCLNX-0337 *//* FCLNX-0368 */

	if(ap->pkg.type == HFC_PKTYPE_FPP)	/* FPP */
		memcpy(wkbuf, ((struct hfc_vpd *)(ap->mp_adap_info->vpd_buf))->pn_value, VPD_PN_LEN);				/* FCLNX-0337 *//* FCLNX-0368 */
	else								/* FIVE */
		memcpy(wkbuf, ((struct hfc_vpd_five *)(ap->mp_adap_info->vpd_buf))->pn_value, VPD_PN_LEN);				/* FCLNX-0337 *//* FCLNX-0368 */

	if( ap->raslog_install )
		HFC_ERRPRT("hfcldd%d: Parts number : %11s\n", ap->dev_minor, wkbuf);
	
	/* Output SFP Information *//* FCLNX-GPL-526 */
	
	
	if( ap->raslog_install ){
		if((ap->fw_support & 0x80)||(ap->fw_init_p->func & HFC_FWF_SFPINF)){		/* FCLNX-GPL-530 */
			if(!(ap->fw_init_p->sfp_info.sfp_status & HFC_SFP_INSTALL)){
				HFC_ERRPRT("hfcldd%d: SFP Information : N/A\n", ap->dev_minor);
			}else {
				memset(sfp_type_name,0,sizeof(sfp_type_name));
				memcpy(sfp_type_name, ap->fw_init_p->sfp_info.sfp_type_name, sizeof(ap->fw_init_p->sfp_info.sfp_type_name)); 
				hfc_delete_space(sfp_type_name);
				
				memset(sfp_serial_no,0,sizeof(sfp_serial_no));
				memcpy(sfp_serial_no, ap->fw_init_p->sfp_info.sfp_serial_no, sizeof(ap->fw_init_p->sfp_info.sfp_serial_no)); 
				hfc_delete_space(sfp_serial_no);

				memset(sfp_date_code,0,sizeof(sfp_date_code));
				memcpy(sfp_date_code, ap->fw_init_p->sfp_info.sfp_date_code, sizeof(ap->fw_init_p->sfp_info.sfp_date_code)); 
				hfc_delete_space(sfp_date_code);

				if(!(ap->fw_init_p->sfp_info.sfp_status & HFC_SFP_VALID)){
					HFC_ERRPRT("hfcldd%d: SFP Information : incorrect data(%s, %s, %s)\n",
						ap->dev_minor,
						sfp_type_name,
						sfp_serial_no,
						sfp_date_code);
				}else{
					HFC_ERRPRT("hfcldd%d: SFP Information : %s, %s, %s\n",
						ap->dev_minor,
						sfp_type_name,
						sfp_serial_no,
						sfp_date_code);
				}
			}
		}
	}/* FCLNX-GPL-526 */
	
	HFC_4L_TO_4B(err_rec->head.err_num, err_num);
	
	if( ap != NULL ){
		err_rec->head.con_type = ap->connect_type;
		err_rec->head.mb_status = ap->mb_status;
		err_rec->head.open_status = ap->open_status;
		err_rec->head.wait_exec_cnt = ap->xob_wait_exec_cnt;
		if( type == HFC_ERRLOG_TYPE_XRB )
			HFC_2L_TO_2B(err_rec->head.xob_xrb_no, ap->xrb_no);
		if( type != HFC_ERRLOG_TYPE_XRB )
			HFC_2L_TO_2B(err_rec->head.xob_xrb_no, ap->xob_no);
		HFC_4L_TO_4B(err_rec->head.adap_status, ap->status);
		scsi_id = (uint)ap->scsi_id;
		HFC_4L_TO_4B(err_rec->head.a_scsi_id, scsi_id);
		err_rec->head.port_cnt = (uchar)ap->pkg.port;
		err_rec->head.pkg_type = (uchar)ap->pkg.type;
		err_rec->head.port_no = (uchar)ap->port_no;

		mpap = ap->mp_adap_info;
		if( mpap != NULL)
		{
			err_rec->head.mp_adap_status = mpap->status;
			HFC_4L_TO_4B(err_rec->head.mp_adap_lock, mpap->lock);
		}
	}

	if( target != NULL ){
		HFC_4L_TO_4B(err_rec->head.target_status, target->status);
		err_rec->head.target_flags = target->flags;
		err_rec->head.target_id = (uchar)target->target_id;
		scsi_id = (uint)target->scsi_id;
		HFC_4L_TO_4B(err_rec->head.d_scsi_id, scsi_id);
		we_que_cnt = (ushort)target->we_que_cnt;
		HFC_2L_TO_2B(err_rec->head.we_que_cnt, we_que_cnt);
	}

	if( hfcp != NULL )
	{
		xob_no = (ushort)hfcp->cmd_xob;
		HFC_2L_TO_2B(err_rec->head.xob_xrb_no, xob_no);
		if(hfcp->cmd_pkt != NULL)
		{
			err_rec->head.resv[0] = (uchar)(hfcp->cmd_pkt->timeout_per_command/HZ);
			err_rec->head.resv[1] = hfcp->cmd_pkt->cmnd[0];

			err_rec->head.tmo_sec   = (uchar)(hfcp->cmd_pkt->timeout_per_command/HZ);		/* FCLNX-0429 */
			err_rec->head.cdb       = hfcp->cmd_pkt->cmnd[0];								/* FCLNX-0429 */
			
			err_rec->head.tmo_hz[0] = (uchar)(hfcp->cmd_pkt->timeout_per_command >> 24);		/* FCLNX-0429 */
			err_rec->head.tmo_hz[1] = (uchar)(hfcp->cmd_pkt->timeout_per_command >> 16);		/* FCLNX-0429 */
			err_rec->head.tmo_hz[2] = (uchar)(hfcp->cmd_pkt->timeout_per_command >> 8);		/* FCLNX-0429 */
			err_rec->head.tmo_hz[3] = (uchar)(hfcp->cmd_pkt->timeout_per_command);			/* FCLNX-0429 */
		}
		
		if(target != NULL)
			err_rec->head.lun_status = (uchar)target->lustat[lun];

		err_rec->head.lun = hfcp->lun_id;
	}

	if( ( data != NULL ) && ( data_len != 0 ) ){
		if( data_len > 16 )
			data_len = 16;
		HFC_MEMCPY( err_rec->hfc_drv_log, data, data_len );
	}

	errlog_size += 16;	
	if( ( ap != NULL ) &&
		( ( type == HFC_ERRLOG_TYPE_MCK ) ||
		 (type == HFC_ERRLOG_TYPE_CHKSTP) ||
		 (type == HFC_ERRLOG_TYPE_IMLLOG)) ){
			if(mpap != NULL)
			{
				if ( ap->pkg.type == HFC_PKTYPE_FPP )
				{
					HFC_MEMCPY( err_rec->hfc_detail_log, (uchar *)mpap->hw_log, 960 );
					errlog_size += 960;
					dd_log = TRUE ;
				}
				else
				{
					HFC_MEMCPY( err_rec->hfc_detail_log, (uchar *)&mpap->hw_log[16], 960 );
					errlog_size += 960;
					dd_log = TRUE ;
					
					/* Shadow LPAR driver save driver log for Guest LPAR driver.*/
					if ( HFC_MMODE_CHECK_SHADOW(ap) )
						HFC_MEMCPY(ap->mlpf_drv_log, err_rec, 64 );
				}
			}
	} else {
		if( ap != NULL ){
			if( type == HFC_ERRLOG_TYPE_XRB ){
				if( ap->xrb[ap->xrb_no].esw & HFC_ESW_MEINT_REPO ){
					ssn = ap->xrb[ap->xrb_no].ssn;
					son = ap->xrb[ap->xrb_no].son;
					HFC_2B_TO_2L(sbc, ap->xrb[ap->xrb_no].sbc);
				}
			}
			if( type == HFC_ERRLOG_TYPE_MBRESP ){
				if( ap->mb->mb_resp.esw & HFC_ESW_MEINT_REPO ){
					ssn = ap->mb->mb_resp.ssn;
					son = ap->mb->mb_resp.son;
					HFC_2B_TO_2L(sbc, ap->mb->mb_resp.sbc);
				}
			}
			if( type == HFC_ERRLOG_TYPE_MBINT ){
				if( ap->mb->mb_intreq.type.fwintreq0.esw & HFC_ESW_MEINT_REPO ){
					ssn = ap->mb->mb_intreq.type.fwintreq0.ssn;
					son = ap->mb->mb_intreq.type.fwintreq0.son;
					HFC_2B_TO_2L(sbc, ap->mb->mb_intreq.type.fwintreq0.sbc);
				}
			}
			if( type == HFC_ERRLOG_TYPE_TOUTLOG ){	
				if( hfcp != NULL ){		
					ssn = hfcp->tout_slog_ssn;	
					son = hfcp->tout_slog_son;	
					sbc = hfcp->tout_slog_sbc;
				}	
			}
			if( sbc != 0 ){
				wk_slog_len = ap->fw_init_p->slog_len;
				wk_slog_num = ap->fw_init_p->slog_num;
				HFC_2B_TO_2L(slog_len, wk_slog_len);
				HFC_2B_TO_2L(slog_num, wk_slog_num);
				slog_in_page = HFC_PAGE_SIZE / (uint)slog_len ;
				if( (son < slog_in_page) && (ssn < slog_num) )
				{
					slog_adr = (HFC_PAGE_SIZE * ssn)+(slog_len * son) ; 
					sbc_wk = sbc ;	
					if( sbc_wk > 960 )
						sbc_wk = 960;

					errlog_size += sbc_wk;
					HFC_MEMCPY( err_rec->hfc_detail_log, (uchar *)(ap->slog_addr[ssn]+(slog_len * son)), sbc_wk );
					if( (err_id != ERRID_HFCP_EVNT1) && (err_id != ERRID_HFCP_EVNT2) &&
						(err_id != ERRID_HFCP_EVNT3) && (err_id != ERRID_HFCP_EVNT4) )
					{
						dd_log = TRUE ;
					}
				}
			}
			else	/* 1.102 */
			{
				/*--- Non MCKLOG / SOFTLOG ---*/		
				dderrlog = (Typ_dderrlog *)err_rec->hfc_detail_log ;
				trc_num = ap->trc_num ;
				
				for(i=0 ; i< ( HFC_TRC_HEAD_CNT + HFC_TRC_ID_CNT ) ; i++)                       /*  FCWIN-160    */
				{
					if(i < HFC_TRC_HEAD_CNT)
					{
						HFC_MEMCPY( (uchar *)&dderrlog->trace_head[i][0],&ap->trc_ptr[trc_num].trc_id, 32);
					}
					else
					{
						dderrlog->trace_id[i-HFC_TRC_HEAD_CNT][0] = ap->trc_ptr[trc_num].trc_id ;
						dderrlog->trace_id[i-HFC_TRC_HEAD_CNT][1] = ap->trc_ptr[trc_num].trc_data[0] ;
					}
					if( trc_num == 0 )
						trc_num = ap->trc_max - 1 ;
					else
						trc_num-- ;
				}
				errlog_size +=960 ;
				
			}
		}
	}

	if( !ap->raslog_install ){
		raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id1,
					 (char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
	}
	if( ( ap->raslog_install ) || (raslog != 0) ) {
		hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
		errlog_info_p = 17;			/* FCLNX-0494	*/
		page++;	
	}

	if ( ap->pkg.type == HFC_PKTYPE_FPP )                                               /* FCWIN-0220 */
	{
		if( ( type == HFC_ERRLOG_TYPE_IMLLOG ) &&
			!(test_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status) ) ){
			if(mpap != NULL)
				HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)&mpap->hw_log[240], 1024 ); /* FCLNX-GPL-210 */
			if( !ap->raslog_install ){
				raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
						(char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
			}
			if( ( ap->raslog_install ) || (raslog != 0) ) {
				hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
				page++;	
			}
			if(mpap != NULL)																
				HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)&mpap->hw_log[496], 1024 ); /* FCLNX-GPL-210 */
			if( !ap->raslog_install ){
				raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
						(char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
			}
			if( ( ap->raslog_install ) || (raslog != 0) ) {
				hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
				page++;	
			}
		} 
		else if( (sbc > 1024) &&	
			 !(test_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status) ) ){
				wk_slog_len = ap->fw_init_p->slog_len;
				wk_slog_num = ap->fw_init_p->slog_num;
				HFC_2B_TO_2L(slog_len, wk_slog_len);
				HFC_2B_TO_2L(slog_num, wk_slog_num);
				son++;
				if( son == slog_in_page ){
					son = 0;
					ssn++;
					if( ssn == slog_num )
						ssn = 0;
				}
				if( ( son < slog_in_page ) &&
					 ( ssn < slog_num ) ){
					slog_adr =
						 ( HFC_PAGE_SIZE * ssn ) + ( slog_len * son );
					HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)(ap->slog_addr[ssn]+(slog_len * son)), 1024 ); /* FCLNX-GPL-210 */
				    if( !ap->raslog_install ){
						raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
							 (char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
					}
					if( ( ap->raslog_install ) || (raslog != 0) ) {
						hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
						page++;	
					}
				}
		}
	}	
	else
	{
		if( ( type == HFC_ERRLOG_TYPE_IMLLOG ) &&
			!(test_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status) ) ){
			if(mpap != NULL)
				HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)&mpap->hw_log[256], 1024 ); /* FCLNX-GPL-210 */
			if( !ap->raslog_install ){
				raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
							 (char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
			}
			if( ( ap->raslog_install ) || (raslog != 0) ) {
				hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
				page++;	
			}
			if(mpap != NULL)															
				HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)&mpap->hw_log[512], 1024 );
			if( !ap->raslog_install ){
				raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
							 (char *)&ras_error_id2,(char *)resource_name, NULL,NULL,type);
			}
			if( ( ap->raslog_install ) || (raslog != 0) ) {
				hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
				page++;	
			}
			if(mpap != NULL)
				HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)&mpap->hw_log[768], 1024 );
			if( !ap->raslog_install ){
				raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
							 (char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
			}
			if( ( ap->raslog_install ) || (raslog != 0) ) {
				hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
				page++;	
			}
		} 
		if ((type == HFC_ERRLOG_TYPE_MCK) ||
		 (type == HFC_ERRLOG_TYPE_CHKSTP))
		{
			if(mpap != NULL)
				HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)&mpap->hw_log[256], 1024 );
			if( !ap->raslog_install ){
				raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
							 (char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
			}
			if( ( ap->raslog_install ) || (raslog != 0) ) {
				hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
				page++;	
			}
		}
		else if( (sbc > 1024) &&	
			 !(test_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status) ) ){
			wk_slog_len = ap->fw_init_p->slog_len;
			wk_slog_num = ap->fw_init_p->slog_num;
			HFC_2B_TO_2L(slog_len, wk_slog_len);
			HFC_2B_TO_2L(slog_num, wk_slog_num);
			son++;
			if( son == slog_in_page ){
				son = 0;
				ssn++;
				if( ssn == slog_num )
					ssn = 0;
			}
			if( ( son < slog_in_page ) &&
				 ( ssn < slog_num ) ){
				slog_adr =
					 ( HFC_PAGE_SIZE * ssn ) + ( slog_len * son );
				HFC_MEMCPY( (uchar *)&err_rec->head.err_num, (uchar *)(ap->slog_addr[ssn]+(slog_len * son)), 1024 );
			    if( !ap->raslog_install ){
					memset(ras_error_id2,0,sizeof(ras_error_id2));
					sprintf(&ras_error_id2[strlen(ras_error_id2)], "KALB%c%c7F",
							hraslog_info[errlog_info_p].alart2[0], hraslog_info[errlog_info_p].alart2[1]);
					raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
								 (char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
				}
				if( ( ap->raslog_install ) || (raslog != 0) ) {
					hfc_errsave( ap, errlog_info_p, err_num, err_rec, errlog_size, 0 );	
					page++;	
				}
			}
		}
	}


	if( (dd_log == TRUE) && (err_num != 0x00000014) )
	{
		HFC_BZERO((char*)err_rec,sizeof(struct hfc_err_rec)) ;
		errlog_size = sizeof(struct hfc_err_head) ;
		err_rec->head.err_num = 0xf0 ;
		errlog_size += 16 ;
		
		dderrlog = (Typ_dderrlog *)err_rec->hfc_detail_log ;

		trc_num = ap->trc_num ;

		for(i=0 ; i< ( HFC_TRC_HEAD_CNT + HFC_TRC_ID_CNT ) ; i++)                       /*  FCWIN-160    */
		{
			if(i < HFC_TRC_HEAD_CNT)
			{
				HFC_MEMCPY( (uchar *)&dderrlog->trace_head[i][0],&ap->trc_ptr[trc_num].trc_id, 32);
			}
			else
			{
				dderrlog->trace_id[i-HFC_TRC_HEAD_CNT][0] = ap->trc_ptr[trc_num].trc_id ;
				dderrlog->trace_id[i-HFC_TRC_HEAD_CNT][1] = ap->trc_ptr[trc_num].trc_data[0] ;
			}
			if( trc_num == 0 )
				trc_num = ap->trc_max - 1 ;
			else
				trc_num-- ;
		}
		errlog_size+=960 ;
		if( !ap->raslog_install ){
			memset(ras_error_id2,0,sizeof(ras_error_id2));
					sprintf(&ras_error_id2[strlen(ras_error_id2)], "KALB%c%cF0",
							hraslog_info[errlog_info_p].alart2[0], hraslog_info[errlog_info_p].alart2[1]);
			raslog = hfc_raslog( ap, (struct hfc_err_rec *)err_rec, (char *)&ras_error_id2,
							(char *)&ras_error_id2,(char *)&resource_name, NULL,NULL,type);
		}
		if( ( ap->raslog_install ) || (raslog != 0) ) {
			hfc_errsave( ap, errlog_info_p, 0xf0, err_rec, errlog_size, 0 );	
		}
	}
	
	if ( HFC_MMODE_CHECK_SHADOW(ap) )
	{
		if ( ( ap != NULL ) &&
		( ( type == HFC_ERRLOG_TYPE_MCK ) ||
		 (type == HFC_ERRLOG_TYPE_CHKSTP) ||
		 (type == HFC_ERRLOG_TYPE_IMLLOG) ) )
		{
			hfc_mlpf_errlog_slpar(ap);
		}
	}
	
	return;
}


/*
 * Function:    hfc_errsave
 *
 * Purpose:     Output error log data 
 *
 * Arguments:   
 *  ap            - Pointer to adap_info 
 *  errlog_info_p - Error name table number
 *  err_num       - Number of pages
 *  err_rec       - Pointer to hfc_err_rec 
 *  cnt           - Output data length
 *  offset        - Address offset value
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_errsave( struct adap_info *ap,
				  int errlog_info_p,	
				  uint err_num,		
				  struct hfc_err_rec *err_rec,
				  uint cnt,
				  uint offset)
{
	uint *err_info;
	int lp;
	int lp2, lp3;
	int int_cnt;
	uint amari;
//	char log_wk[470] ;							/* FCLNX_GPL-0151 */
	char *name;		
	char c_instance[9];	
	uint wlog0, wlog1, wlog2, wlog3;
	uint log0, log1, log2, log3;

	HFC_DBGPRT(" hfcldd : hfc_errsave - start\n");
	
	if( ap != NULL ){
		if( cnt > sizeof( struct hfc_err_rec ) ){	
			cnt = sizeof( struct hfc_err_rec );
		}

#ifdef _HFC_FCGW								/* FC-GW rev.4 */
		switch (err_num) {
			case 0x0c :	cnt = 0x40;	break;		/* failed LOGIN */
			case 0x0d :	cnt = 0x40;	break;
			case 0x0e :	cnt = 0x40;	break;
//			case 0x14 :	cnt = 0x40;	break;
//			case 0x15 :	cnt = 0x40;	break;
			case 0x16 :	cnt = 0x40;	break;		/* received PLOGI */
			case 0x17 :	cnt = 0x40;	break;		/* received LOGO */
			case 0x7b :	cnt = 0x40;	break;		/* failed GID_FT */
			case 0x7c :	cnt = 0x40;	break;
			case 0x7d :	cnt = 0x40;	break;
			case 0x81 :	cnt = 0x40;	break;		/* failed GID_PN */
			case 0x82 :	cnt = 0x40;	break;
			case 0x83 :	cnt = 0x40;	break;
			case 0x84 :	cnt = 0x40;	break;		/* failed GPN_ID */
			case 0x85 :	cnt = 0x40;	break;
			case 0x86 :	cnt = 0x40;	break;
		}
#endif /* _HFC_FCGW */							/* FC-GW rev.4 */

		/* Convert output data bytes in word-unit */
		int_cnt = cnt / 16;
		amari   = cnt % 16;
		if( amari > 12 ){
			int_cnt++;
			amari = 0;
		}

		err_info = (uint *)&err_rec->head.err_num;
		
		name = "hfcldd";	
		
		c_instance[0] = 0 ;
		sprintf( c_instance, "%x", ap->dev_minor );	
		HFC_BZERO( ap->log_wk, sizeof( ap->log_wk ) );

#ifdef _HFC_FCGW									/* FC-GW */
		if ( hfc_manage_info.hfcldd_mp_mod ) {
			hfc_manage_info.npubp->hfc_fcgw_simmsg(ap, name, errlog_info_p, err_num, 1,0,0,0);	/* FC-GW */
		}
#endif /* _HFC_FCGW */							/* FC-GW */

		HFC_ERRPRT("%s%d: %s (ErrNo:0x%02x)\n",
			 name, ap->dev_minor, errlog_info[errlog_info_p].errmsg, err_num );	

		lp2 = 0 ;
		lp3 = 0;

		for( lp = 0; lp < ( int_cnt * 4 ); ){
#if defined(__x86_64)  /* FCLNX-0629 */
			touch_nmi_watchdog();		/* FCLNX-0398 */
#endif	
			wlog0 = (uint)err_info[lp];
			wlog1 = (uint)err_info[lp+1];
			wlog2 = (uint)err_info[lp+2];
			wlog3 = (uint)err_info[lp+3];
			HFC_4L_TO_4B(log0, wlog0);
			HFC_4L_TO_4B(log1, wlog1);
			HFC_4L_TO_4B(log2, wlog2);
			HFC_4L_TO_4B(log3, wlog3);
			sprintf( (char *)&ap->log_wk[lp2],
				 "0x%04x:[ %08x %08x %08x %08x ]\n",
				 ( lp * 4 +  offset ),
				 log0, log1, log2, log3);

			lp += 4;
			lp2 = strlen( ap->log_wk );
			lp3++;
			if(lp3 == 10 )  /* Ten line output: 470 bytes */
			{
				switch( errlog_info[errlog_info_p].type ){
					case ERRID_HFCP_ERR1:
					case ERRID_HFCP_ERR2:
					case ERRID_HFCP_ERR3:
						HFC_ERRPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_ERR4:
						HFC_WRNPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_ERR5:
						HFC_ERRPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_ERR6:
						HFC_WRNPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_ERR7:
					case ERRID_HFCP_ERR8:
					case ERRID_HFCP_ERR9:
					case ERRID_HFCP_ERRA:
						HFC_WRNPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_ERRB:
						HFC_WRNPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_ERRC:
						HFC_WRNPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_ERRD:
					case ERRID_HFCP_ERRE:
					case ERRID_HFCP_ERRF:
						HFC_ERRPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_EVNT1:
					case ERRID_HFCP_EVNT2:
						HFC_WRNPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_EVNT3:
						HFC_WRNPRT("%s", ap->log_wk );	
						break;
					case ERRID_HFCP_EVNT4:
						HFC_WRNPRT("%s", ap->log_wk );
						break;
					default :
						HFC_INFPRT("%s", ap->log_wk );
						break;
				}
				lp2 = 0;
				lp3 = 0;
				HFC_BZERO( ap->log_wk, sizeof( ap->log_wk ) );	
			}
		}	

		switch( amari ){
			case 1 :
			case 2 :
			case 3 :
			case 4 :
				wlog0 = (uint)err_info[lp];
				HFC_4L_TO_4B(log0, wlog0);
				sprintf( (char *)&ap->log_wk[lp2],	
					 "0x%04x:[ %08x ] \n",
					 ( lp * 4 + offset ), wlog0 );
				lp++;
				lp2 = strlen( ap->log_wk );
				break;
			case 5 :
			case 6 :
			case 7 :
			case 8 :
				wlog0 = (uint)err_info[lp];
				wlog1 = (uint)err_info[lp+1];
				HFC_4L_TO_4B(log0, wlog0);
				HFC_4L_TO_4B(log1, wlog1);
				sprintf( (char *)&ap->log_wk[lp2],	
					 "0x%04x:[ %08x %08x ] \n",
					 ( lp * 4 + offset ),
					 wlog0, wlog1);
				lp += 2;
				lp2 = strlen( ap->log_wk );
				break;
			case 9 :
			case 10 :
			case 11 :
			case 12 :
				wlog0 = (uint)err_info[lp];
				wlog1 = (uint)err_info[lp+1];
				wlog2 = (uint)err_info[lp+2];
				HFC_4L_TO_4B(log0, wlog0);
				HFC_4L_TO_4B(log1, wlog1);
				HFC_4L_TO_4B(log2, wlog2);
				sprintf( (char *)&ap->log_wk[lp2],	
					 "0x%04x:[ %08x %08x %08x ] \n",
					 ( lp * 4 + offset ),
					 wlog0, wlog1, wlog2);
				lp += 3;
				lp2 = strlen( ap->log_wk );	
				break;
			default :
				break;
		}
		switch( errlog_info[errlog_info_p].type ){
			case ERRID_HFCP_ERR1:
			case ERRID_HFCP_ERR2:
			case ERRID_HFCP_ERR3:
				HFC_ERRPRT("%s", ap->log_wk );	
				HFC_ERRPRT("\n" );
				break;
			case ERRID_HFCP_ERR4:
				HFC_WRNPRT("%s", ap->log_wk );	
				HFC_WRNPRT("\n" );
				break;
			case ERRID_HFCP_ERR5:
				HFC_ERRPRT("%s", ap->log_wk );	
				HFC_ERRPRT("\n" );
				break;
			case ERRID_HFCP_ERR6:
				HFC_WRNPRT("%s", ap->log_wk );	
				HFC_WRNPRT("\n" );
				break;
			case ERRID_HFCP_ERR7:
			case ERRID_HFCP_ERR8:
			case ERRID_HFCP_ERR9:
			case ERRID_HFCP_ERRA:
				HFC_WRNPRT("%s", ap->log_wk );	
				HFC_WRNPRT("\n" );
				break;
			case ERRID_HFCP_ERRB:
				HFC_WRNPRT("%s", ap->log_wk );	
				HFC_WRNPRT("\n" );
				break;
			case ERRID_HFCP_ERRC:
				HFC_WRNPRT("%s", ap->log_wk );	
				HFC_WRNPRT("\n" );
				break;
			case ERRID_HFCP_ERRD:
			case ERRID_HFCP_ERRE:
			case ERRID_HFCP_ERRF:
				HFC_ERRPRT("%s", ap->log_wk );	
				HFC_ERRPRT("\n");
				break;
			case ERRID_HFCP_EVNT1:
			case ERRID_HFCP_EVNT2:
				HFC_WRNPRT("%s", ap->log_wk );	
				HFC_WRNPRT("\n" );
				break;
			case ERRID_HFCP_EVNT3:
				HFC_WRNPRT("%s", ap->log_wk );	
				HFC_WRNPRT("\n" );
				break;
			case ERRID_HFCP_EVNT4:
				HFC_WRNPRT("%s", ap->log_wk );
				HFC_WRNPRT("\n");
				break;
			default :
				HFC_INFPRT("%s", ap->log_wk );
				HFC_INFPRT("\n");
				break;
		}	
	}

	return;
}


/*
 * Function:    hfc_raslog
 *
 * Purpose:     Output error log data to raslog files
 *
 * Arguments:   
 *  ap            - Pointer to adap_info 
 *  err_rec       - Pointer to hfc_err_rec 
 *  ras_error_id  - Pointer to hraslog_errid
 *  uchar         - Pointer to resource_name
 *
 * Returns:     
 *
 * Notes:       
 */
uint hfc_raslog( struct adap_info *ap,
				  struct hfc_err_rec *err_rec,
				  char 	*ras_error_id1,
				  char 	*ras_error_id2,
				  char	 *resource_name,
				  struct hfc_pkt		*hfcp,
				  struct scsi_cmnd		*cmnd,
				  uchar	 type)
{
//	uchar detail_data[HFC_RASLOG_LEN];		/* FCLNX_GPL-0151 */
	uint raslog=2;
	char	sfp_type_name[HFC_SFP_TYPE_NAME_LEN+1];	/* FCLNX-GPL-526 */
	char	sfp_serial_no[HFC_SFP_SERIAL_NO_LEN+1];	/* FCLNX-GPL-526 */
	char	sfp_date_code[HFC_SFP_DATE_CODE_LEN+1];	/* FCLNX-GPL-526 */
#ifndef _HFC_NO_RASLOG
	uint raslog_retry;
	struct hfc_vpd          *vpd_info=NULL;
	struct hfc_vpd_five     *vpdf_info=NULL;
#endif
	HFC_DBGPRT(" hfcldd : hfc_raslog - start\n");

	memset(ap->detail_data, 0, sizeof(ap->detail_data));		/* FCLNX_GPL-0151 */
	if( ap == NULL ){
#ifndef _HFC_NO_RASLOG
		raslog = _hraslog((char *)ras_error_id1, (char *)resource_name, 0, (char *)ap->detail_data );
#endif
		return(raslog);
	}

#ifndef _HFC_NO_RASLOG
	memcpy( ap->detail_data, &err_rec->head.err_num, 512);
	sprintf( &ap->detail_data[HFC_MFC], "%s", "Hitachi");
	sprintf( &ap->detail_data[HFC_MODELNAME], "%s", ap->mp_adap_info->model_name);
	if(ap->pkg.type == HFC_PKTYPE_FPP){
		vpd_info = (struct hfc_vpd *)ap->mp_adap_info->vpd_buf;
		sprintf( &ap->detail_data[HFC_FWVERSION], "%06x", vpd_info->fw_ver);
		sprintf( &ap->detail_data[HFC_DRVVERSION], "%s", vpd_info->driver_ver);
		sprintf( &ap->detail_data[HFC_PARTSNUM], "%s", vpd_info->pn_value);
		sprintf( &ap->detail_data[HFC_EC], "%c", vpd_info->ec_value[0]);
	}
	else{
		vpdf_info = (struct hfc_vpd_five *)ap->mp_adap_info->vpd_buf;
		sprintf( &ap->detail_data[HFC_FWVERSION], "%06x", vpdf_info->fw_ver);
		sprintf( &ap->detail_data[HFC_DRVVERSION], "%s", vpdf_info->driver_ver);
		sprintf( &ap->detail_data[HFC_PARTSNUM], "%s", vpdf_info->pn_value);
		sprintf( &ap->detail_data[HFC_EC], "%c", vpdf_info->ec_level);
	}
	sprintf( &ap->detail_data[HFC_LOCATION], "%02x:%02x.%02x", 
			ap->pci_cfginf->bus->number,PCI_SLOT(ap->pci_cfginf->devfn),PCI_FUNC(ap->pci_cfginf->devfn));
	sprintf( &ap->detail_data[HFC_LOGSEQNUM], "%d", ap->raslog_cnt);
	sprintf( &ap->detail_data[HFC_IRQNUM], "%d", ap->irq);
	sprintf( &ap->detail_data[HFC_ADAPWWPN], "%llx", (unsigned long long)ap->ww_name);
	sprintf( &ap->detail_data[HFC_DEVICEID], "%04x", (short)ap->pkg.device_id);
	
	/* Output SFP Information *//* FCLNX-GPL-526 */
	if((ap->fw_support & 0x80)||(ap->fw_init_p->func & HFC_FWF_SFPINF)){			/* FCLNX-GPL-530 */
		if(!(ap->fw_init_p->sfp_info.sfp_status & HFC_SFP_INSTALL)){
				sprintf( &ap->detail_data[HFC_SFP_INFO], "N/A");
		}else {
			memset(sfp_type_name,0,sizeof(sfp_type_name));
			memcpy(sfp_type_name, ap->fw_init_p->sfp_info.sfp_type_name, sizeof(ap->fw_init_p->sfp_info.sfp_type_name)); 
			hfc_delete_space(sfp_type_name);
				
			memset(sfp_serial_no,0,sizeof(sfp_serial_no));
			memcpy(sfp_serial_no, ap->fw_init_p->sfp_info.sfp_serial_no, sizeof(ap->fw_init_p->sfp_info.sfp_serial_no)); 
			hfc_delete_space(sfp_serial_no);

			memset(sfp_date_code,0,sizeof(sfp_date_code));
			memcpy(sfp_date_code, ap->fw_init_p->sfp_info.sfp_date_code, sizeof(ap->fw_init_p->sfp_info.sfp_date_code)); 
			hfc_delete_space(sfp_date_code);

			if(!(ap->fw_init_p->sfp_info.sfp_status & HFC_SFP_VALID)){
				sprintf( &ap->detail_data[HFC_SFP_INFO], "incorrect data(%s, %s, %s)",
						sfp_type_name,
						sfp_serial_no,
						sfp_date_code);
			}else{
				sprintf( &ap->detail_data[HFC_SFP_INFO], "%s, %s, %s",
						sfp_type_name,
						sfp_serial_no,
						sfp_date_code);
			}
		}
	}/* FCLNX-GPL-526 */

#endif

	if( ( cmnd != NULL )&&( hfcp != NULL ) ){
		switch(type){
			case HFC_ERRLOG_TYPE_IODONE_WARN1 :
				sprintf( &ap->detail_data[HFC_WARNING], "INFO - Force to set retires from %d",cmnd->allowed);
				sprintf( &ap->detail_data[HFC_WARNING+36], "cdb=0x%02x for host%d:channel%d:id%d:lun%d.(instance%d)",
																		cmnd->cmnd[0],
																		CMND_HOST_NO(cmnd),
																		CMND_CHANNEL(cmnd),
																		CMND_TARGET(cmnd),
																		CMND_LUN(cmnd),
																		ap->instance);
				break;
			case HFC_ERRLOG_TYPE_IODONE_WARN2 :
				sprintf( &ap->detail_data[HFC_WARNING], "WARNING - Incomplete data write command.");
				sprintf( &ap->detail_data[HFC_WARNING+36], "cdb=0x%02x for host%d:channel%d:id%d:lun%d.(instance%d)",
																		cmnd->cmnd[0],
																		CMND_HOST_NO(cmnd),
																		CMND_CHANNEL(cmnd),
																		CMND_TARGET(cmnd),
																		CMND_LUN(cmnd),
																		ap->instance);
				break;
			case HFC_ERRLOG_TYPE_IODONE_WARN3 :
				sprintf( &ap->detail_data[HFC_WARNING], "retries:-1 allowed:-1 result=0x%08x adap_status=0x%08x.",
																		cmnd->result,
																		hfcp->adap_status);
				break;
			case HFC_ERRLOG_TYPE_IODONE_WARN4 :
				sprintf( &ap->detail_data[HFC_WARNING], "retries:%d allowed:%d  result=0x%08x  adap_status=0x%08x.",
																		hfcp->fail,
																		cmnd->allowed,
																		cmnd->result,
																		hfcp->adap_status);
				break;
			case HFC_ERRLOG_TYPE_IODONE_WARN5 :
				sprintf( &ap->detail_data[HFC_WARNING], "retries:%d allowed:%d  result=0x%08x adap_status=0x%08x.",
																		cmnd->retries,
																		cmnd->allowed,
																		cmnd->result,
																		hfcp->adap_status);
				break;
	    	default :
				break;
		}
	}

#ifndef _HFC_NO_RASLOG
	for(raslog_retry=0; raslog_retry < HFC_RASLOG_RETRY; raslog_retry++){
		raslog = _hraslog((char *)ras_error_id1, (char *)resource_name, HFC_RASLOG_LEN, (char *)ap->detail_data );	/* FCLNX-GPL-526 */
		if( raslog == 0 ){
			ap->raslog_cnt++; 
			break;

		}
	}
	if( raslog == 1 ){
		HFC_ERRPRT("hfcldd%d: HFC_EVNT3 FC Adapter Driver Warning Event (ErrNo:0xBD)\n", ap->dev_minor); 
	}
	else if( raslog == 2 ){
		HFC_ERRPRT("hfcldd%d: HFC_EVNT3 FC Adapter Driver Warning Event (ErrNo:0xBC)\n", ap->dev_minor); 
	}
	else if( raslog == 0 ){
		memcpy( ap->detail_data, &err_rec->hfc_detail_log[448], 512);
		sprintf( &ap->detail_data[HFC_LOGSEQNUM], "%d", ap->raslog_cnt);
		for(raslog_retry=0; raslog_retry < HFC_RASLOG_RETRY; raslog_retry++){
			raslog = _hraslog((char *)ras_error_id2, (char *)resource_name, HFC_RASLOG_LEN, (char *)ap->detail_data );	/* FCLNX-GPL-526 */
			if( raslog == 0 ){
				ap->raslog_cnt++; 
				break;
			}
		}
		if( raslog == 1 ){
			HFC_ERRPRT("hfcldd%d: HFC_EVNT3 FC Adapter Driver Warning Event (ErrNo:0xBD)\n", ap->dev_minor); 
		}
		else if( raslog == 2 ){
			HFC_ERRPRT("hfcldd%d: HFC_EVNT3 FC Adapter Driver Warning Event (ErrNo:0xBC)\n", ap->dev_minor); 
		}
	}
#endif

	return(raslog);

}


/*
 * Function:    hfc_fpp_logout
 *
 * Purpose:     LOG OUT for FPPf
 *
 * Arguments:   
 *  ap         - Pointer to adap_info
 *  err_no     - Error code
 *  mode       - mode
 *
 * Returns:     
 *
 * Notes:       
 */
 
const Type_mem fpp_iml_log[] = {
	/*---type-----address------size--*/
	{ TYPE_CFG , 0x0		, 80 } ,		/* PCI Config Reg.	*/
	{ TYPE_PCI , 0x0		, 32 } ,		/* PCI Memory		*/
	{ TYPE_PCI , 0x200		, 32 } ,		/* *				*/
	{ TYPE_PCI , 0x300		, 32 } ,		/*					*/
	{ TYPE_PCI , 0x800		, 256 } ,		/*					*/
	{ TYPE_TRC0, 0x41000000 , 528 } ,		/* TRC				*/
	{ TYPE_ELOG, 0			, 0   } ,		/*------------------*/
	{ TYPE_TRC1, 0x41000000 , 1024 } ,		/* TRC				*/
	{ TYPE_PCI , 0x900		, 1024 } ,		/* PCI				*/
	{ 0		   , 0			, 0    } ,
};

const Type_mem fpp_mck_loga0[] = {
	/*---type-----address------size--*/
	{ TYPE_SEQ , 0			, 8  }, 		/* Sequense No		*/
	{ TYPE_PCI , 0x0		, 48 } ,		/* PCI				*/
	{ TYPE_PCI , 0x200		, 16 } ,
	{ TYPE_PCI , 0x800		, 208 } ,
	{ TYPE_TRC0, 0x41000000 , 120 } ,		/* TRC0 			*/
	{ TYPE_TRC1, 0x41000000 , 512 } ,		/* TRC1 			*/
	{ TYPE_CFG , 0x4		, 48  } ,		/* PCI				*/
	{ TYPE_ELOG, 0			, 0   } ,		/*------------------*/
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x8d0		, 1072 } ,		/* PCI				*/
	{ TYPE_PCI , 0x210		, 16   } ,
	{ TYPE_TRC0, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_TRC1, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_0   , 0x00000200 , 512  } ,		/* WS				*/
	{ TYPE_0   , 0x00000800 , 256  } ,		/* WS				*/
	{ TYPE_0   , 0x00000a00 , 1536 } ,		/* WS				*/
	{ TYPE_01  , 0x41000000 , 4480 } ,		/* RHDR/THDR/TBUF	*/
	{ TYPE_0   , 0x43000000 , 512  } ,		/* BCR				*/
	{ TYPE_ILS , 0x08000000 , 2048 } ,		/* ILS				*/
	{ 0		   , 0			, 0    } ,
};

const Type_mem fpp_mck_log84[] = {
	/*---type-----address------size--*/
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x0		, 48 } ,		/* PCI				*/
	{ TYPE_PCI , 0x200		, 16 } ,
	{ TYPE_PCI , 0x800		, 208 } ,
	{ TYPE_TRC0, 0x41000000 , 168 } ,		/* TRC0 			*/
	{ TYPE_PCI , 0xb10		, 208 } ,		/* PCI				*/
	{ TYPE_PCI , 0xc40		, 48  } ,
	{ TYPE_0   , 0x03000000 , 256 } ,		/* BCR				*/
	{ TYPE_ELOG, 0			, 0   } ,		/*------------------*/
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x8d0		, 1072 } ,		/* PCI				*/
	{ TYPE_PCI , 0x210		, 16   } ,
	{ TYPE_TRC0, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_TRC1, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_0   , 0x00000200 , 512  } ,		/* WS				*/
	{ TYPE_0   , 0x00000800 , 256  } ,		/* WS				*/
	{ TYPE_0   , 0x00000a00 , 1536 } ,		/* WS				*/
	{ TYPE_01  , 0x41000000 , 4480 } ,		/* RHDR/THDR/TBUF	*/
	{ TYPE_0   , 0x43000000 , 512  } ,		/* BCR				*/
	{ TYPE_ILS , 0x08000000 , 2048 } ,		/* ILS				*/
	{ 0		   , 0			, 0    } ,
};

const Type_mem fpp_mck_log82[] = {
	/*---type-----address------size--*/
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x0		, 48 } ,		/* PCI				*/
	{ TYPE_PCI , 0x200		, 16 } ,		
	{ TYPE_PCI , 0x800		, 208 } ,		
	{ TYPE_TRC0 ,0x41000000 , 168 } ,		/* TRC0 			*/
	{ TYPE_PCI , 0x950		, 256 } ,		/* PCI				*/
	{ TYPE_ILS , 0x08000000 , 256 } ,		
	{ TYPE_ELOG, 0			, 0   } ,		/*------------------*/
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x8d0		, 1072 } ,		/* BCR				*/
	{ TYPE_PCI , 0x210		, 16   } ,		/* PCI				*/
	{ TYPE_TRC0, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_TRC1, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_0   , 0x00000200 , 512  } ,		/* WS				*/
	{ TYPE_0   , 0x00000800 , 256  } ,		/* WS				*/
	{ TYPE_0   , 0x00000a00 , 1536 } ,		/* WS				*/
	{ TYPE_01  , 0x41000000 , 4480 } ,		/* RHDR/THDR/TBUF	*/
	{ TYPE_0   , 0x43000000 , 512  } ,		/* BCR				*/
	{ TYPE_ILS , 0x08000000 , 2048 } ,		/* ILS				*/
	{ 0		   , 0			, 0    } ,
};

const Type_mem fpp_mck_log81[] = {
	/*---type-----address------size--*/ 	
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x0		, 48 } ,		
	{ TYPE_PCI , 0x200		, 16 } ,		
	{ TYPE_PCI , 0x800		, 208 } ,		
	{ TYPE_TRC0, 0x41000000 , 168 } ,		/* TRC0 			*/
	{ TYPE_PCI , 0xa00		, 256 } ,		
	{ TYPE_ILS , 0x08000000 , 256 } ,		/* BCR				*/
	{ TYPE_ELOG, 0			, 0   } ,		/*------------------*/
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x8d0		, 1072 } ,		/* PCI				*/
	{ TYPE_PCI , 0x210		, 16   } ,		
	{ TYPE_TRC0, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_TRC1, 0x41000000 , 2048 } ,		/* TRC				*/
	{ TYPE_0   , 0x00000200 , 512  } ,		/* WS				*/
	{ TYPE_0   , 0x00000800 , 256  } ,		/* WS				*/
	{ TYPE_0   , 0x00000a00 , 1536 } ,		/* WS				*/
	{ TYPE_01  , 0x41000000 , 4480 } ,		/* RHDR/THDR/TBUF	*/
	{ TYPE_0   , 0x43000000 , 512  } ,		/* BCR				*/
	{ TYPE_ILS , 0x08000000 , 2048 } ,		/* ILS				*/
	{ 0		   , 0			, 0    } ,
};

const Type_mem fpp_mck_other[] = {
	/*---type-----address------size--*/
	{ TYPE_SEQ , 0			, 8 },			/* Sequense No		*/
	{ TYPE_PCI , 0x0		, 48 } ,		 /* PCI 			*/
	{ TYPE_PCI , 0x200		, 16 } ,			
	{ TYPE_PCI , 0x800		, 208 } ,			
	{ TYPE_PCI , 0x300		, 32  } ,		 /* TRC0			*/
	{ TYPE_TRC0, 0x41000000 , 648 } ,		 /* PCI 			*/
	{ TYPE_ELOG, 0			, 0   } ,		 /*-----------------*/
	{ TYPE_SEQ , 0			, 8 },			 /* Sequense No 	*/
	{ TYPE_PCI , 0x8d0		, 1072 } ,			
	{ TYPE_PCI , 0x210		, 16   } ,		 /* BCR 			*/
	{ TYPE_TRC0, 0x41000000 , 2048 } ,		 /* TRC 			*/
	{ TYPE_TRC1, 0x41000000 , 2048 } ,		 /* TRC 			*/
	{ TYPE_0   , 0x00000200 , 512  } ,			
	{ TYPE_0   , 0x00000800 , 256  } ,		 /* TRC 	Type0/1 */
	{ TYPE_0   , 0x00000a00 , 1536 } ,		 /* WS				*/
	{ TYPE_01  , 0x41000000 , 4480 } ,		 /* RHDR/THDR/TBUF	*/
	{ TYPE_0   , 0x43000000 , 512  } ,		 /* BCR 			*/
	{ TYPE_ILS , 0x08000000 , 2048 } ,		 /* ILS 			*/
	{ 0		   , 0			, 0    } ,
};

void hfc_fpp_logout( struct adap_info *ap, uint err_no, uchar mode )
{
	struct mp_adap_info     *mpap ;
	Type_mem	*logmap;
	uchar		err_detail;
	int 		log_offset;
	int 		num;
	int 		i, rc;
	uint		reg_adr;
	uint		reg_adr_curr;
	uint        wk4;

	HFC_DBGPRT(" hfcldd%d : hfc_logout - start\n",ap->dev_minor);

	mpap = ap->mp_adap_info;

	rc = lock_try_mpap( mpap );										/* FCLNX-0279 */
	
	if( rc == 0 ){	/* Lock acquisition failure	*/
		set_bit( HFC_LOCK_WAIT_3, (ulong *)&ap->mpap_lock );
		ap->err_no = err_no;
		ap->mode = mode;
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
		hfc_w_start( ap, HFC_MPAP_LOCK_TMR );
		return;
	}

	if ( test_bit( HFC_LOCK_WAIT_3, (ulong *)&ap->mpap_lock ) ) {
		/* stop retry timer */
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
		clear_bit( HFC_LOCK_WAIT_3, (ulong *)&ap->mpap_lock );
	}																/* FCLNX-0279 */
	
	if( mode == HFC_ERRLOG_TYPE_IMLLOG ){
		logmap = (Type_mem *) fpp_iml_log;
	} else {
		err_detail =
			 hfc_read_reg_ext( ap, (uint)HFC_IOSPACE_ERRDETAIL0, 0x1 );
		if( err_detail == 0xa0 )
			logmap = (Type_mem *) fpp_mck_loga0;
		else if( err_detail == 0x84 )
			logmap = (Type_mem *) fpp_mck_log84;
		else if( err_detail == 0x82 )
			logmap =(Type_mem *)  fpp_mck_log82;
		else if( err_detail == 0x81 )
			logmap = (Type_mem *) fpp_mck_log81;
		else
			logmap = (Type_mem *) fpp_mck_other;
		mpap->mck_seq_no++;
	}
	
	log_offset = 0;
	num = 0;

	while( (log_offset < HFC_HWLOG_SIZE/4) && (logmap[num].type != 0) )
	{
		if( logmap[num].type == TYPE_SEQ )
		{
			mpap->hw_log[log_offset] = err_no ;
			mpap->hw_log[log_offset+1] = mpap->mck_seq_no ;
			log_offset+=2 ;
		}
		else if( logmap[num].type == TYPE_PCI )
		{
			for(reg_adr=0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				wk4 = (uint)hfc_read_reg_ext(									/* FCWIN-0197 */
					ap, logmap[num].reg_adr+reg_adr, (char)0x4) ;
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				log_offset++ ;
			}
		}
		else if( logmap[num].type == TYPE_CFG )
		{
			for(reg_adr=0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				wk4 = (uint)hfc_read_cnfg(										/* FCWIN-0197 */
					ap, logmap[num].reg_adr+reg_adr, (char)0x4) ;
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				log_offset++ ;
			}
		}
		else if( (logmap[num].type == TYPE_0) || (logmap[num].type == TYPE_1) ||
				 (logmap[num].type == TYPE_01) )
		{
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,logmap[num].type);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,logmap[num].reg_adr);
			for(reg_adr=0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				wk4 = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4);			/* FCWIN-0197 */
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				log_offset++ ;
			}
		}
		else if( logmap[num].type == TYPE_ILS )
		{
			if( logmap[num].size < 2048 )
			{
				/* FRAME TRC */
				hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(uchar)TYPE_0);
				hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,(uint)0x00000340); /* @1.82 */
				reg_adr_curr = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4) & 0x0000ffff;
				if( (reg_adr_curr < 0x9000) || (reg_adr_curr > 0x9fff) )
				{
					/* A current pointer is outside of the range */
					for(i = 0 ; i < logmap[num].size ; i+=4 )
					{
						mpap->hw_log[log_offset] = 0 ;
						log_offset++ ;
					}
				}
				else
				{
					reg_adr = reg_adr_curr - logmap[num].size/4 ;
					if( reg_adr < 0x9000 )
						reg_adr = 0xa000 - (0x9000 - reg_adr) ;
					reg_adr = reg_adr * 4 ;
					hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
					while( reg_adr != (reg_adr_curr*4) )
					{
						wk4 = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4);	/* FCWIN-0197 */
						HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
						reg_adr += 4 ;
						log_offset++ ;
						if( reg_adr >= (0xa000*4) )
						{
							reg_adr = 0x9000*4 ;
							hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
						}
					}
				}
			}
			else
			{
				/* EVENT TRC */
				hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(uchar)TYPE_0);
				hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,(uint)0x00000324);
				reg_adr_curr = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4) & 0x0000ffff;
				if( (reg_adr_curr < 0x8000) || (reg_adr_curr > 0x8fff) )
				{
					/* A current pointer is outside of the range */
					for(i = 0 ; i < 1024 ; i+=4 )
					{
						mpap->hw_log[log_offset] = 0 ;
						log_offset++ ;
					}
				}
				else
				{
					reg_adr = reg_adr_curr - 0x100 ;
					if( reg_adr < 0x8000 )
						reg_adr = 0x9000 - (0x8000 - (reg_adr_curr - 0x100)) ;
					reg_adr = reg_adr * 4 ;
					hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
					while( reg_adr != (reg_adr_curr*4) )
					{
						wk4 = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4);	/* FCWIN-0197 */
						HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
						reg_adr += 4 ;
						log_offset++ ;
						if( reg_adr >= (0x9000*4) )
						{
							reg_adr = 0x8000*4 ;
							hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
						}
					}
				}
				/* FRAME TRC */
				hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(uchar)TYPE_0);
				hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,(uint)0x00000340); /* @1.82 */
				reg_adr_curr = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4) & 0x0000ffff;
				if( (reg_adr_curr < 0x9000) || (reg_adr_curr > 0x9fff) )
				{
					/* A current pointer is outside the range */
					for(i = 0 ; i < 1024 ; i+=4 )
					{
						mpap->hw_log[log_offset] = 0 ;
						log_offset++ ;
					}
				}
				else
				{
					reg_adr = reg_adr_curr - 0x100 ;
					if( reg_adr < 0x9000 )
						reg_adr = 0xa000 - (0x9000 - reg_adr) ;
					reg_adr = reg_adr * 4 ;
					hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
					while( reg_adr != (reg_adr_curr*4) )
					{
						wk4 = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4);		/* FCWIN-0197 */
						HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
						reg_adr += 4 ;
						log_offset++ ;
						if( reg_adr >= (0xa000*4) )
						{
							reg_adr = 0x9000*4 ;
							hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
						}
					}
				}
			}
		}
		else if( logmap[num].type == TYPE_TRC0 )
		{
			reg_adr_curr = (uint)hfc_read_reg_ext(ap,(uint)0xf0,(char)0x2) + 0x800 ;
			reg_adr = reg_adr_curr - logmap[num].size/4 ;
			if( reg_adr < 0x800 )
				reg_adr = 0xc00 - ( 0x800 - reg_adr ) ;
			reg_adr = reg_adr * 4 ;
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(uchar)TYPE_1);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
			for(i=0 ; i<logmap[num].size ; i+=4)
			{
				wk4 = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4);	/* FCWIN-0197 */
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				log_offset++ ;
			}
		}
		else if( logmap[num].type == TYPE_TRC1 )
		{
			reg_adr_curr = (uint)hfc_read_reg_ext(ap,(uint)0xf2,(char)0x2) + 0xc00 ;
			reg_adr = reg_adr_curr - logmap[num].size/4 ; ;
			if( reg_adr < 0xc00 )
				reg_adr = 0x1000 - ( 0xc00 - reg_adr ) ;
			reg_adr = reg_adr * 4 ;
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(uchar)TYPE_1);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,reg_adr+logmap[num].reg_adr);
			for(i=0 ; i<logmap[num].size ; i+=4)
			{
				wk4 = (uint)hfc_read_reg_ext(ap,REG_INDAREA,(char)0x4);	/* FCWIN-0197 */
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				log_offset++ ;
			}
		}
		else if( logmap[num].type == TYPE_ELOG )
		{
			if( mode == HFC_ERRLOG_TYPE_CHKSTP )
				break ;
			if( test_bit(HFC_HWLOG_VALID, (ulong *)&ap->io_status) )
				break ;
			else
				set_bit(HFC_HWLOG_VALID, (ulong *)&ap->io_status );
		}
		num++ ;
	}
	hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(char)0x00);

	HFC_ADAP_UNLOCK( mpap, HFC_MP_ADAP_BUSY );
	HFC_DBGPRT(" hfcldd : hfc_fpp_logout - end\n");
}

const Type_mem five_iml_log[] = {
	/*---type-----address------size--*/
	{ TYPE_CFG    , 0x0        , 544   } , /* 0x0100 PCI Config Reg.  */    /*  FCWIN-0284  */
	{ TYPE_PCI    , 0x0        , 48    } , /* 0x0320 PCI Memory       */
	{ TYPE_PCI    , 0x10A0     , 64    } , /* 0x0350 INT Information  */
	{ TYPE_RSV    , 0x0        , 112   } , /* 0x0390 Reserve          */    /*  FCWIN-0284  */
	{ TYPE_TRC0   , 0x06000000 , 384   } , /* 0x0400 TRC              */
	{ TYPE_0      , 0x00000000 , 32    } , /* 0x0580 Com Area         */
	{ TYPE_0      , 0x00000080 , 32    } , /* 0x05A0                  */
	{ TYPE_0      , 0x00000100 , 32    } , /* 0x05C0                  */
	{ TYPE_0      , 0x00000180 , 32    } , /* 0x05E0                  */
	{ TYPE_TRC1   , 0x06002000 , 512   } , /* 0x0600 TRC              */
	{ TYPE_PCI    , 0x00001800 , 176   } , /* 0x0800 WA               */
	{ TYPE_PCI    , 0x00001040 , 64    } , /* 0x08B0                  */
	{ TYPE_PCI    , 0x000010E0 , 80    } , /* 0x08F0                  */
	{ TYPE_PCI    , 0x000011C0 , 64    } , /* 0x0940                  */
	{ TYPE_PCI    , 0x0210     , 40    } , /* 0x0980 MCW              */
	{ TYPE_PCI    , 0x000018B0 , 592   } , /* 0x09A8 WA               */
	{ TYPE_RSV    , 0x0        , 8     } , /* 0x0BF8 Reserve          */
	{ TYPE_PCI    , 0x00001B00 , 1024  } , /* 0x0C00 WA               */    /*  FCWIN-0265  */
	{ 0           , 0          , 0     } ,
};
	
	/* Mck log format for each installation port */
const Type_mem five_mck_iport1[] = {
	{ TYPE_SEQ    , 0x0        , 8     } , /* 0x0100 Sequence No      */
	{ TYPE_PCI    , 0x0        , 48    } , /* 0x0108 PCI              */
	{ TYPE_PCI    , 0x10A0     , 64    } , /* 0x0138 INT information  */
	{ TYPE_PCI    , 0x00001800 , 176   } , /* 0x0178 WA               */
	{ TYPE_PCI    , 0x00001040 , 64    } , /* 0x0228                  */
	{ TYPE_PCI    , 0x00001920 , 64    } , /* 0x0268                  */
	{ TYPE_TRC1   , 0x06002000 , 344   } , /* 0x02A8 TRC              */
	{ TYPE_TRC0   , 0x06000000 , 1024  } , /* 0x0400                  */
	{ TYPE_SEQ    , 0x0        , 8     } , /* 0x0800 Sequence No      */
	{ TYPE_CFG    , 0x0        , 544   } , /* 0x0808 PCI Config Reg.  */
	{ TYPE_0      , 0x00002050 , 48    } , /* 0x0A28                  */
	{ TYPE_0      , 0x00002450 , 48    } , /* 0x0A58                  */
	{ TYPE_0      , 0x00002850 , 48    } , /* 0x0A88                  */
	{ TYPE_0      , 0x00002C50 , 48    } , /* 0x0AB8                  */
	{ TYPE_PCI    , 0x00001090 , 624   } , /* 0x0AE8                  */
	{ TYPE_PCI    , 0x000018B0 , 1616  } , /* 0x0D58                  */
	{ TYPE_2      , 0x0E000000 , 1280  } , /* 0x13A8                  */
	{ TYPE_2      , 0x0E000780 , 240   } , /* 0x18A8                  */
	{ TYPE_2      , 0x0E000890 , 208   } , /* 0x1998                  */
	{ TYPE_2      , 0x0E000980 , 112   } , /* 0x1A68                  */
	{ TYPE_2      , 0x0E000A10 , 208   } , /* 0x1AD8                  */
	{ TYPE_2      , 0x0E000B00 , 112   } , /* 0x1BA8                  */
	{ TYPE_2      , 0x0E000B90 , 208   } , /* 0x1C18                  */
	{ TYPE_2      , 0x0E000C80 , 112   } , /* 0x1CE8                  */
	{ TYPE_2      , 0x0E000D10 , 208   } , /* 0x1D58                  */
	{ TYPE_2      , 0x0E000E00 , 80    } , /* 0x1E28                  */
	{ TYPE_2      , 0x0E000E80 , 256   } , /* 0x1E78                  */
	{ TYPE_RSV    , 0x0        , 136   } , /* 0x1F78                  */
	{ TYPE_EVT0   , 0x04020000 , 4096  } , /* 0x2000 Event Trace      */
	{ TYPE_FRM0   , 0x04030000 , 4096  } , /* 0x3000 Frame Trace      */
	{ 0           , 0          , 0     } ,
};
	
const Type_mem five_mck_iport2[] = {
	{ TYPE_SEQ    , 0x0        , 8     } , /* 0x0100 Sequence No      */
	{ TYPE_PCI    , 0x0        , 48    } , /* 0x0108 PCI              */
	{ TYPE_PCI    , 0x10A0     , 64    } , /* 0x0138 INT information  */
	{ TYPE_PCI    , 0x00001800 , 176   } , /* 0x0178 WA               */
	{ TYPE_PCI    , 0x00001040 , 64    } , /* 0x0228                  */
	{ TYPE_PCI    , 0x00001920 , 64    } , /* 0x0268                  */
	{ TYPE_TRC1   , 0x06002000 , 344   } , /* 0x02A8 TRC              */
	{ TYPE_TRC0   , 0x06000000 , 1024  } , /* 0x0400                  */    /**/
	{ TYPE_SEQ    , 0x0        , 8     } , /* 0x0800 Sequence No      */
	{ TYPE_CFG    , 0x0        , 544   } , /* 0x0808 PCI Config Reg.  */
	{ TYPE_0      , 0x00002050 , 48    } , /* 0x0A28                  */
	{ TYPE_0      , 0x00002450 , 48    } , /* 0x0A58                  */
	{ TYPE_0      , 0x00002850 , 48    } , /* 0x0A88                  */
	{ TYPE_0      , 0x00002C50 , 48    } , /* 0x0AB8                  */
	{ TYPE_PCI    , 0x00001090 , 624   } , /* 0x0AE8                  */
	{ TYPE_PCI    , 0x000018B0 , 1616  } , /* 0x0D58                  */
	{ TYPE_2      , 0x0E000000 , 1280  } , /* 0x13A8                  */
	{ TYPE_2      , 0x0E000780 , 240   } , /* 0x18A8                  */
	{ TYPE_2      , 0x0E000890 , 208   } , /* 0x1998                  */
	{ TYPE_2      , 0x0E000980 , 112   } , /* 0x1A68                  */
	{ TYPE_2      , 0x0E000A10 , 208   } , /* 0x1AD8                  */
	{ TYPE_2      , 0x0E000B00 , 112   } , /* 0x1BA8                  */
	{ TYPE_2      , 0x0E000B90 , 208   } , /* 0x1C18                  */
	{ TYPE_2      , 0x0E000C80 , 112   } , /* 0x1CE8                  */
	{ TYPE_2      , 0x0E000D10 , 208   } , /* 0x1D58                  */
	{ TYPE_2      , 0x0E000E00 , 80    } , /* 0x1E28                  */
	{ TYPE_2      , 0x0E000E80 , 256   } , /* 0x1E78                  */
	{ TYPE_RSV    , 0x0        , 136   } , /* 0x1F78                  */
	{ TYPE_EVT0   , 0x04020000 , 2048  } , /* 0x2000 Event Trace      */
	{ TYPE_EVT1   , 0x04024000 , 2048  } , /* 0x2800 Frame Trace      */
	{ TYPE_FRM0   , 0x04030000 , 2048  } , /* 0x3000 Event Trace      */
	{ TYPE_FRM1   , 0x04034000 , 2048  } , /* 0x3800 Frame Trace      */
	{ 0           , 0          , 0     } ,
};
	
const Type_mem five_mck_iport4[] = {
	{ TYPE_SEQ    , 0x0        , 8     } , /* 0x0100 Sequence No      */
	{ TYPE_PCI    , 0x0        , 48    } , /* 0x0108 PCI              */
	{ TYPE_PCI    , 0x10A0     , 64    } , /* 0x0138 INT information  */
	{ TYPE_PCI    , 0x00001800 , 176   } , /* 0x0178 WA               */
	{ TYPE_PCI    , 0x00001040 , 64    } , /* 0x0228                  */
	{ TYPE_PCI    , 0x00001920 , 64    } , /* 0x0268                  */
	{ TYPE_TRC1   , 0x06002000 , 344   } , /* 0x02A8 TRC              */
	{ TYPE_TRC0   , 0x06000000 , 1024  } , /* 0x0400                  */    /**/
	{ TYPE_SEQ    , 0x0        , 8     } , /* 0x0800 Sequence No      */
	{ TYPE_CFG    , 0x0        , 544   } , /* 0x0808 PCI Config Reg.  */
	{ TYPE_0      , 0x00002050 , 48    } , /* 0x0A28                  */
	{ TYPE_0      , 0x00002450 , 48    } , /* 0x0A58                  */
	{ TYPE_0      , 0x00002850 , 48    } , /* 0x0A88                  */
	{ TYPE_0      , 0x00002C50 , 48    } , /* 0x0AB8                  */
	{ TYPE_PCI    , 0x00001090 , 624   } , /* 0x0AE8                  */
	{ TYPE_PCI    , 0x000018B0 , 1616  } , /* 0x0D58                  */
	{ TYPE_2      , 0x0E000000 , 1280  } , /* 0x13A8                  */
	{ TYPE_2      , 0x0E000780 , 240   } , /* 0x18A8                  */
	{ TYPE_2      , 0x0E000890 , 208   } , /* 0x1998                  */
	{ TYPE_2      , 0x0E000980 , 112   } , /* 0x1A68                  */
	{ TYPE_2      , 0x0E000A10 , 208   } , /* 0x1AD8                  */
	{ TYPE_2      , 0x0E000B00 , 112   } , /* 0x1BA8                  */
	{ TYPE_2      , 0x0E000B90 , 208   } , /* 0x1C18                  */
	{ TYPE_2      , 0x0E000C80 , 112   } , /* 0x1CE8                  */
	{ TYPE_2      , 0x0E000D10 , 208   } , /* 0x1D58                  */
	{ TYPE_2      , 0x0E000E00 , 80    } , /* 0x1E28                  */
	{ TYPE_2      , 0x0E000E80 , 256   } , /* 0x1E78                  */
	{ TYPE_RSV    , 0x0        , 136   } , /* 0x1F78                  */
	{ TYPE_EVT0   , 0x04020000 , 1024  } , /* 0x2000 Event Trace      */
	{ TYPE_EVT1   , 0x04024000 , 1024  } , /* 0x2400 Frame Trace      */
	{ TYPE_EVT2   , 0x04028000 , 1024  } , /* 0x2800 Event Trace      */
	{ TYPE_EVT3   , 0x0402C000 , 1024  } , /* 0x2C00 Frame Trace      */
	{ TYPE_FRM0   , 0x04030000 , 1024  } , /* 0x3000 Event Trace      */
	{ TYPE_FRM1   , 0x04034000 , 1024  } , /* 0x3400 Frame Trace      */
	{ TYPE_FRM2   , 0x04038000 , 1024  } , /* 0x3800 Event Trace      */
	{ TYPE_FRM3   , 0x0403C000 , 1024  } , /* 0x3C00 Frame Trace      */
	{ 0           , 0          , 0     } ,
};

void hfc_five_logout(
	struct adap_info        *ap,
	uint                    err_no,
	uchar                   mode)
{
	const uint  mck_point[] = {
		0x00002108,         /* Port0 Event Trace */
		0x00002508,         /* Port1 Event Trace */
		0x00002908,         /* Port2 Event Trace */
		0x00002D08,         /* Port3 Event Trace */
		0x00002100,         /* Port0 Frame Trace */
		0x00002500,         /* Port1 Frame Trace */
		0x00002900,         /* Port2 Frame Trace */
		0x00002D00};        /* Port3 Frame Trace */
	
	struct mp_adap_info     *mpap;
	struct adap_info        *wk_ap;
	Type_mem                *logmap=NULL ;
	int                     log_offset ;
	int                     num, rc;
	uint                    i, j ;
	uint                    reg_adr, wk_reg_adr ;
	uint                    wk4, wk_cfg;
	uint                    read_pos;
	uint                    point_adr;
	uint                    ram_adr;
	uint                    trc_range=0;
	uchar                   func_num=0, wk_func_num=0;
	uchar                   iport, wk_iport;


	struct hfc_err_head_plus head_plus;
	
	HFC_DBGPRT(" hfcldd%d : hfc_five_logout - start\n",ap->dev_minor);
	
	mpap = ap->mp_adap_info;
	
	rc = lock_try_mpap( mpap );											/* FCLNX-0276 */
	
	if( rc == 0 ){		/* Lock failed */
		set_bit( HFC_LOCK_WAIT_4, (ulong *)&ap->mpap_lock );
		ap->err_no = err_no;
		ap->mode = mode;
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
		hfc_w_start( ap, HFC_MPAP_LOCK_TMR );
		return;
	}

	if( test_bit( HFC_LOCK_WAIT_4, (ulong *)&ap->mpap_lock ) )
	{
		/* Stop retry timer */
		clear_bit( HFC_LOCK_WAIT_4, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
	}																	/* FCLNX-0276 */

	set_bit(HFC_HWLOG_SAVING, (ulong *)&mpap->status );

	mpap->hwlog_result |= ( 0x00000001 << ap->port_no );
	
	if( ( mode == HFC_ERRLOG_TYPE_IMLLOG ) || ( mode == HFC_ERRLOG_TYPE_IMLLOG_DIAG) )
	{
		logmap = (Type_mem *) five_iml_log ;
	}
	else
	{
		wk_iport = (uchar)hfc_read_reg_ext(ap,(uint)0x1e,(char)1);

		iport = (wk_iport >> 4) & 0x03;                                         /* FCWIN-0232 */

		if( iport == 0x03 )
		{
			logmap = (Type_mem *) five_mck_iport4 ;
			trc_range = 0x1000;
		}
		else if( iport == 0x01 )
		{
			logmap = (Type_mem *) five_mck_iport2 ;
			trc_range = 0x1000;
		}
		else
		{
			logmap = (Type_mem *) five_mck_iport1 ;
			trc_range = 0x1000;
		}
			
		if (mode == HFC_ERRLOG_TYPE_MCK)
		{
			/* Count mck_seq_no only for MCK log */
			mpap->mck_seq_no++ ;
		}
	}

	log_offset = 0 ;
	num = 0 ;
	
	for (j=0; j<4;j++)
	{
		head_plus.type = mode;
		head_plus.errno = err_no;
		
		HFC_MEMCPY(&mpap->hw_log[log_offset], &head_plus, 64);
		memset(&head_plus, 0, sizeof(struct hfc_err_head_plus));
		log_offset += 16;
	}
	
	while( (log_offset < (HFC_HWLOG_SIZE - 256)/4) && (logmap[num].type != 0) )
	{
		if( logmap[num].type == TYPE_SEQ )
		{
			HFC_4L_TO_4B( mpap->hw_log[log_offset], err_no ) ;
			HFC_4L_TO_4B( mpap->hw_log[log_offset+1], mpap->mck_seq_no ) ;
			log_offset+=2 ;
		}
		else if( logmap[num].type == TYPE_RSV )
		{
			for(reg_adr=0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				mpap->hw_log[log_offset] = 0xffffffff;
				log_offset++;
			}
		}
		else if( logmap[num].type == TYPE_PCI )
		{
			for(reg_adr=0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				wk4 = (uint)hfc_read_reg_ext(
					ap, logmap[num].reg_adr+reg_adr, (char)0x4) ;
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				log_offset++ ;
			}
		}
		else if( logmap[num].type == TYPE_ROTATE )
		{
			/* Sort the area where ULP rotates in ULP number order */
			for(reg_adr=0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				if (reg_adr + (func_num * (logmap[num].size / 4)) < logmap[num].size)
					wk_reg_adr = reg_adr + (func_num * (logmap[num].size / 4));
				else
					wk_reg_adr = reg_adr + (func_num * (logmap[num].size / 4)) - logmap[num].size;
				
				wk4 = (uint)hfc_read_reg_ext(ap, logmap[num].reg_adr+wk_reg_adr, 0x4);     /* FCWIN-0233 STR   */
				HFC_4L_TO_4B(mpap->hw_log[log_offset], wk4);
				log_offset++;                                                                           /* FCWIN-0233 END   */
			}
		}
		else if( logmap[num].type == TYPE_CFG )
		{
			for( j=0 ; j<4 ; j++)
			{
				wk_ap = ap;
				while( wk_ap->prev != NULL){
					wk_ap = wk_ap->prev;
				}
				
				for( i=0 ; i<4 ; i++)
				{
					wk_func_num = wk_ap->port_no;
					if ( wk_func_num == j)
						break;
					
					wk_ap = wk_ap->next;
					if (wk_ap == NULL)
						break;
				}
				
				if ( wk_ap == NULL )
				{
					/* 0 pading */
					for(reg_adr = 0 ; reg_adr<(logmap[num].size/4) ; reg_adr+=4)
					{
						memset(&mpap->hw_log[log_offset], 0, 4);
						log_offset ++;                                           /*  FCWIN-0232 */
					}
				}
				else
				{
					for(reg_adr=0 ; reg_adr<(logmap[num].size/4 + 0x10 ) ; reg_adr+=4)
					{
						/* No output for config area address 0x80 - 0x8F */
						if ( reg_adr == 0x80 )
							reg_adr = 0x90;                                     /*  FCWIN-0283  */

						wk_cfg = (uint) hfc_read_cnfg( wk_ap, reg_adr, 0x04);
						HFC_4L_TO_4B(mpap->hw_log[log_offset], wk_cfg);
						log_offset++ ;
					}
				}
			}
		}
		else if( (logmap[num].type == TYPE_0) || (logmap[num].type == TYPE_1) ||
				 (logmap[num].type == TYPE_01) || (logmap[num].type == TYPE_2) )
		{
			/* Set indirect access flag */
			hfc_write_reg_ext(ap, 0x2fb, 0x01, 0x08);
			
			hfc_read_reg_ext(ap, (uint)REG_RAMADR, (char)0x1);                                              /* FCWIN-0236 */
			
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,logmap[num].type);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,logmap[num].reg_adr);
			for(reg_adr=0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				read_pos = ( reg_adr + logmap[num].reg_adr + REG_INDAREA_FIVE ) % 0x80 + REG_INDAREA_FIVE;  /* FCWIN-0234 */
				
				wk4 = (uint)hfc_read_reg_ext(ap, read_pos, (char)0x4);
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				log_offset++ ;
			}
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,0x00);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x1,0x80);
			
			/* Reset indirect access flag */
			hfc_write_reg_ext(ap, 0x2fb, 0x01, 0x00);
		}
		else if ( ( logmap[num].type == TYPE_TRC0 ) || ( logmap[num].type == TYPE_TRC1 ) )
		{
			if ( logmap[num].type == TYPE_TRC0 )
				ram_adr = (uint)hfc_read_reg_ext(ap,(uint)0xf0,(char)0x2);
			else
				ram_adr = (uint)hfc_read_reg_ext(ap,(uint)0xf2,(char)0x2);
			
			if ( logmap[num].type == TYPE_TRC0 )
			{
				if ( ram_adr < logmap[num].size/4)                                 /* FCWIN-255 */
					ram_adr = logmap[num].reg_adr + 0x2000 + ram_adr * 4 - logmap[num].size;
				else
					ram_adr = logmap[num].reg_adr + ram_adr * 4 - logmap[num].size;
			}
			else
			{
				if ( ram_adr < logmap[num].size/8)
					ram_adr = logmap[num].reg_adr + 0x2000 + ram_adr * 8 - logmap[num].size ;
				else
					ram_adr = logmap[num].reg_adr + ram_adr * 8 - logmap[num].size ;
			}
			
			/* Set indirect access flag */
			hfc_write_reg_ext(ap, 0x2fb, 0x01, 0x08);
			
			hfc_read_reg_ext(ap, (uint)REG_RAMADR, (char)0x1);
			
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(uchar)TYPE_1);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,ram_adr);
			
			read_pos = ram_adr % 0x80 + REG_INDAREA_FIVE;
			for( reg_adr =0 ; reg_adr <logmap[num].size ; reg_adr +=4)
			{
				if ( ram_adr == (logmap[num].reg_adr + 0x2000) )                                  /*  FCWIN-0284  */
				{
					hfc_write_reg_ext(ap, (uint)REG_RAMADR, (char)0x4, logmap[num].reg_adr);
					ram_adr = logmap[num].reg_adr;                                              /*  FCWIN-0284  */
					read_pos = logmap[num].reg_adr % 0x80 + REG_INDAREA_FIVE;
				}                                                                               /*  FCWIN-0284  */
				
				wk4 = (uint)hfc_read_reg_ext(ap,read_pos,(char)0x4);
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				
				ram_adr += 4;
				hfc_write_reg_ext(ap, (uint)REG_RAMADR, (char)0x4, ram_adr );    /*  FCWIN-0284  */
				read_pos = ram_adr % 0x80 + REG_INDAREA_FIVE;
				
				log_offset++ ;
			}
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,0x00);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x1,0x80);
			
			/* Reset indirect access flag */
			hfc_write_reg_ext(ap, 0x2fb, 0x01, 0x00);
		}
		else if( ( logmap[num].type >= TYPE_EVT0 ) && ( logmap[num].type <= TYPE_FRM3 ) )
		{
			/* Set indirect access flag */
			hfc_write_reg_ext(ap, 0x2fb, 0x01, 0x08);
			
			hfc_read_reg_ext(ap, (uint)REG_RAMADR, (char)0x1);                                              /* FCWIN-0236 */
			
			point_adr = mck_point[logmap[num].type - TYPE_EVT0];
			
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(uchar)TYPE_0);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x4,point_adr);
			read_pos = ( point_adr + REG_INDAREA_FIVE ) % 0x80 + REG_INDAREA_FIVE;  /* FCWIN-0234 */
			wk4 = (uint)hfc_read_reg_ext(ap, read_pos, (char)0x4);
			wk4 &= 0x00ffffff;
			
			if( wk4 * 4 - logmap[num].size <= logmap[num].reg_adr - 0x04000000)
				ram_adr = (wk4 + trc_range - logmap[num].size/4) * 4 + 0x04000000;
			else
				ram_adr = (wk4 - logmap[num].size/4) * 4 + 0x04000000;
			
			hfc_write_reg_ext(ap, (uint)REG_RAMADR, (char)0x4, ram_adr);
			
			read_pos = ram_adr % 0x80 + REG_INDAREA_FIVE;
			for( reg_adr = 0 ; reg_adr<logmap[num].size ; reg_adr+=4)
			{
				if ( ram_adr == logmap[num].reg_adr + trc_range * 4 )
				{
					hfc_write_reg_ext(ap, (uint)REG_RAMADR, (char)0x4, logmap[num].reg_adr);
					ram_adr = logmap[num].reg_adr;                              /* FCWIN-0284 */
					read_pos = logmap[num].reg_adr % 0x80 + REG_INDAREA_FIVE;   /* FCWIN-0234 */
				}
				
				wk4 = (uint)hfc_read_reg_ext(ap, read_pos, (char)0x4);
				HFC_4L_TO_4B(mpap->hw_log[log_offset],wk4);
				
				ram_adr += 4;
				hfc_write_reg_ext(ap, (uint)REG_RAMADR, (char)0x4, ram_adr );    /*  FCWIN-0284  */
				read_pos = ram_adr % 0x80 + REG_INDAREA_FIVE;
								
				log_offset++ ;
			}
			hfc_write_reg_ext(ap,( uint )REG_RAMMSK,( char )0x1,(char)0x00);
			hfc_write_reg_ext(ap,( uint )REG_RAMADR,( char )0x1,0x80);
			
			/* Reset indirect access flag */
			hfc_write_reg_ext(ap, 0x2fb, 0x01, 0x00);
		}
		num++ ;
	}
#if _HFC_DEBUG_TIMER_00
	memorydump("LOGOUT", (uchar *)mpap->hw_log, HFC_HWLOG_SIZE);
#endif
	HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);										/* FCLNX-0279 */
	HFC_DBGPRT(" hfcldd : hfc_five_logout - end\n");
}


/*
 * Function:    hfc_abend
 *
 * Purpose:     Abend processing
 *
 * Arguments:   
 *  ap         - 
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_abend(
	struct adap_info        *ap,
	uchar                   type)
{
	struct mp_adap_info     *mpap;
	uint                     lp ;
	struct target_info      *target ;
	uint                    dump_reg[4] ;
	uint                    err_no=0 ;
	uint                    adap_status=0;
	uint					lun, rsp;

	HFC_DBGPRT(" hfcldd%d : hfc_abend - start type = %x \n", ap->dev_minor, type);

	mpap = ap->mp_adap_info;
	hfc_hand2_trace(HFC_TRC_ABEND, type, ap, NULL, NULL, 0, 0, 0);
	switch( type )
	{
		case HFC_ABEND_XRB_INVALID:
		case HFC_ABEND_CCC:
		case HFC_ABEND_MB_RSPERR:
			/* (1) In the mailbox response, xcc=83 or fsb=CCC    */
			/* (2) In the SCSI response, xcc=83 or fsb=CCC       */
			/* (3) In the SCSI response, xrb Valid=0             */
			/*==> Process <==                                    */
			/* Count over (ap->link_dead_cnt >HFC_LINK_DEAD_CNT) */
			/*   -> Execute force check stop                     */
			/* Otherwise                                         */
			/*   -> Return (Counter Up)                          */

			ap->link_dead_cnt++ ;
			if( ap->link_dead_cnt > HFC_LINK_DEAD_CNT )
			{
				HFC_ISSUE_FMCK(ap, type);									/* FCLNX-0279 */
			}
			break ;
		case HFC_ABEND_SERR:
		case HFC_ABEND_PERR:
		case HFC_ABEND_SPERR:
			/* (1) PCI ERROR INT                                 */
			/*==> Process <==                                    */
			/* Count over (ap->link_dead_cnt >HFC_LINK_DEAD_CNT) */
			/*   -> Return (Counter Up)                          */
			/* Cancel pending SCSI commands under this adapter   */
			if( HFC_MMODE_CHECK_SHARED(ap) && !( HFC_MMODE_CHECK_SHADOW(ap) ) ) /* @MLPF */
				hfc_mlpf_pci_error(ap, type);
			
			if( type == HFC_ABEND_SERR )
			{
				adap_status = SCS_PCI_BUS_SERR;
				err_no = 0x00000032 ;
			}
			if( type == HFC_ABEND_PERR )
			{
				adap_status = SCS_PCI_BUS_PERR;
				err_no = 0x00000033 ;
			}
			if( type == HFC_ABEND_SPERR )
			{
				adap_status = SCS_PCI_BUS_SPERR;
				err_no = 0x00000034 ;
			}
			
			for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)
			{
				target = hfc_hash_target_info(ap, lp);
				if (target != NULL)
				{
					hfc_cancel_scsi_cmd(
						ap, target, 0, NULL, adap_status, HFC_CSCSI_ERROR,
						TRUE, TRUE, HFC_FLASH_TARGET);
					
					if ( hfc_manage_info.hfcldd_mp_mod ) {
						hfc_manage_info.npubp->hfc_forced_offline_e(target, TRUE);
					}
				}
			}
			
			dump_reg[0] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_INTA, (char)0x4) ;
			dump_reg[1] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_STATUS0, (char)0x4) ;
			dump_reg[2] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_STATUS1, (char)0x4) ;
			dump_reg[3] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_ERRDETAIL0, (char)0x4) ;
			
			hfc_logout(ap,err_no,HFC_ERRLOG_TYPE_MCK);
			HFC_MEMCPY(logdata,(uchar*)dump_reg,16);
			hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERRD,err_no,logdata,16) ;
			
			if (hfc_manage_info.npubp->hfc_mp_hsd_enable)						/* FCLNX-0429 */
				hfc_manage_info.npubp->hfc_make_fcinfo(ap, NULL, err_no, adap_status, NULL, 0);
			
			/* Counter Up      */
			ap->pci_err_cnt++ ;
			if( ap->pci_err_cnt > HFC_PCIERR_CNT )
			{
				clear_bit(HFC_ONLINE, (ulong *)&ap->status );
				HFC_ISSUE_CSTP(ap, FALSE, HFC_ABEND_FCSTP );		/* FCLNX-GPL-527 */
				if(test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status )) 
				{
					hfc_wake_up(&ap->mb_event,&ap->mb_event_wait);
				}
				
				if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) || (ap->initialize == 1) )
				{
					hfc_watchdog_enter( ap , NULL, NULL, 0, HFC_LOGIN_DELAY_TMR, 0, 1);			/* FCLNX-0243 */
					clear_bit(HFC_LOGIN_DELAY, (ulong *)&ap->status ); 							/* FCLNX-0243 */
					unlock_mailbox( ap ) ; /*goto*/
					hfc_wake_up(&ap->init_event,&ap->int_a_poll);
					clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);		
				}
			}
			else                                                                    /* FCWIN-0286 */
			{
				HFC_ISSUE_FMCK(ap, type);										/* FCLNX-0279 */
			}
			break ;
			
		case HFC_ABEND_MCK_INT:
		case HFC_ABEND_MPCHK:
		case HFC_ABEND_T3:
			/* (1) MCK INT										*/
			/*==> Processing <==								*/
			/* Machine check recovery and increment counter 	*/
			/* Cancel pending SCSI commands under this adapter	*/

			HFC_MAILBOX_UNLOCK( ap, HFC_MAILBOX_BUSY);	
			hfc_reset_watchdog(ap);										/* FCLNX-GPL-038 */

			for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)						
			{
				target = hfc_hash_target_info(ap, lp);
				if (target != NULL)
				{
					hfc_cancel_scsi_cmd(
						ap, target, 0, NULL, SCS_MCK, HFC_CSCSI_ERROR,
						TRUE, TRUE, HFC_FLASH_TARGET);
					target->status = HFC_NON_STATUS ;
					
					for (lun=0;lun<MAX_DEV_CNT;lun++)
						target->lustat[lun] = 0x00;
					
					if ( hfc_manage_info.hfcldd_mp_mod ) {
						hfc_manage_info.npubp->hfc_forced_offline_e(target, TRUE);
					}
				}
			}
			
			for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)											/* FCLNX-GPL-038 */
			{
				target = hfc_pseq_target_info(ap, lp);
				if (target != NULL)
				{
					/* Release all target_info */
					if( test_bit( HFC_WAIT_LOGIN, (ulong *)&target->status ) 
					  || test_bit( HFC_WAIT_CANCEL, (ulong *)&target->status ) ){	
						clear_bit( HFC_WAIT_LOGIN, (ulong *)&target->status );
						clear_bit( HFC_WAIT_CANCEL, (ulong *)&target->status );
						if (test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags)) {
							rsp = hfc_clear_target_info( ap, target, TRUE, TRUE );
						}
					}
					else if( test_bit(HFC_TARGETINF_VALID, (ulong *)&target->flags) &&		/* FCLNX-GPL-0052 */
					  !test_bit(HFC_DEVFLG_VALID, (ulong *)&target->flags) ){
						clear_bit( HFC_NEED_LOGIN, (ulong *)&target->status );
						clear_bit( HFC_NEED_CANCEL, (ulong *)&target->status );
						rsp = hfc_clear_target_info( ap, target, TRUE, TRUE );				/* FCLNX-GPL-0052 */
					}
				}
			}																				/* FCLNX-GPL-038 */
			
			
			hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )0x00000000 );	/* FCLNX-0275 */

			if ( ap->pkg.type == HFC_PKTYPE_FPP )
			{
				if( hfc_pcibus_chk(ap) == 0 )
				{
					hfc_mck_recovery_fpp(ap,type);
				}
			}
			else if ( ap->pkg.type == HFC_PKTYPE_FIVE )
			{
				hfc_mck_recovery_five(ap,type);
			}

			dump_reg[0] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_INTA, (char)0x4) ;
			dump_reg[1] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_STATUS0, (char)0x4) ;
			dump_reg[2] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_STATUS1, (char)0x4) ;
			dump_reg[3] = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_ERRDETAIL0, (char)0x4) ;
			
			if( type == HFC_ABEND_MPCHK )
				err_no = 0x0000002c ;
			else if( type == HFC_ABEND_T3 )
				err_no = 0x0000002d ;
			else
				err_no = 0x0000002b ;
	
			if( test_bit( HFC_DIAG_PROGRESS, (ulong *)&mpap->status ) )
			{
				/*--- DIAG Mode ---*/
				HFC_MEMCPY(logdata,(uchar*)dump_reg,16);
				hfc_logout(ap,err_no,HFC_ERRLOG_TYPE_MCK);
				if( type == HFC_ABEND_MCK_INT )
					hfc_errlog(
						ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR2,
						err_no,logdata,16) ;
				else
					hfc_errlog(
						ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR4,
						err_no,logdata,16) ;			
			}
			
			if (hfc_manage_info.npubp->hfc_mp_hsd_enable)						/* FCLNX-0429 */
				hfc_manage_info.npubp->hfc_make_fcinfo(ap, NULL, err_no, SCS_MCK, NULL, 0);
			
			break ;
		case HFC_ABEND_MB_TOUT:
		case HFC_ABEND_TOUTCHK_XOB:
		case HFC_ABEND_RID_INVALID:
			/* (1) Mail Box timeout */
			/*==> Process <==                                    */
			/*   -> Execute force check stop                     */
			HFC_ISSUE_FMCK(ap, type);										/* FCLNX-0279 */
			break ;
			
		default:
			HFC_ISSUE_FMCK(ap, type);										/* FCLNX-0279 */
			break ;
	}
	
	hfc_hand2_trace(HFC_TRC_ABEND, 0x10, ap, NULL, NULL, 0, 0, 0);
	return;
}


/*
 * Function:    hfc_pcibus_chk
 *
 * Purpose:     Check the status of PCI bus
 *
 * Arguments:   
 *  ap         -
 *
 * Returns:     
 *  0          - OK
 *  -1         - NG
 *
 * Notes:       
 */
int hfc_pcibus_chk(struct adap_info *ap)
{
	uint			read_reg = 0 ;

	HFC_DBGPRT(" hfcldd : hfc_pcibus_chk - start\n");

	
	read_reg = hfc_read_reg_ext( ap,(uint)0, (char)0x4) ;
	
	if( read_reg == 0xffffffff )
	{
		hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_EVNT4,
													0x5c,NULL,0) ;
		return(-1) ;
	}
	return(0) ;
}


/*
 * Function:    hfc_mck_recovery_fpp
 *
 * Purpose:     Machine check recovery process for FPP
 *
 * Arguments:   
 *  ap         - 
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mck_recovery_fpp(struct adap_info *ap,uchar type)
{
	uint				dump_reg[4] ;
	uint				err_no ;
	struct mp_adap_info		*mpap;

	HFC_DBGPRT(" hfcldd : hfc_mck_recovery - start\n");

	hfc_hand2_trace(HFC_TRC_MCKREC,0x10,
					ap,NULL,NULL,0,(uint64_t)type,0);
	
	mpap = ap->mp_adap_info;
	
	ap->mck_type = type;

	if( test_bit(HFC_DIAG_DELAY, (ulong *)&ap->status ) )
	{
		hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);					/* FCLNX-0269 */
		clear_bit(HFC_DIAG_DELAY, (ulong *)&ap->status );
	}

	if( test_bit( HFC_DIAG_PROGRESS, (ulong *)&mpap->status ) )
	{
		/*--- DIAG Mode ---*/
		if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status) )
		{
			hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
		}
		clear_bit(HFC_ONLINE, (ulong *)&ap->status);
		return ;
	}
	if( test_bit(HFC_CHK_STOP, (ulong *)&ap->status ) )
	{
		return ;
	}
	mpap->mck_err_cnt++ ;	
	if( test_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status ) )
	{
		/* Is MCK recovery executing */
		hfc_chk_stop( ap, FALSE ) ;								/* FCLNX-0276 */
	}
	else if( (ap->max_mck_cnt) && (mpap->mck_err_cnt >= ap->max_mck_cnt) )
	{
		/* MCK occured more than specified count by user (ap->max_mck_cnt) */
		/*  ( Do not force HBA checkstop if ap->max_mck_cnt is zero        */
		hfc_chk_stop( ap, FALSE ) ;								/* FCLNX-0276 */
	}
	else
	{
			/* MCK recovery flag ON */
			dump_reg[0] = hfc_read_reg_ext( ap, (uint)HFC_IOSPACE_INTA, (char)0x4) ;
			dump_reg[1] = hfc_read_reg_ext( ap, (uint)HFC_IOSPACE_STATUS0, (char)0x4) ;
			dump_reg[2] = hfc_read_reg_ext( ap, (uint)HFC_IOSPACE_STATUS1, (char)0x4) ;
			dump_reg[3] = hfc_read_reg_ext( ap, (uint)HFC_IOSPACE_ERRDETAIL0, (char)0x4) ;
			if( type == HFC_ABEND_MPCHK )
				err_no = 0x0000002c ;
			else if( type == HFC_ABEND_T3 )
				err_no = 0x0000002d ;
			else
				err_no = 0x0000002b ;
			memcpy(logdata,(uchar*)dump_reg,16);
			hfc_fpp_logout(ap,err_no,HFC_ERRLOG_TYPE_MCK);
			if( type == HFC_ABEND_MCK_INT )
				hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR2,
															err_no,logdata,16) ;
			else
				hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR4,
														err_no,logdata,16) ;
			if( dump_reg[1] & 0x00800000 )
			{
				/* 
				 * Reset device status in config register (0xffff write) 
				 * if PCI ERR DETECTED in status register equals 1.
				 */
				
				hfc_write_cnfg(ap,6,0x2,0xffff);
			}

		set_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status );
		hfc_reset_start(ap,HFC_INI_RESET);
		hfc_reset_adap_info(ap) ;

		/* WS Comu Area(0x318) */
		hfc_reset_start(ap,HFC_WSCA_CLEAR);
		hfc_reset_start(ap,HFC_SET_WS80);			/* WS Comu Area(0x318) */
		hfc_reset_start(ap,HFC_SET_INIADR);			/* INITTBL ADR(0x310)  */
													/* ALPA(0x319)         */

		hfc_w_stop(ap,HFC_REBOOT_DELAY_TMR) ;
		hfc_w_start(ap,HFC_REBOOT_DELAY_TMR) ;	

		hfc_reset_start(ap,HFC_REBOOT);
		hfc_hand2_trace(HFC_TRC_MCKREC,0x11,
					ap,NULL,NULL, 0,(uint64_t)type,0);
	}
}


/*
 * Function:    hfc_mck_recovery_five
 *
 * Purpose:     Machine check recovery(for Five)
 *
 * Arguments:   
 *  ap         - Pointer to dap_info
 *  type       - error type
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mck_recovery_five(                                                         /* FCWIN-0220 */
	struct adap_info            *ap,
	uchar                        type)
{
	struct mp_adap_info     *mpap;
	uint                    dump_reg[4], wk4 ;
	uint                    status0;
	uint                    err_no ;
	int						rc ;
	
	mpap = ap->mp_adap_info;
	
	hfc_hand2_trace(HFC_TRC_MCKREC,0x20,
					ap,NULL,NULL,0,(uint64_t)type,0);
	
	ap->mck_type = type;

	rc = lock_try_mpap( mpap );												/* FCLNX-0276 */
	
	if( rc == 0 ){		/* Lock failed */
		set_bit( HFC_LOCK_WAIT_1, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
		hfc_w_start( ap, HFC_MPAP_LOCK_TMR );
		return;
	}

																			/* FCLNX-0287 */
	hfc_w_stop(ap,HFC_MCKINT_TMR) ;

	if( test_bit( HFC_LOCK_WAIT_1, (ulong *)&ap->mpap_lock ) )
	{
		/* Stop retry timer */
		clear_bit( HFC_LOCK_WAIT_1, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );

		hfc_hand2_trace(HFC_TRC_MCKREC,0x22,
				ap,NULL,NULL,0,(uint64_t)type,0);
	}																		/* FCLNX-0276 */
	
#if _HFC_ERROR_INJ_00
	set_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
	set_bit(HFC_MCK_HW_INIT, (ulong *)&ap->mck_result );
	set_bit(HFC_HWCHKSTOP, (ulong *)&mpap->status );
	clear_bit(HFC_ONLINE, (ulong *)&ap->status);
	
	if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status) )
	{
		hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
	}
	if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) || (ap->initialize == 1) )
	{
		unlock_mailbox( ap ) ; /*goto*/
		hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);		/*goto*/
	}
	HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY); 
	return ;
#endif

	if( test_bit(HFC_DIAG_DELAY, (ulong *)&ap->status ) )
	{
		hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);					/* FCLNX-0296 */
		clear_bit(HFC_DIAG_DELAY, (ulong *)&ap->status );
	}

	if( test_bit( HFC_DIAG_PROGRESS, (ulong *)&mpap->status ) )
	{
		/*--- DIAG Mode ---*/
		set_bit(HFC_MCK_HW_INIT, (ulong *)&ap->mck_result );
		set_bit(HFC_HWCHKSTOP, (ulong *)&mpap->status );
		clear_bit(HFC_ONLINE, (ulong *)&ap->status);
		/* Timer stop */
		if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status) )
		{
			hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
		}
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY); 
		return ;
	}
	if ( test_bit(HFC_HWCHKSTOP, (ulong *)&mpap->status ) )
	{
		set_bit(HFC_MCK_HWCHKSTOP, (ulong *)&ap->mck_result );

		if(test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status )) 
		{
			hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
		}
		
		if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) || (ap->initialize == 1) )
		{
			unlock_mailbox( ap ) ; 
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
			clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);		
		}
		
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY); 
		return ;
	}
	
	
	wk4 = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_INTA, (char)0x4) ;		/* FCWIN-0197 STR */
	HFC_4L_TO_4B(dump_reg[0],wk4);
	wk4 = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_STATUS0, (char)0x4) ;
	HFC_4L_TO_4B(dump_reg[1],wk4);
	wk4 = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_STATUS1, (char)0x4) ;
	HFC_4L_TO_4B(dump_reg[2],wk4);
	wk4 = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_ERRDETAIL0, (char)0x4) ;
	HFC_4L_TO_4B(dump_reg[3],wk4);											/* FCWIN-0197 END */
	
	if( type == HFC_ABEND_MPCHK )
		err_no = 0x0000002c ;
	else if( type == HFC_ABEND_T3 )
		err_no = 0x0000002d ;
	else
		err_no = 0x0000002b ;
	
	HFC_MEMCPY(logdata,(uchar*)dump_reg,16);
	
	if( ( test_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status ) ) || ( test_bit(HFC_SUBMCK_RECOVERY, (ulong *)&mpap->status ) ) ) 
	{	/* FCLNX-0494	*/
		/* Did MCK occur eight times or more? */
		HFC_ISSUE_CSTP( ap , TRUE, HFC_ABEND_FCSTP ) ;						/* FCLNX-0276 *//* @MLPF *//* FCLNX-GPL-527 */
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);								/* FCLNX-0276 */

		if(test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status )) 
		{
			hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
		}
		
		if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) || (ap->initialize == 1) )
		{
			unlock_mailbox( ap ) ;
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
			clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);		
		}
		return;
	}
	

	if( dump_reg[1] & 0x00800000 )
	{
		/* 
		 * Reset device status in config register (0xffff write) 
		 * if PCI ERR DETECTED in status register equals 1.
		 */
		hfc_write_cnfg(ap, 0x06, 0x2, 0xffff);
		hfc_write_cnfg(ap, 0x6c, 0x4, 0xffffffff);
		hfc_write_cnfg(ap, 0x70, 0x4, 0x0fffffff);
	}
	
	status0 = (uint)hfc_read_reg( ap, (uint)HFC_IOSPACE_STATUS0, (char)0x4) ;
	if (!(status0 & 0x00400000 )
	 || test_bit(HFC_HMCK_RECOVRTY, (ulong *)&mpap->lock) )				/* FCLNX-0279 */
	{
		/* Other port is executing Machine Check recovery process */
		set_bit(HFC_SUBMCK_RECOVERY, (ulong *)&mpap->status );			/* FCLNX-0493	*/
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY); 
		hfc_reset_adap_info(ap) ;
		set_bit(HFC_MCK_END, (ulong *)&ap->mck_result );
		set_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status );

		hfc_hand2_trace(HFC_TRC_MCKREC,0x25, ap,NULL,NULL,(uint64_t)status0, mpap->lock, 0);	/* FCLNX-0287 */

		hfc_w_stop(ap,HFC_MCK_DELAY_TMR) ;
		hfc_w_start(ap,HFC_MCK_DELAY_TMR) ;
		return ;
	}
	/* FCLNX-GPL-057 */
	if ( !((hfc_manage_info.hfcplus_enable ) && ( ap->isol_detail != HFC_NO_ISOLATE )) )
	{
		mpap->mck_err_cnt++ ;	
	}/* FCLNX-GPL-057 */
	
	if( (ap->max_mck_cnt) && (mpap->mck_err_cnt >= ap->max_mck_cnt) )	/* FCLNX-GPL-057 */
	{
		/* MCK occured eight times or more */

		HFC_ISSUE_CSTP( ap, TRUE, HFC_ABEND_FCSTP ) ;					/* FCLNX-0276 *//* FCLNX-GPL-527 */
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);							/* FCLNX-0276 */
		set_bit(HFC_MCK_CNT_OVER, (ulong *)&ap->mck_result );
		if(test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status )) 
		{
			hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
		}
		
		if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) || (ap->initialize == 1) )
		{
			unlock_mailbox( ap ) ; 
			hfc_wake_up(&ap->init_event,&ap->int_a_poll);
			clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);		
		}
		return;
	}
	
	set_bit( HFC_HMCK_RECOVRTY, (ulong *)&mpap->lock );
	
	set_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
	
	HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
	
	hfc_five_logout(ap, err_no, HFC_ERRLOG_TYPE_MCK);
	
	if( type == HFC_ABEND_MCK_INT )
	{
		hfc_errlog(
			ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR2,
			err_no,logdata,16) ;
	}
	else{
		hfc_errlog(
			ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR4,
			err_no,logdata,16) ;
	}
	
	hfc_watchdog_enter(ap, NULL, NULL, 0, HFC_CTLRST_DELAY_TMR, 0, TRUE);	/* FCLNX-0276 */
	hfc_watchdog_enter(ap, NULL, NULL, 0, HFC_CTLRST_DELAY_TMR, 1, FALSE);	/* FCLNX-0276 */

	return;
}


/*
 * Function:    hfc_mck_port_recovery
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - 
 *
 * Returns:     
 *
 * Notes:       
 */
int hfc_mck_port_recovery(struct adap_info	*ap)
{
	struct mp_adap_info		*mpap;
	
	
	HFC_DBGPRT(" hfcldd : hfc_port_recovery - start\n");
	mpap = ap->mp_adap_info;

	if( ( test_bit(HFC_MCK_SUCCESS,   (ulong *)&ap->mck_result) ) ||
		( test_bit(HFC_ISOL_RECOVERY, (ulong *)&ap->status  )   ) )
		{
		/* Mck process succeeded */
		hfc_reset_start(ap, HFC_FW_START);                      /* FCWIN-0237 */
		clear_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
		
		//@MLPF
		if( ! ( HFC_MMODE_CHECK_SHARED(ap) ) )
			set_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status);
		else if (HFC_MMODE_CHECK_SHADOW (ap) )
			set_bit(HFC_MLPF_WAIT_MCKEND, (ulong *)&ap->status);
		
		if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
			if(ap->mck_on_sleep) {
				hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);				/* FCLNX-0296 */
			}
		} 
		/* Interruption mask setting */
		//@MLPF
		if( HFC_MMODE_CHECK_SHARED(ap) )
		{
			hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_INTA_MASK_MLPF);
		}
		else
			hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_INTA_MASK);

		/* Link Up Timer Set */
		//@MLPF
		if( ! ( HFC_MMODE_CHECK_SHARED(ap) ) )
		{
			hfc_w_stop( ap, HFC_LINKUP2_TMR );						/* FCLNX-0241 */
			hfc_w_start( ap, HFC_LINKUP2_TMR );						/* FCLNX-0241 */
		}
		else if (HFC_MMODE_CHECK_SHADOW (ap) )
		{
			hfc_w_stop( ap, HFC_MLPF_MCKEND_TMR );
			hfc_w_start( ap, HFC_MLPF_MCKEND_TMR );
		}
	}
	else if( test_bit(HFC_MCK_HW_INIT, (ulong *)&ap->mck_result) )
	{
		/* HW initialization is in progress */
		clear_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
		if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
			if(ap->mck_on_sleep) {
				hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);				/* FCLNX-0296 */
			}
		} 
		/* Set interruption mask */
		set_bit( HFC_WAIT_LINKUP, (ulong *)&ap->status );
	}
	else if( test_bit(HFC_MCK_HWCHKSTOP, (ulong *)&ap->mck_result) )
	{
		clear_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
		if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
			if(ap->mck_on_sleep) {
				hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);				/* FCLNX-0296 */
			}
		} 
		/* Set interruption mask */
		clear_bit( HFC_ONLINE, (ulong *)&ap->status );
	}
	else if( test_bit(HFC_MCK_CNT_OVER, (ulong *)&ap->mck_result) )
	{
		clear_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
		if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
			if(ap->mck_on_sleep) {
				hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);				/* FCLNX-0296 */
			}
		} 
		/* Set interruption mask */
		clear_bit( HFC_ONLINE, (ulong *)&ap->status );
	}
	else if( test_bit(HFC_MCK_END, (ulong *)&ap->mck_result)||test_bit(HFC_MCK_EXEC, (ulong *)&ap->mck_result) )
	{
		hfc_reset_adap_info(ap);
		/* WS Comu Area(0x318) */
		hfc_reset_start(ap, HFC_SET_WS80);
		/* INITTBL ADR(0x310)  */
		/* ALPA(0x319)         */
		hfc_reset_start(ap, HFC_SET_INIADR);
		hfc_reset_start(ap, HFC_FW_START);                      /* FCWIN-0237 */
		clear_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
		//@MLPF
		if( ! ( HFC_MMODE_CHECK_SHARED(ap) ) )
			set_bit( HFC_WAIT_LINKUP, (ulong *)&ap->status );
		else if (HFC_MMODE_CHECK_SHADOW (ap) )
			set_bit(HFC_MLPF_WAIT_MCKEND, (ulong *)&ap->status);
		/* Set interruption mask */
		//@MLPF
		if( HFC_MMODE_CHECK_SHARED(ap) )
		{
			hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_INTA_MASK_MLPF);
		}
		else
			hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_INTA_MASK);

		/* Link Up Timer Set */
		//@MLPF
		if( ! ( HFC_MMODE_CHECK_SHARED(ap) ) )
		{
			hfc_w_stop( ap, HFC_LINKUP2_TMR );						/* FCLNX-0241 */
			hfc_w_start( ap, HFC_LINKUP2_TMR );						/* FCLNX-0241 */
		}
		else if (HFC_MMODE_CHECK_SHADOW (ap) )
		{
			hfc_w_stop( ap, HFC_MLPF_MCKEND_TMR );
			hfc_w_start( ap, HFC_MLPF_MCKEND_TMR );
		}
	}
	
	unlock_mailbox(ap);
	
	return TRUE;
}


/*
 * Function:    hfc_chk_stop
 *
 * Purpose:     Check stop processing
 *
 * Arguments:   
 *  ap         - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_chk_stop(struct adap_info *ap, uchar mpap_lock)
{
	struct mp_adap_info         *mpap;
	struct adap_info            *wk_ap;             /* FCWIN-0220 */
	int                         i, lp, rc;          /* FCWIN-0220 */
	struct target_info			*target;


	HFC_DBGPRT(" hfcldd : hfc_chk_stop - start\n");

	hfc_hand2_trace(HFC_TRC_MCKREC,0x40,
					ap,NULL,NULL,0,0,0);
	
	mpap = ap->mp_adap_info;
	HFC_DBGPRT(" hfcldd%d : hfc_chk_stop \n",ap -> dev_minor);

	if ( mpap_lock == FALSE ) {										/* FCLNX-0279 */
		rc = lock_try_mpap( mpap );

		if( rc == 0 ){
			set_bit( HFC_LOCK_WAIT_2, (ulong *)&ap->mpap_lock );
			hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
			hfc_w_start( ap, HFC_MPAP_LOCK_TMR );
			return;
		}
	}

	if( test_bit( HFC_LOCK_WAIT_2, (ulong *)&ap->mpap_lock ) )
	{
		/* Stop retry timer */
		clear_bit( HFC_LOCK_WAIT_2, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
	}																/* FCLNX-0279 */

	set_bit(HFC_HWCHKSTOP, (ulong *)&mpap->status ) ;

	if( hfc_manage_info.hfcplus_enable ){
		set_bit(HFC_HWISOL,(ulong *)&mpap->status);					/* FCLNX-0506 */
	}

	if ( mpap_lock == FALSE ) {										/* FCLNX-0279 */
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
	}																/* FCLNX-0279 */

	hfc_logout(ap,0x00000031,HFC_ERRLOG_TYPE_CHKSTP);
	hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_CHKSTP,ERRID_HFCP_ERR1,0x31,logdata,16) ;
													
	if (hfc_manage_info.npubp->hfc_mp_hsd_enable)					/* FCLNX-0429 */
		hfc_manage_info.npubp->hfc_make_fcinfo(ap, NULL, 0x31, SCS_MCK, NULL, 0);

	if ( ap->pkg.type == HFC_PKTYPE_FPP)
	{
		clear_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status);
		if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
			if(ap->mck_on_sleep) {
				hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);				/* FCLNX-0296 */
			}
		} 
		/* Set interruption mask */
		set_bit(HFC_CHK_STOP, (ulong *)&ap->status );
		clear_bit(HFC_ONLINE, (ulong *)&ap->status );
		hfc_reset_start(ap,HFC_INI_RESET);
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDFCIF,( char )0x1,(char)0x80);
		hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )0x00000000 );
		/*-- 1.67 LED 0x44 dummy write(H/W Bug) --*/
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDLED,( char )0x1,(char)0x44);
		hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDLED,( char )0x1,HFC_WAKE_UP_FAILURE);
	}
	else
	{
		/* Inhibit interruption in all ports */	
		if(hfc_manage_info.hfcplus_enable){
			hfc_w_stop(ap,HFC_LDLERR_TMR);	/*FCLNX-0506*/
			hfc_w_stop(ap,HFC_LDSERR_TMR);	/*FCLNX-0506*/
			hfc_w_stop(ap,HFC_IFERR_TMR);	/*FCLNX-0506*/
			hfc_w_stop(ap,HFC_TOERR_TMR);	/*FCLNX-0506*/
		}		
		wk_ap = ap->mp_adap_info->ap;
		
		for( i=0 ; i<4 ; i++)
		{
			if (wk_ap == NULL)
				break;
			else
			{
				for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)
				{
					target = hfc_hash_target_info(wk_ap, lp);
					if( target != NULL )
					{
						hfc_cancel_scsi_cmd(wk_ap,target,0, NULL, SCS_MCK, HFC_CSCSI_ERROR,	/* FCLNX-0429 */
								TRUE, TRUE, HFC_FLASH_TARGET);
						target->status = HFC_NON_STATUS;
						
						if ( hfc_manage_info.hfcldd_mp_mod ) {
							hfc_manage_info.npubp->hfc_forced_offline_e(target, TRUE);
						}
					}
				}
				
				clear_bit(HFC_MCK_RECOVERY, (ulong *)&wk_ap->status);
				if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&wk_ap->status)){
					if(ap->mck_on_sleep) {
						hfc_wake_up(&wk_ap->mck_event, &wk_ap->mck_event_wait);					/* FCLNX-0296 */
					}
				} 
		
				set_bit(HFC_CHK_STOP, (ulong *)&wk_ap->status );
				set_bit(HFC_ISOL,(ulong *)&wk_ap->status );	/* FCLNX-707 */
				wk_ap->isol_detail = HFC_ISOLATE_CHKSTP;		/* FCLNX-707 */

				hfc_reset_start(wk_ap, HFC_CTLRST);
				udelay(1000);
				/* Stop optical transmission */
				hfc_write_reg(wk_ap,( uint )HFC_IOSPACE_CMDFCIF,( char )0x4,(char)0x80808080);
				/* Turn LED (Yellow and Green) off */
				hfc_write_reg(wk_ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x4,HFC_WAKE_UP_FAILURE_FIVE);
				
				hfc_write_reg(wk_ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )0x00000000 );
				clear_bit(HFC_ONLINE, (ulong *)&wk_ap->status );
				
				if( HFC_MMODE_CHECK_SHADOW(wk_ap) )
				{
					HFC_ERRPRT("hfcldd%d : shared FC driver enable status off by HW error.\n",wk_ap->dev_minor);
					hfc_mlpf_change_state(wk_ap, HFC_HG_HYPSTATUS_ENABLE, HFC_DISABLE_HYPER_STATE );   /* FCLNX-0385 */
					hfc_write_hg_reg(wk_ap, HFC_IOHGSPC_CMNDREG, 4, HFC_MLPF_CSTPEND);     //@MLPF
				}
			}
			wk_ap = wk_ap->next;
			
		}
	}
	hfc_hand2_trace(HFC_TRC_MCKREC,0x21,ap,NULL,NULL,0,0,0);

	
	return;
}


/*
 * Function:    hfc_logout
 *
 * Purpose:     LOG OUT processing
 *
 * Arguments:   
 *  ap         -
 *  err_no     -
 *  mode       -
 *
 * Returns:     
 *
 * Notes:       
 */
 void hfc_logout(                          /* FCWIN-0220 STR */
	struct adap_info        *ap,
	uint                    err_no,
	uchar                   mode)
{
	if ( ap->pkg.type == HFC_PKTYPE_FPP )
		hfc_fpp_logout(ap, err_no, mode);
	else
		hfc_five_logout(ap, err_no, mode);
}


/*
 * Function:    hfc_watchdog
 *
 * Purpose:     Watchdog timer processing
 *
 * Arguments:   
 *  w_timer    -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_watchdog(struct wtimer *w_timer)
{
	struct adap_info *ap=NULL;
	struct target_info *target=NULL ;
	struct hfc_pkt *hfcp=NULL,*wx_hfcp=NULL ;
	struct mp_adap_info		*mpap;
	unsigned long	flags = 0;													/* FCLNX-0274 */
	int		lp;
	uint	xob_no;

	int		lun=0 ;
	uint	timeout;
	uint	int_a_rst;
	int 	rc=0 ;
	
	struct dev_info *dev=NULL;													/* FCLNX-GPL-038 */
	
	HFC_DBGPRT(" hfcldd : hfc_watchdog - start id=%x\n",w_timer->timer_id);

	if (hfc_manage_info.adap_info_arg[w_timer->ap_dev_minor] == NULL)			/* FCLNX-0322 */
		return;																	/* FCLNX-0322 */
	
	ap = w_timer->ap ;

	target = w_timer->target;
	hfcp = w_timer->hfcpk;
	if(hfcp != NULL)
	{
		lun = hfcp->lun_id;
	}
	dev = w_timer->dev;															/* FCLNX-GPL-047 */

	if(ap == NULL) return;
	
    HFC_ADAPLOCK_IRQSAVE(flags);

	hfc_hand2_trace(HFC_TRC_WDOG,0x00,
					ap,target,hfcp,0,
					(uint64_t)w_timer->timer_id,
					0);
					
	w_timer->timer_flag &= ~HFC_TIMER_VALID;
	mpap = ap->mp_adap_info;
	
	switch(w_timer->timer_id)
	{
		case HFC_DELAY_TMR:			/* DELAY timer after executing TARGET RESET		*/
			if(target == NULL) return;												/* FCLNX-GPL-038 */
			clear_bit( HFC_SCSI_DELAY, (ulong *)&target->status );

			if( (test_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status) ) && (test_bit(HFC_ONLINE, (ulong *)&ap->status) ) )
			{
				/* If adapter is waiting LINK_UP, I/O start can not be executed. */
				break;																/* FCLNX-GPL-038 */
			}
			else if( !(test_bit(HFC_ONLINE, (ulong *)&ap->status) ) )
			{
				/* 
				 * If adapter is not online, target should have been already canceled.
				 * Break this process 
				 */
				break ;																/* FCLNX-GPL-038 */
			}
			else if( !(test_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status) )  && (test_bit(HFC_ONLINE, (ulong *)&ap->status) ) )
			{
				/* Restart waiting request if adapter is ONLINE */
				if(!(test_bit(HFC_SCN_WLINKUP, (ulong *)&target->status) )  
				&& test_bit(HFC_NEED_LOGIN, (ulong *)&target->status)  )			/* FCLNX-GPL-038 */
				{
					clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status );
					if( hfc_issue_relogin(ap, target) )
					{
						/* Initiated LOGIN but failed with mailbox busy */
						set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );			/* FCLNX-GPL-038 */
						clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status );		/* FCLNX-GPL-038 */
						hfc_enque_login_req(ap, target);
					}
					break;
				}
				else if( test_bit(HFC_NEED_CANCEL, (ulong *)&target->status) )  	/* FCLNX-GPL-038 */
				{
					clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status );
					if( hfc_issue_relogin(ap, target) )
					{
						/* Initiated CANCEL LOGIN but failed with mailbox busy */
						set_bit(HFC_NEED_CANCEL, (ulong *)&target->status );		/* FCLNX-GPL-038 */
						clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status );		/* FCLNX-GPL-038 */
						hfc_enque_login_req(ap, target);
					}
					break;
				}																	/* FCLNX-GPL-038 */
				if( ( target->wx_que_cnt > 0 ) || ( ap->xob_wait_exec_cnt > 0 )){					/*-- FCLNX-019 STR--*/
					hfc_issue_intl_start(ap,target) ;
				}												/*-- FCLNX-019 END--*/
			}
			break ;
			
		case HFC_SCSI_CMD_TMR:		/* SCSI Command(Normal) Time-Out	*/
			if(target == NULL) return;
			if( ( target->we_que_cnt == 0 )&&( target->wx_que_cnt == 0 ) )
				break ;
			if( hfcp == NULL ) break;
			
			if((hfc_manage_info.hfcplus_enable)&&(ap->to_errcnt_info!=NULL)){                              /* FCLNX-0506 */
				hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_OCCURED_FAILURE, HFC_TO_ERR);  /* FCLNX-0506 */
			}
				
			set_bit(CFLAG_TIMEOUT, (ulong *)&hfcp->cmd_flags );

			if( hfc_pcibus_chk(ap) == 0 )
			{
				wx_hfcp = target->wx_que_top;
				
				while( wx_hfcp != NULL )
				{
					if( wx_hfcp == hfcp )
						break;
					
					wx_hfcp = wx_hfcp->cmd_forw;
				}
				
				if( wx_hfcp != NULL )
				{
					if( (target->status == HFC_NON_STATUS) && !(test_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status)) )
					{
						timeout = (uint)(wx_hfcp->cmd_pkt->timeout_per_command/HZ);
					}
					hfc_deque_wx_que(target, wx_hfcp);
					hfc_iodone(ap, wx_hfcp->cmd_pkt, wx_hfcp);	/* FCLNX-0429 */
					break ;
				}
				
				if( hfc_toutchk_xob(ap, target, hfcp, lun, HFC_ISSUE_ABORT) )
				{
					/* 
					 * Timed-out xob remain in queue so do not initiate
					 * MIH-LOG and Abort-Task-Set 
					 */
					break ;
				}

				/* MIH-LOG Request */
				/* Issue Abort_Task_Set after collecting MIH-LOG in case SCSI command is timeout */
			    if( hfc_issue_mihlog(ap, target, hfcp) )
			    {
					/* Failed to initiate mailbox request */
					/* Execute Abort without collecting MIH-LOG */
					if(ap->c_err!=0) break;
					if(!ap->abort_t_restrain){	/* FCLNX-0506 */
						if( test_bit(HFC_NEED_LOGIN, (ulong *)&target->status )  )		/* FCLNX-GPL-038 */
						{
							if( hfc_issue_relogin(ap,target) )
							{
								set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );		/* FCLNX-GPL-038 */
								clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */
								hfc_enque_login_req(ap, target);
							}
							break ;
						}
						else if( test_bit(HFC_NEED_CANCEL, (ulong *)&target->status ) )	/* FCLNX-GPL-038 */
						{
							if( hfc_issue_relogin(ap,target) )
							{
								set_bit(HFC_NEED_CANCEL, (ulong *)&target->status );	/* FCLNX-GPL-038 */
								clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status );	/* FCLNX-GPL-038 */
								hfc_enque_login_req(ap, target);
							}
							break ;
						}
					
						if( (test_bit(HFC_WAIT_TARGET_RESET, (ulong *)&target->status ) ) ||
							(test_bit(HFC_WAIT_LOGIN, (ulong *)&target->status ) ) ||
							(test_bit(HFC_WAIT_CANCEL, (ulong *)&target->status ) ) ||	/* FCLNX-GPL-038 */
							(target->lustat[lun] & HFC_WAIT_ABORT) )
							break ;                                            
						hfc_issue_task_mgm(ap, target, hfcp, lun, HFC_ISSUE_ABORT) ;
					/* If initiation succeeds, set HFC_WAIT_HALT in hfc_isuue_task_mgm() */
					}
					else{
						if(ap->login_restrain)	/* FCLNX-0506 */
						{
							set_bit(HFC_NEED_CANCEL, (ulong *)&target->status);
							clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status);		/* FCLNX-GPL-038 */
						}
						if( test_bit(HFC_NEED_LOGIN, (ulong *)&target->status )  )		/* FCLNX-GPL-038 */
						{
							if( hfc_issue_relogin(ap,target) )
							{
								set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );		/* FCLNX-GPL-038 */
								clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */
								hfc_enque_login_req(ap, target);
							}
							break ;
						}
						else if( test_bit(HFC_NEED_CANCEL, (ulong *)&target->status ) )	/* FCLNX-GPL-038 */
						{
							if( hfc_issue_relogin(ap,target) )
							{
								set_bit(HFC_NEED_CANCEL, (ulong *)&target->status );	/* FCLNX-GPL-038 */
								clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status );	/* FCLNX-GPL-038 */
								hfc_enque_login_req(ap, target);
							}
							break ;
						}
						if( (test_bit(HFC_WAIT_TARGET_RESET, (ulong *)&target->status ) ) ||
							(test_bit(HFC_WAIT_LOGIN, (ulong *)&target->status ) ) ||
							(test_bit(HFC_WAIT_CANCEL, (ulong *)&target->status ) ) ||	/* FCLNX-GPL-337 *//* FCLNX-GPL-534 */
							(target->lustat[lun] & HFC_WAIT_ABORT) )
							break ;                                            
						hfc_issue_task_mgm(ap, target, hfcp, lun, HFC_ISSUE_LOGIN) ;	/* FCLNX-0500 */
					}
				}	
				if(ap->c_err!=0) break; /* FCLNX-0597 */
				if(ap->abort_t_restrain){
					set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );
					clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status );				/* FCLNX-GPL-038 */
				 	if(ap->login_restrain){
						set_bit(HFC_NEED_CANCEL, (ulong *)&target->status );
						clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status );			/* FCLNX-GPL-038 */
					}
				}
				else{
					if( (test_bit(HFC_WAIT_TARGET_RESET, (ulong *)&target->status ) ) ||
						(test_bit(HFC_WAIT_LOGIN, (ulong *)&target->status ) ) ||
						(test_bit(HFC_WAIT_CANCEL, (ulong *)&target->status ) )||	/* FCLNX-GPL-337 *//* FCLNX-GPL-534 */
						(target->lustat[lun] & HFC_WAIT_ABORT) )
						break ;                                            
					target->lustat[lun] |= HFC_NEED_ABORT;
				}
			}
			
			break ;
			
		case HFC_ABORT_TMR:		/* Abort Task Set/Clear ACA Time-Out*/
			if(target == NULL) return;
			if( target->we_que_cnt == 0 )
				break ;
			if( hfcp == NULL ) break;
		
			if(hfc_manage_info.hfcplus_enable){
				if(ap->rt_err_enable){                              /* FCLNX-0506 */
					hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_OCCURED_FAILURE, HFC_RT_ERR);  /* FCLNX-0506 */
				}
				else{
					if(ap->to_errcnt_info!=NULL)              /* FCLNX-0506 */
						 hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_OCCURED_FAILURE, HFC_TO_ERR);
				}
			}
									
			ap->scsi_err_cnt++ ;
			xob_no = hfcp->cmd_xob;
			hfcp->adap_status |= SCS_CMD_TIMEOUT;
			target->lustat[lun] &= ~HFC_NEED_ABORT;
			target->lustat[lun] &= ~HFC_WAIT_ABORT;
			
			if( hfc_toutchk_xob(ap, target, hfcp, lun, HFC_ISSUE_TARGET_RESET) )
			{
					/*
					 * Timed-out xob remain in queue so do not initiate
					 * MIH-LOG and Abort-Task-Set 
					 */
				break ;
			}

			if( hfc_pcibus_chk(ap) == 0 )
			{
				/* MIH-LOG Request */
				/* Issue Abort_Task_Set after collecting MIH-LOG in case SCSI command is timeout */
				if(ap->login_restrain) {
					set_bit(HFC_NEED_CANCEL, (ulong *)&target->status);					/* FCLNX-0506 */
					clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status);				/* FCLNX-0506 */ /* FCLNX-GPL-038 */
				}
				if( hfc_issue_mihlog(ap, target, hfcp) )
				{
					if(ap->c_err!=0) break; /* FCLNX-0597 */
					/* Failed to initiate mailbox request       */
					/* Execute Abort without collecting MIH-LOG */
					if( test_bit( HFC_NEED_LOGIN, (ulong *)&target->status ) )
					{
						if( hfc_issue_relogin(ap, target) )
						{
							set_bit( HFC_NEED_LOGIN, (ulong *)&target->status );
							clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */
							hfc_enque_login_req(ap, target);
						}
						break ;														/* FCLNX-GPL-038 */
					}
					if( test_bit( HFC_NEED_CANCEL, (ulong *)&target->status ) )
					{
						if( hfc_issue_relogin(ap, target) )
						{
							set_bit( HFC_NEED_CANCEL, (ulong *)&target->status );
							clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status);	/* FCLNX-GPL-038 */
							hfc_enque_login_req(ap, target);			/* 1.107 */
						}
						break;
					}
					if( (test_bit(HFC_WAIT_TARGET_RESET, (ulong *)&target->status ) ) ||
						(test_bit(HFC_WAIT_LOGIN, (ulong *)&target->status ) ) ||
						(test_bit(HFC_WAIT_CANCEL, (ulong *)&target->status ) ) )		/* FCLNX-GPL-038 */
						break ;                                            
					hfc_issue_task_mgm(ap, target, hfcp, lun, HFC_ISSUE_LOGIN) ;		/* FCLNX-0500 */
					/* If initiation succeeds, set HFC_WAIT_HALT in hfc_isuue_task_mgm() */
				}
				if(ap->c_err!=0) break; /* FCLNX-0597 */
				if( (test_bit(HFC_WAIT_TARGET_RESET, (ulong *)&target->status ) ) ||
					(test_bit(HFC_WAIT_LOGIN, (ulong *)&target->status ) ) ||
					(test_bit(HFC_WAIT_CANCEL, (ulong *)&target->status ) ) )		/* FCLNX-GPL-337 *//* FCLNX-GPL-534 */
					break ;                                            
				set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );
				clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status );					/* FCLNX-GPL-038 */
			}
			
			break ;

		case HFC_TARGET_RST_TMR:	/* Target Reset Time-Out			*/
			if((hfc_manage_info.hfcplus_enable)&&(ap->to_errcnt_info!=NULL)){
			/* FCLNX-0506 */
				hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_OCCURED_FAILURE, HFC_TO_ERR);  /* FCLNX-0506 */
			}
			hfc_timeout_by_reset(ap, target, hfcp);
			break ;
			
		case HFC_ELS_TMR:			/* ELS Time_Out						*/
		case HFC_LINKINIT_TMR:		/* Link Initialize Time-Out			*/
		case HFC_MB_TMR:			/* Other MailBox Time-Out			*/
			if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status ) )
			{
				ap->mb_status = 0 ;				/* FCLNX-0402 */
				ap -> mb_resp = HFC_MBR_TIMEOUT;
				hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
				break ;
			}
			ap->mb_status = 0 ;
			ap -> mb_resp = HFC_MBR_TIMEOUT;

			memset(logdata,0,16);
			logdata[0] = ap->mb->mb_init.command ;
			logdata[1] = ap->mb->mb_init.sub_cmd ;
			hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_NONE,ERRID_HFCP_EVNT4,0x2a,
														logdata,16) ;
            hfc_abend(ap,HFC_ABEND_MB_TOUT);

			break ;
			
		case HFC_CTLRST_DELAY_TMR :											/* FCLNX-0279 */
			/* FIVE only */

			if( !lock_try_mpap( mpap ) ) {									/* FCLNX-0287 */
				hfc_watchdog_enter( ap,NULL,NULL,0,HFC_CTLRST_DELAY_TMR,1,FALSE);
				break;
			}	
			/* Aapter recovery process from isolation */
			if ( !test_bit ( HFC_ISOL_RECOVERY,   (ulong *)&ap->status) ){
				if ( !test_bit( HFC_HMCK_RECOVRTY, (ulong *)&mpap->lock ) ) {
					HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
					break;
				}
			}

			set_bit( HFC_CTLRST_PROCESS, (ulong *)&mpap->status );

			hfc_reset_start(ap, HFC_CTLRST);
			
			udelay(1000); 	/* 1ms wait */
			
			clear_bit( HFC_CTLRST_PROCESS, (ulong *)&mpap->status );
			HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);							/* FCLNX-0287 */
			
			hfc_reset_start(ap, HFC_WSCA_CLEAR);
			
			if( HFC_MMODE_CHECK_MLPF(ap) )                                  /* FCLNX-0379 */
				hfc_reset_start(ap, HFC_SET_MLPF_MODE);
			
			hfc_reset_adap_info(ap) ;
			
			hfc_w_stop(ap,HFC_REBOOT_DELAY_TMR) ;
			hfc_w_start(ap,HFC_REBOOT_DELAY_TMR) ;

			/* Aapter recovery process from isolation */
			if ( test_bit ( HFC_ISOL_RECOVERY,   (ulong *)&ap->status) ){

			 /** Clear Interruption register (This process is different from machine check recovery */
				hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_RST,( char )0x4,  0xffffffff);
			}
			
			hfc_reset_start(ap, HFC_REBOOT);
			set_bit(HFC_MCK_SUCCESS, (ulong *)&ap->mck_result );
			
			break;

		case HFC_REBOOT_DELAY_TMR :

			/* Check HW status and POST result (retry count is zero) */
			if( ap->pkg.type == HFC_PKTYPE_FPP )
			{
				if( hfc_config_hw_set(ap,0) )
				{
					/* H/W Initialize failed. */
					hfc_chk_stop( ap, FALSE ) ;											/* FCLNX-0276 */
				}
				else
				{
					/* H/W Initialize succeeded. */
					clear_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status);
					if( test_bit (HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
						if(ap->mck_on_sleep) {
							hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);				/* FCLNX-0296 */
						}
					}
					set_bit( HFC_WAIT_LINKUP, (ulong *)&ap->status);
					hfc_w_stop( ap, HFC_LINKUP2_TMR );									/* FCLNX-0241 */
					hfc_w_start( ap, HFC_LINKUP2_TMR );									/* FCLNX-0241 */
					hfc_inta_mask_set(ap, HFC_INTA_MASK) ;
				}
			}
			else if( ap->pkg.type == HFC_PKTYPE_FIVE )
			{
				rc = lock_try_mpap( mpap );	
				
				if( rc == 0 ){
					/* restart HFC_REBOOT_DELAY_TMR (1s) */
					hfc_w_stop( ap, HFC_REBOOT_DELAY_TMR );
					hfc_watchdog_enter( ap,NULL,NULL,0,HFC_REBOOT_DELAY_TMR,1,0);
					break;
				}																		/* FCLNX-0279 */

				if ( hfc_config_hw_set_five(ap, 1))                                     /* FCWIN-0286 */
				{	
					if ( test_bit ( HFC_ISOL_RECOVERY, (ulong *)&ap->status) ){
						hfc_force_linkdown(ap, FALSE, FALSE);
					}else{
						/* H/W Initialize failed. */
						HFC_ISSUE_CSTP( ap, TRUE, HFC_ABEND_FCSTP) ; /* FCLNX-0279 *//* @MLPF *//* FCLNX-GPL-527 */
					}
				}
				else
				{
					/* H/W Initialize succeeded. */
					/* WS Comu Area(0x318) */
					hfc_reset_start(ap, HFC_SET_WS80);
					
					/* INITTBL ADR(0x310)  */
					/* ALPA(0x319)         */
					hfc_reset_start(ap, HFC_SET_INIADR);

					hfc_mck_port_recovery(ap);
					
					clear_bit(HFC_MCK_SUCCESS, (ulong *)&ap->mck_result );
					clear_bit( HFC_HMCK_RECOVRTY, (ulong *)&mpap->lock );

					ap->mck_linkup = HFC_LINKUP_MCK; /* FCLNX-0595 */
					if ( test_bit ( HFC_HWISOL, (ulong *)&mpap->status) ){
						clear_bit( HFC_HWISOL, (ulong *)&mpap->status );
						ap->isol_detail = HFC_NO_ISOLATE;
						ap->c_err = 0x00;
						clear_bit( HFC_ISOL,  (ulong *)&ap->status );
						clear_bit( HFC_ISOL_RECOVERY, (ulong *)&ap->status );

						clear_bit( HFC_HWCHKSTOP, (ulong *)&mpap->status ) ;
						clear_bit( HFC_CHK_STOP, (ulong *)&ap->status );
						mpap->mck_err_cnt = 0;	/* FCLNX-GPL-057 */
						ap->mck_type = 0;
						ap->mck_result = 0;

						hfc_errlog(
							ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE,
							ERRID_HFCP_EVNT2, 0xD3, NULL, 0);

						HFC_ERRPRT("hfcldd : Device %02x:%02x.%02x is recovered \n",
							ap->pci_cfginf->bus->number,
							PCI_SLOT(ap->pci_cfginf->devfn),
							PCI_FUNC(ap->pci_cfginf->devfn));
					}
				}

				
				HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
			}
			
			if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status) )
			{
				hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
			}
			
			break ;
			
		case HFC_MCK_DELAY_TMR :
			/* Wait for executing MCK recovery process issued by other port. */
			mpap = ap->mp_adap_info;
			if( test_bit(HFC_HWCHKSTOP, (ulong *)&mpap->status) ){
			}
			else if( test_bit(HFC_HMCK_RECOVRTY, (ulong *)&mpap->lock ) )
			{
				hfc_w_stop(ap,HFC_MCK_DELAY_TMR) ;
				hfc_w_start(ap,HFC_MCK_DELAY_TMR) ;
				break;
			}
			if( ap->pkg.type == HFC_PKTYPE_FIVE )				/* FCLNX-0493	*/
			{
				rc = lock_try_mpap( mpap );	
				clear_bit(HFC_SUBMCK_RECOVERY, (ulong *)&mpap->status );
				HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
			}													/* FCLNX-0493	*/
			/* WS Comu Area(0x318) */
			hfc_reset_start(ap, HFC_SET_WS80);
			/* INITTBL ADR(0x310)  */
			/* ALPA(0x319)         */
			hfc_reset_start(ap, HFC_SET_INIADR);
			set_bit(HFC_MCK_SUCCESS, (ulong *)&ap->mck_result );
			
			hfc_mck_port_recovery( ap );
			clear_bit(HFC_MCK_SUCCESS, (ulong *)&ap->mck_result );
			
			if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status) )
			{
				hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
			}
			break;
			
		case HFC_MCKINT_TMR :															/* FCLNX-0275 */
			
			int_a_rst = HFC_INTA_MASK;
			hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_RST,( char )0x4, int_a_rst);

			if( test_bit( HFC_WAIT_T3, (ulong *)&ap->status ) )
			{
				clear_bit( HFC_WAIT_T3, (ulong *)&ap->status );
				if( HFC_MMODE_CHECK_SHADOW(ap) )
					hfc_mlpf_issue_ffcstp(ap, HFC_ABEND_FCSTP);
				else
					hfc_abend(ap, HFC_ABEND_T3);
			}
			else {
				if( HFC_MMODE_CHECK_SHADOW(ap) )
					hfc_mlpf_issue_ffcstp(ap, HFC_ABEND_FCSTP);
				else
					hfc_abend(ap, HFC_ABEND_MCK_INT);
			}
			break;																		/* FCLNX-0275 */
			
// @MLPF
		case HFC_MLPF_FMCK_TMR :
			clear_bit(HFC_MLPF_WAIT_FMCK, (ulong *)&ap->status );
			
			if( HFC_MMODE_CHECK_SHADOW(ap) )
				hfc_issue_forced_mck(ap, HFC_ABEND_T3);
			
			break;
		
		case HFC_MLPF_MCKEND_TMR :
			clear_bit(HFC_MLPF_WAIT_MCKEND, (ulong *)&ap->status );
			
			if( HFC_MMODE_CHECK_SHADOW(ap) )
				hfc_mlpf_issue_ffcstp(ap, HFC_ABEND_FCSTP);
			else // Guest LPAR
				hfc_mlpf_cstpend_int(ap);
			
			break;
		
		case HFC_MLPF_FCSTP_TMR :
			clear_bit(HFC_MLPF_WAIT_FCSTP, (ulong *)&ap->status );
			if( HFC_MMODE_CHECK_SHADOW(ap) )
				hfc_mlpf_fcstp_int(ap, HFC_ABEND_FCSTP);
			
			break;

		case HFC_RESTART_TMR :						/* FCLNX-0500 */
			hfc_timeout_by_restart(ap, target, hfcp);	/* FCLNX-GPL-531 */
			break;
			
		case HFC_WEXEC_TMR:
			if(target == NULL) return;
			hfc_watchdog_enter(ap, target, NULL, 0, HFC_WEXEC_TMR, 0, TRUE);
			if( ( target->wx_que_cnt == 0 ) && ( ap->xob_wait_exec_cnt == 0 ) )
				break ;
			if( test_bit( HFC_SCSI_DELAY, (ulong *)&target->status ) )
				break ;
			if( test_bit( HFC_NEED_LOGIN, (ulong *)&target->status ) )
			{
				if( hfc_issue_relogin(ap, target) )
				{
					set_bit( HFC_NEED_LOGIN, (ulong *)&target->status );
					clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status);	/* FCLNX-GPL-038 */
					hfc_enque_login_req(ap, target);			/* 1.107 */
				}
			}
			
			if( test_bit( HFC_NEED_CANCEL, (ulong *)&target->status ) )
			{
				if( hfc_issue_relogin(ap, target) )
				{
					set_bit( HFC_NEED_CANCEL, (ulong *)&target->status );
					clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status);	/* FCLNX-GPL-038 */
					hfc_enque_login_req(ap, target);			/* 1.107 */
				}
			}
			
			if( ( target->wx_que_cnt > 0 ) || ( ap->xob_wait_exec_cnt > 0 ) )
			{
				hfc_issue_intl_start(ap,target) ;
			}
			break ;
					
		case HFC_LINKUP_TMR :			/* LINK UP waiting timer			*/
			if((hfc_manage_info.hfcplus_enable)&&(ap->ldl_errcnt_info!=NULL)){                              /* FCLNX-0506 */
				hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_OCCURED_FAILURE, HFC_LDL_ERR);  /* FCLNX-0506 */
			}

		case HFC_LINKUP2_TMR :                  /* LINK UP waiting timer after MCK recovers */ /* FCLNX-241 */

			clear_bit(HFC_ONLINE, (ulong *)&ap->status);
			set_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status );

			/* Cancel pending SCSI processes under this adapter */
			for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)
			{
				target = hfc_hash_target_info(ap, lp);
				if( target != NULL )
				{
					hfc_cancel_scsi_cmd(ap,target,0, NULL, SCS_LINKUP_TO, HFC_CSCSI_ERROR,	/* FCLNX-0429 */
							TRUE, TRUE, HFC_FLASH_TARGET);
					target->status = HFC_NON_STATUS;
				}
			}
			hfc_wwnverify_linkup_timeout(ap, NULL, 0);
			
			if (hfc_manage_info.npubp->hfc_mp_hsd_enable)									/* FCLNX-0429 */
				hfc_manage_info.npubp->hfc_make_fcinfo(ap, NULL, 0x14, SCS_LINKUP_TO, NULL, 0);
			
			if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) || (ap->initialize == 1) ) 	/* FCLNX-0279 */
			{
				HFC_DBGPRT(KERN_ERR "hfcldd%d : hfc_watchdog() lock fail. \n", ap->dev_minor);
				unlock_mailbox( ap ) ; 
				hfc_wake_up(&ap->init_event,&ap->int_a_poll);
				clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);
			}																						/* FCLNX-0279 */
			break ;
			
		case HFC_SCN_LINKUP_TMR :
			hfc_timeout_by_scnlinkup(ap, target);
			break;
			
		case HFC_DIAG_DELAY_TMR:			/* Delayed Time-Out when diag process completes	*/
			if( test_bit(HFC_DIAG_DELAY, (ulong *)&ap->status ) )
			{
				hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);										/* FCLNX-0296 */
				break ;
			}
			break ;
			
		case HFC_MPAP_LOCK_TMR:				/* Wait for acquiring lock in MP_ADAP_INFO	*/
			if( test_bit( HFC_LOCK_WAIT_1, (ulong *)&ap->mpap_lock ) )
			{
				hfc_mck_recovery_five( ap, ap->mck_type );
			}
			else if( test_bit( HFC_LOCK_WAIT_2, (ulong *)&ap->mpap_lock ) )
			{
				hfc_chk_stop( ap, FALSE );															/* FCLNX-0279 */
			}
			else if( test_bit( HFC_LOCK_WAIT_3, (ulong *)&ap->mpap_lock ) )
			{
				hfc_fpp_logout( ap, ap->err_no, ap->mode );
			}
			else if( (test_bit( HFC_LOCK_WAIT_4, (ulong *)&ap->mpap_lock )) ||
						(test_bit( HFC_LOCK_WAIT_5, (ulong *)&ap->mpap_lock ))	 )
			{
				hfc_five_logout( ap, ap->err_no, ap->mode );
			}
			else if( test_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock ) )     // @MLPF
			{
				if( HFC_MMODE_CHECK_SHADOW(ap) )
					hfc_mlpf_errlog_slpar( ap );
				else if( HFC_MMODE_CHECK_SHARED(ap) && !( HFC_MMODE_CHECK_SHADOW(ap) ) )
					hfc_mlpf_errlog_glpar( ap );
			}
			else if( test_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock ) )     /* FCLNX-GPL-0576 */
			{
				hfc_mck_recovery_fpp( ap, ap->mck_type );
			}
			break;
			
		case HFC_LOGIN_DELAY_TMR:		/* Delayed Time-Out when LOGIN process completes	*/ 	/* FCLNX-0243 */
			clear_bit(HFC_LOGIN_DELAY, (ulong *)&ap->status );
			
			if (ap->initialize)												/* FCLNX-0270 */
				start_next_mailbox( ap );									/* FCLNX-0270 */
			else {															/* FCLNX-0270 */
				if( !(test_bit(HFC_ONLINE, (ulong *)&ap -> status ) ) )
					break;
				
				if ( (ap->connect_type == HFC_SWITCH )
					|| ((ap->connect_type == HFC_AL) && (ap -> scsi_id & 0x00ffff00)) ) {
					if (!test_bit(HFC_WAIT_NMSRV, (ulong *)&ap->status) ) {
						set_bit(HFC_NEED_NMSRV, (ulong *)&ap->status);
//						hfc_issue_gidft( ap );								/* FCLNX-GPL-038 */
						start_next_mailbox( ap );							/* FCLNX-GPL-038 */
					}
				}
				else {
					start_next_mailbox( ap );								/* FCLNX-0279 */
				}
			}																/* FCLNX-0270 */
			break;
		case HFC_LDLERR_TMR:	/* Long Link Down Error count timer time-out*/
			hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_WATCHERR_TMR_TO, HFC_LDL_ERR);		/* FCLNX-0506 */
			break;		/* FCLNX-0506 */
		case HFC_LDSERR_TMR:	/* Short Link Down Error count timer time-out*/
			hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_WATCHERR_TMR_TO, HFC_LDS_ERR);		/* FCLNX-0506 */
			break;		/* FCLNX-0506 */
		case HFC_IFERR_TMR:	/* Interface Error count timer time-out*/
			hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_WATCHERR_TMR_TO, HFC_IF_ERR);			/* FCLNX-0506 */
                        break;		/* FCLNX-0506 */
		case HFC_TOERR_TMR:	/* SCSI T.O Error count timer time-out*/
			hfc_manage_info.npubp->hfc_watched_errcount(ap, HFC_WATCHERR_TMR_TO, HFC_TO_ERR);			/* FCLNX-0506 */
                        break;		/* FCLNX-0506 */

		case HFC_DELAY_TMR_DEV:			/* DELAY timer after executing LUN RESET	*/
			if(target == NULL) return;												/* FCLNX-GPL-038 */
			if(dev == NULL) return;													/* FCLNX-GPL-038 */
			hfc_manage_info.npubp->hfc_clear_dev_info( dev );

			if( test_bit( HFC_SCSI_DELAY, (ulong *)&target->status ) )
				break;
			if( (test_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status) ) && (test_bit(HFC_ONLINE, (ulong *)&ap->status) ) )
			{
				/* If adapter is waiting LINK_UP, I/O start can not be executed. */
				break;
			}
			else if( !(test_bit(HFC_ONLINE, (ulong *)&ap->status) ) )
			{
				/* 
				 * If adapter is not online, target should have been already canceled.
				 * Break this process 
				 */
				break ;
			}
			else if( !(test_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status) )  && (test_bit(HFC_ONLINE, (ulong *)&ap->status) ) )
			{
				/* Restart waiting request if adapter is ONLINE */
				if(!(test_bit(HFC_SCN_WLINKUP, (ulong *)&target->status) )  
				&& test_bit(HFC_NEED_LOGIN, (ulong *)&target->status)  )			/* FCLNX-GPL-038 */
				{
					if( hfc_issue_relogin(ap, target) )
					{
						/* Initiated LOGIN but failed with mailbox busy */
						set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );			/* FCLNX-GPL-038 */
						clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status );		/* FCLNX-GPL-038 */
						hfc_enque_login_req(ap, target);
					}
					break;
				}
				else if( test_bit(HFC_NEED_CANCEL, (ulong *)&target->status) )  	/* FCLNX-GPL-038 */
				{
					if( hfc_issue_relogin(ap, target) )
					{
						/* Initiated CANCEL LOGIN but failed with mailbox busy */
						set_bit(HFC_NEED_CANCEL, (ulong *)&target->status );		/* FCLNX-GPL-038 */
						clear_bit(HFC_NEED_LOGIN, (ulong *)&target->status );		/* FCLNX-GPL-038 */
						hfc_enque_login_req(ap, target);
					}
					break;
				}																	/* FCLNX-GPL-038 */
				if( ( target->wx_que_cnt > 0 ) || ( ap->xob_wait_exec_cnt > 0 )){					/*-- FCLNX-019 STR--*/
					hfc_issue_intl_start(ap,target) ;
				}												/*-- FCLNX-019 END--*/
			}
			break ;

		case HFC_ISOLATE_DELAY_TMR:
			hfc_force_linkdown(ap, FALSE, TRUE);
			break;
			
	}	/*-- end of switch --*/

	if(hfc_manage_info.hfcplus_enable){
		hfc_manage_info.npubp->hfc_check_errcount(ap);
		/* FCLNX-0506 */
	}

	if ( hfc_manage_info.hfcldd_mp_mod && hfc_manage_info.lg_target_info )
	{
		if ( ap->retry_hfcp_top )
			hfc_manage_info.npubp->hfc_retry_strategy(ap);
	
		if ( hfc_manage_info.wait_reset_mp )								/* FCLNX-0429 */
		{
			hfc_manage_info.npubp->hfc_check_dev_reset_complete();			/* FCLNX-0429 */
			hfc_manage_info.npubp->hfc_check_bus_reset_complete();			/* FCLNX-0429 */
		}
	}

	HFC_ADAPUNLOCK_IRQRESTORE(flags);

	return;
	
}


/*
 * Function:    hfc_timeout_by_reset
 *
 * Purpose:     Responce Time out of Target Reset in Bus Reset
 *
 * Arguments:   
 *  ap         - Pointer to dap_info 
 *  target     - Pointer to arget_info 
 *  hfcp       - Pointer to hfc_pkt 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_timeout_by_reset(												/* FCLNX-0500 */
	struct adap_info        *ap,
	struct target_info      *target,
	struct hfc_pkt     		*hfcp)
{
	int	   lun=0;
	
	if(hfcp != NULL)		/* FCLNX-0608 */
	{
		lun = hfcp->lun_id;
	}						/* FCLNX-0608 */
									
	memset( logdata, 0, 16 );			/* FCLNX-0608 */
	memcpy( logdata, (uchar*)&ap->xob[hfcp->cmd_xob].fcp_cmd.fcp_cntl, 4 );
	hfc_errlog( ap, target, hfcp, HFC_ERRLOG_TYPE_NONE,
							 ERRID_HFCP_ERRA, 0x29, logdata, 16 );
	if(test_bit(CFLAG_SCMD_SLOGV, (ulong *)&hfcp->cmd_flags)){
		hfc_errlog( ap, target, hfcp, HFC_ERRLOG_TYPE_TOUTLOG,
					 		ERRID_HFCP_EVNT3, 0x7f, NULL, 0 );	
	}					/* FCLNX-0608 */

	clear_bit( HFC_WAIT_TARGET_RESET, (ulong *)&target->status );

	if( hfc_toutchk_xob(ap, target, hfcp, lun, HFC_ISSUE_TARGET_RESET) )
	{
		/*
		 * Timed-out xob remain in queue so do not initiate
		 * MIH-LOG and Abort-Task-Set
		 */
		return ;
	}

	if( hfc_pcibus_chk(ap) == 0 )/* FCLNX-0608 */
	{	
		if(ap->c_err!=0) return;
		/* Failed to initiate mailbox request       */
		/* Execute Abort without collecting MIH-LOG */
		if( test_bit(HFC_NEED_CANCEL, (ulong *)&target->status ) ){
			set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );
			clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status );	/* FCLNX-GPL-038 */
		}
		if( test_bit(HFC_NEED_LOGIN, (ulong *)&target->status ) )
		{
			if( hfc_issue_relogin(ap,target) )
			{
				set_bit(HFC_NEED_LOGIN, (ulong *)&target->status );
				clear_bit(HFC_NEED_CANCEL, (ulong *)&target->status );	/* FCLNX-GPL-038 */
				hfc_enque_login_req(ap, target);
			}
			return ;
		}
		if(test_bit(HFC_WAIT_LOGIN, (ulong *)&target->status ))  return ;                                            
		hfc_issue_task_mgm(ap, target, hfcp, lun, HFC_ISSUE_LOGIN) ;
		/* If initiation succeeds, set HFC_WAIT_HALT in hfc_isuue_task_mgm() */
		if(ap->c_err!=0) return; 
	}			/* FCLNX-0608 */
	return;
}

/*
 * Function:    hfc_occurred_mck
 *
 * Purpose:      occured Machin Check
 *
 * Arguments:
 *  ap         - Pointer to adap_info structure
 *  event          - cause of MCK
 *
 * Returns:
 *
 * Notes:
 */
void hfc_occurred_mck(struct adap_info *ap, uchar point)
{        /* FCLNX-0533 */
	HFC_DBGPRT("hfc_mck_point() : Machine check occurred  hfcldd%d (",ap->instance);
	switch(point){
		case HFC_BEFORE_POSTCHK:
			HFC_DBGPRT(KERN_ERR "  before Post Check.  ");
			break;
		case HFC_AFTER_LINKINITIALIZE:
			HFC_DBGPRT(KERN_ERR "  after Link Initilaize. ");
			break;
		case HFC_AFTER_SCSI_HOST_RESCAN:
			HFC_DBGPRT(KERN_ERR "  after scsi host rescan. ");
			break;
		default:
			break;
		}
        HFC_DBGPRT(KERN_ERR ")\n");
//      hfc_issue_forced_mck(ap, HFC_ABEND_T3);

        hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDCTL,( char )0x1, 0x08 );

        return;
}       /* FCLNX-0533 */

/*
 * Function:    hfc_reset_start
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - Pointer to adap_info
 *  type       - 1:ctlrst , 2:reboot , 3:f_start, 4:fw_start
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_reset_start(struct adap_info *ap,uchar type)
{
	uint			wk1,wk2 ;
	uint			length ;  
	unsigned long long	wkp;
	int				dma_size;

	HFC_DBGPRT(" hfcldd : hfc_reset_start - start\n");

	switch( type )
	{
		case HFC_CTLRST :
			hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDRES,( char )0x1,(char)0x02);
			break ;
		case HFC_REBOOT :
			hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDBOOT,( char )0x1,(char)0x20);
			break ;
		case HFC_F_START :
			hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDCTL,( char )0x1,(char)0x80);
			break ;
		case HFC_SET_INIADR :
			wk1 = 0;
			wk2 = (uint)ap->padr_init ;
			/* 
			 * Reboot with AL_PA=0 in MCK recovery process before link initialization 
			 *  -- always set pref_alpa. 
			 */
			hfc_write_reg(ap, ( uint )HFC_IOSPACE_CA_RSTINFO+1,( char )0x1,(char)ap->pref_alpa);
			hfc_write_reg(ap, ( uint )HFC_IOSPACE_CA_LNKSPD,( char )0x1,(char)ap->linkspeed); 
			hfc_write_reg(ap, ( uint )HFC_IOSPACE_CA_CNTTYP,( char )0x1,(char)ap->topology); 
// @MLPF
			if( HFC_MMODE_CHECK_SHARED (ap) )
				hfc_write_reg_ext(ap, 0x0308, (char)0x1, 0x80);
			else if( HFC_MMODE_CHECK_DEDICATE(ap) )
				hfc_write_reg_ext(ap, 0x0308, (char)0x1, 0x40);
// @MLPF
			dma_size = sizeof(dma_addr_t);
			wkp = ap->padr_init;
			if(dma_size == 8){
				wkp >>=32;
			}
			else{
				wkp = 0;
			}
			hfc_write_reg(ap, HFC_IOSPACE_CA_INIT_ADDR0, 0x4, wkp);
			hfc_write_reg(ap, HFC_IOSPACE_CA_INIT_ADDR1, 0x4, ap->padr_init);

			if ( ap->pkg.type != HFC_PKTYPE_FPP )										/* FCWIN-0200 */
				hfc_write_reg(ap,( uint )HFC_IOSPACE_CA_PORTNO,( char )0x1,ap->port_no);/* FCWIN-0200 */

			break ;
		case HFC_SET_WS80 :
			hfc_write_reg(ap, ( uint )HFC_IOSPACE_CA_RSTINFO,( char )0x1,(char)0x80);		
			break ;
		case HFC_SET_WS40 :
			hfc_write_reg(ap, ( uint )HFC_IOSPACE_CA_RSTINFO,( char )0x1,(char)0x40);		
			break ;
		case HFC_SET_WS04 :    
			hfc_write_reg(ap, ( uint )HFC_IOSPACE_CA_RSTINFO,( char )0x1,(char)0x04);		
			break ;
		case HFC_FW_START :
			if ( HFC_MMODE_CHECK_SHARED(ap) )
			{
				hfc_write_reg(ap,( uint )HFC_IOSPACE_FRAMEA, 0x4,(int)HFC_FRAMEA_FW_START_MCK);
			}
			else
			{
				if(!test_bit(HFC_ISOL, (ulong *)&ap->status))	/* FCLNX-0639 */
					hfc_write_reg(ap,( uint )HFC_IOSPACE_FRAMEA, 0x4,(int)0x40000000);
			}
			break ;
		case HFC_INI_RESET :
			hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDRES,( char )0x1,(char)0x04);
			/* 1ms dummy wait after iniRes */
			wk1 = 0;
			udelay(1000); /* 1ms wait */
			break ;
		case HFC_WSCA_CLEAR :
			/* clear 256 bytes from PCI address 0x300 */
			wk2 = ap->pkg.map->iosp.reg[HFC_IOSPACE_CA_POSTRESULT] ;
			length = 256 ;													
			if( ap->pkg.type == HFC_PKTYPE_FIVE )                           
				length = 128 ;
            		for(wk1= wk2 ; wk1 < wk2+length ; wk1+=4 )                    
            		{
				if( ap->pkg.type == HFC_PKTYPE_FIVE )                       /* FCLNX-0379 */
				{
					if(
						( wk1 == 0x0330 ) ||
						( wk1 == 0x0334 ) ||
						( wk1 == 0x0338 ) ||
						( wk1 == 0x033C ) )
						continue;
				}							/* FCLNX-401 */
	            		hfc_write_reg_ext(ap,( uint )wk1,( char )0x4,0);
           		 }
            		break ;
		case HFC_SET_MLPF_MODE :                                            /* FCLNX-0379 */
			if( HFC_MMODE_CHECK_SHARED (ap) )
				hfc_write_reg_ext(ap, 0x0308, (char)0x1, 0x80);
			else if( HFC_MMODE_CHECK_DEDICATE(ap) )
				hfc_write_reg_ext(ap, 0x0308, (char)0x1, 0x40);
			break;
	}
}	


/*
 * Function:    hfc_issue_forced_mck
 *
 * Purpose:     Issue forced MCK and initiate timer of all ports for supervision.
 *
 * Arguments:   
 *  ap         - Pointer to adap_info
 *
 * Returns:     
 *  = 0        - Forced MCK initiation succeeded 
 *  !=0        - Forced MCK initiation failed 
 *
 * Notes:       Lock mpap->lock<HFC_MLOCK_MCKTMR_ENTRY to initiate timer. 
 */
int hfc_issue_forced_mck(struct adap_info *ap, uchar type)					/* FCLNX-0279 */
{
	struct mp_adap_info		*mpap = ap->mp_adap_info;
	uint				wk_reg;

	if( !test_bit( HFC_WAIT_T3, (ulong *)&ap->status )
	 && !test_bit( HFC_HWCHKSTOP, (ulong *)&mpap->status ) )
	{
		if ( test_bit(HFC_HMCK_RECOVRTY, (ulong *)&mpap->lock) || test_bit(HFC_SUBMCK_RECOVERY, (ulong *)&mpap->status)  ) {	/* FCLNX-GPL-034 */
			/* skip issue forced mck */
			HFC_DBGPRT(" hfc_issue_forced_mck() - skip issue forced mck\n");
		}																														/* FCLNX-GPL-034 */
		else {
			set_bit( HFC_WAIT_T3, (ulong *)&ap->status );
			
			hfc_w_stop(ap,HFC_MCKINT_TMR) ;
			hfc_w_start(ap,HFC_MCKINT_TMR) ;

			wk_reg = 0;
			wk_reg |= type & 0xff;
			wk_reg <<= 16;
			wk_reg |= ap->mb->mb_init.command & 0xff;
			wk_reg <<= 8;
			wk_reg |= ap->mb->mb_init.sub_cmd & 0xff;
			hfc_write_reg(ap,( uint )HFC_IOSPACE_DRV_USED0,( char )0x4, wk_reg);

			hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDCTL,( char )0x1, 0x08 );
		}
	}
	return (1);
}																			/* FCLNX-0279 */


/*
 * Function:    hfc_reset_adap_info
 *
 * Purpose:     Initiate adap_info
 *
 * Arguments:   
 *  ap         - Pointer to adap_info 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_reset_adap_info(
	struct adap_info        *ap)
{
	uint                    lp;

	HFC_DBGPRT(" hfcldd : hfc_reset_adap_info - start\n");

	
	for(lp=0 ; lp<ap->xob_max ; lp++)       /*--    FCWIN-0071      --*/
	{
		ap->xob[lp].flag = 0 ;
		ap->xob[lp].skip = 0 ;
	}
	ap->fw_init_p->xob_inp = 0 ;
	ap->fw_init_p->xob_outp = 0 ;
	
	for(lp=0 ; lp<ap->xrb_max ; lp++)       /*--    FCWIN-0071      --*/
	{
		ap->xrb[lp].flag = 0 ;
		ap->xrb[lp].skip = 0 ;				/*--    FCLNX-GPL-038   --*/
	}
	ap->fw_init_p->xrb_inp = 0 ;
	ap->fw_init_p->xrb_outp = 0 ;
	
	ap->mb_retry_cnt = 0 ;
	ap->xob_no = 0 ;
	ap->xrb_no = 0 ;
	ap->iov_no = 0 ;
//	ap->iov_map_cnt = 0 ;					/*--	LINUX-049		--*/
	ap->xob_exec_cnt = 0 ;
	ap->xob_wait_exec_cnt = 0 ;
	
	for(lp=0 ; lp<MAX_FRAME_CNT ; lp++)
		ap->save_xob_outp[lp] = 0 ;
	ap->frame_inp = 0 ;
	ap->frame_chkp = 0 ;
	
	ap->next_tstart = NULL ;
	ap->login_target = NULL ;               /*--    FCLNX-GPL-038      --*/
	ap->next_gidpn = FALSE ;                /*--    FCLNX-GPL-038      --*/
	clear_bit(HFC_LOGIN_DELAY, (ulong *)&ap->status );			/* FCLNX-GPL-038 */
	clear_bit(HFC_NEED_LINK_INIT, (ulong *)&ap->status );		/* FCLNX-GPL-038 */
	clear_bit(HFC_NEED_NMSRV, (ulong *)&ap->status );			/* FCLNX-GPL-038 */
	clear_bit(HFC_NEED_GPNID, (ulong *)&ap->status );			/* FCLNX-GPL-038 */
	clear_bit(HFC_WAIT_LINK_INIT, (ulong *)&ap->status );		/* FCLNX-GPL-038 */
	clear_bit(HFC_WAIT_NMSRV, (ulong *)&ap->status );			/* FCLNX-GPL-038 */
	clear_bit(HFC_WAIT_GPNID, (ulong *)&ap->status );			/* FCLNX-GPL-038 */
	clear_bit(HFC_LOGIN_DELAY, (ulong *)&ap->status );			/* FCLNX-GPL-038 */

	return;
}


/*
 * Function:    hfc_timeout_by_scnlinkup
 *
 * Purpose:     Process when HFC_LINKUP_TMR is timed out.
 *
 * Arguments:   
 *  ap         - Pointer to adap_info 
 *  target     - Pointer to target_info 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_timeout_by_scnlinkup(
	struct adap_info        *ap,
	struct target_info      *target)
{
	HFC_DBGPRT( "hfc_timeout_by_scnlinkup() - timeout.");

	clear_bit(HFC_SCN_WLINKUP, (ulong *)&target -> status );

	/* Discards XRB when SCN_WLINKUP timer timed out 	*/

	hfc_cancel_scsi_cmd(ap, target, 0, NULL, SCS_SCN_LINKDOWN, 		/* FCLNX-GPL-038 */
			HFC_CSCSI_ERROR, TRUE, TRUE, HFC_FLASH_TARGET);			/* FCLNX-GPL-038 */

	/* Invalidates this target device					*/

	hfc_clear_target_info( ap, target, TRUE, TRUE );				/* FCLNX-GPL-038 */
}


/*
 * Function:    hfc_timeout_by_restart
 *
 * Purpose:     The task management is reactivated
 *
 * Arguments:   
 *  ap         - Pointer to adap_info structure
 *  target     - 
 *  hfcp       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_timeout_by_restart(								/* FCLNX-0500 */
	struct adap_info        *ap,
	struct target_info 		*target,
	struct hfc_pkt			*hfcp)
{
	uint                    find=FALSE;
	
	if(target != NULL)
	{
		if( test_bit(HFC_NEED_TARGET_RESET, (ulong *)&target->target_reset ) ){		/* FCLNX-GPL-036 */
			/* 
			 * Target Reset is requested but MIHLOG is not requested  
			 *  --> Issue Target Reset
			 */
			hfc_issue_task_mgm(ap, target, hfcp, 0, HFC_ISSUE_TARGET_RESET);		/* FCLNX-GPL-531 */
		}
		if ( test_bit(HFC_NEED_TARGET_RESET, (ulong *)&target->target_reset) )		/* FCLNX-GPL-036 */
			find = TRUE;
	}

	if (find == TRUE){
		hfc_watchdog_enter(ap, target, hfcp, 0, HFC_RESTART_TMR, (HZ/100+1), TRUE);	/* FCLNX-GPL-036 *//* FCLNX-GPL-531 */
		hfc_watchdog_enter(ap, target, hfcp, 0, HFC_RESTART_TMR, (HZ/100+1), FALSE);/* FCLNX-GPL-036 *//* FCLNX-GPL-531 */
	}
}


/*
 * Function:    hfc_reset_watchdog
 *
 * Purpose:     Release all executing timers
 *
 * Arguments:   
 *  ap         - Pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_reset_watchdog(struct adap_info *ap)
{
	uint                            lp;
	struct target_info              *target;
	struct dev_info					*dev;				/* FCLNX-GPL-038	*/
	
	HFC_DBGPRT("hfcldd%d : hfc_reset_watchdog\n",ap->dev_minor);

	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_ELS_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_REBOOT_DELAY_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN0_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN1_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN2_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN3_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN4_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN5_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN6_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LUN7_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LINKUP_TMR, 0, TRUE);
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_LOGIN_DELAY_TMR, 0, TRUE);				/* FCLNX-GPL-038 */
	hfc_watchdog_enter(ap, 0, NULL, 0, HFC_ISOLATE_DELAY_TMR, 0, TRUE);				/* FCLNX-GPL-038 */
	
	for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)			/* FCWIN-0083 */
	{
		target = hfc_hash_target_info(ap, lp);
		if (target != NULL)
		{
			hfc_watchdog_enter(ap, target, NULL, 0, HFC_SCN_LINKUP_TMR, 0, TRUE);
			hfc_watchdog_enter(ap, target, NULL, 0, HFC_DELAY_TMR, 0, TRUE);
			hfc_watchdog_enter(ap, target, NULL, 0, HFC_WEXEC_TMR, 0, TRUE);
			hfc_watchdog_enter(ap, target, NULL, 0, HFC_RESTART_TMR, 0, TRUE);		/* FCLNX-GPL-531 */
			dev = target->dev;										/* FCLNX-GPL-038	*/
			if(dev != NULL) {
				/* stop LUN Reset Delay Timer */
				hfc_manage_info.npubp->hfc_all_clear_dev_info( ap, dev );
			}														/* FCLNX-GPL-038	*/
		}
	}
}


/*
 * Function:    hfc_toutchk_xob 
 *
 * Purpose:     Search timed-out requests remain in xob.
 *              If so, force FW to forced MCK status 
 *
 * Arguments:   
 *  ap         - Pointer to adap_info
 *  target     - Pointer to target_info
 *  hfcp       - Pointer to hfc_pkt
 *  lun        - lun number
 *  mode       - 
 *
 * Returns:     
 *  0          - No timed-out requests remain in xobs
 *  -1         - One or more timed-out requests remain in xobs.
 *
 * Notes:       
 */
int hfc_toutchk_xob(
	struct adap_info		*ap,
	struct target_info		*target,
	struct hfc_pkt			*hfcp,
	uchar					lun,
	uchar					mode)
{
	uint                        outp_num ;
	uint                        save_outp_num;
	uint                        inp_num ;
	struct xob                  *cancel_xob ;
	uint                        save_xob_outp ;	
	uint                        save_xob_inp ;
	uint                        wk_save_xob_outp ;
	uint                        wk_save_xob_inp ;
	
	
	HFC_DBGPRT("hfcldd%d : hfc_toutchk_xob start", ap->dev_minor);
	
	wk_save_xob_inp = ap->fw_init_p->xob_inp ;
	wk_save_xob_outp = ap->fw_init_p->xob_outp ;
	
	HFC_4L_TO_4B(save_xob_inp, wk_save_xob_inp);
	HFC_4L_TO_4B(save_xob_outp, wk_save_xob_outp);
	if( save_xob_outp == save_xob_inp )
	{
		/* xob empty */
		return(0) ;
	}

	HFC_DBGPRT("hfcldd%d : hfc_toutchk_xob xob non empty", ap->dev_minor);

	/* Find sequential number of xob_inp and xob_outp */

	inp_num = ((save_xob_inp & 0x00ff0000)>>16)*HFC_XOB_PER_PAGE ;
	inp_num += (save_xob_inp & 0x0000ffff) ;
	save_outp_num = ((save_xob_outp & 0x00ff0000)>>16)*HFC_XOB_PER_PAGE ;
	save_outp_num += (save_xob_outp & 0x0000ffff) ;

	cancel_xob = ap->xob ;	/* Xob top address */
			
	if(hfcp != NULL)
	{
		if ( (mode == HFC_ISSUE_ABORT)
		  && (hfcp->lun_id != lun) ) {
			return(0);
		}

		HFC_DBGPRT("hfcldd%d : hfc_toutchk_xob find timeout", ap->dev_minor);

		outp_num = save_outp_num;
		do {
			if( hfcp == (struct hfc_pkt *)(ulong)cancel_xob[outp_num].drv_work.hfc_pkt )
			{
				if( (!(cancel_xob[outp_num].skip & HFC_XOB_SKIP)) && ( (hfcp->cmd_pkt->timeout_per_command/HZ) > 10 ) ){
					/* Timed-out requests remains in xobs */
					HFC_DBGPRT("hfcldd%d : hfc_toutchk_xob timeout pkt results in xob", ap->dev_minor);

					memset(logdata,0,16);
					HFC_MEMCPY(logdata, (uchar*)&outp_num, 4);
					HFC_MEMCPY(&logdata[4], (uchar*)&inp_num, 4) ;
					hfc_errlog(
						ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_EVNT4,
						0x8d, logdata, 16) ;
					hfc_abend(ap,HFC_ABEND_TOUTCHK_XOB);
					return(-1) ;
				}
			}

			outp_num++ ;
			if( outp_num >= ap->xob_max )
				outp_num = 0 ;
		}while( outp_num != inp_num ) ;
	}

	HFC_DBGPRT("hfcldd%d : hfc_toutchk_xob end", ap->dev_minor);

	return(0);
}

int hfc_force_linkdown( struct adap_info *ap, uchar proc, uchar skip)
{
	struct target_info		*target=NULL ;
	struct mp_adap_info		*mpap;
	uint					lp;	
	uint					ctlres_flag = 0;
	uchar					logdata[16],eno,support ;
	
	mpap = ap->mp_adap_info;

	HFC_ENTRY("hfc_force_linkdown");                 //FCLNX-0488
//	printk(KERN_ERR "call hfc_force_linkdown(hfcldd%d, %d, %d).\n",ap->dev_minor, proc, skip);
	hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x00, ap, NULL, NULL, proc, skip, 0);

	if (skip != TRUE) {
		if ( HFC_MMODE_CHECK_MLPF(ap) ) {
			hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x01, ap, NULL, NULL, 0, 0, 0);
//			printk(KERN_ERR " This adapter has already isolated \n");	
			return EINVAL;
		}
		
		if ( test_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status ) ) {
			hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x02, ap, NULL, NULL, 0, 0, 0);
			return EINVAL;
		}

		if (test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) {
			hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x03, ap, NULL, NULL, 0, 0, 0);	/* FCLNX-0707 */
			return EINVAL;
		}

		if (proc == TRUE) {
			if (test_bit(HFC_ISOL, (ulong *)&ap->status)) {
				if ((ap->isol_detail == HFC_ISOLATE_PORT_C)
				 || (ap->isol_detail == HFC_ISOLATE_CHKSTP_C)) {
					hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x05, ap, NULL, NULL, 0, 0, 0);
					return 0;
				}
			}
		}
		else {
			support = (uint) hfc_read_reg(ap, HFC_IOSPACE_FW_SUPPORT, 0x1);
			if ((ap->pkg.port > 1) && !(support & 0x80)) {
				hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x06, ap, NULL, NULL, 0, 0, 0);
				return(EINVAL);
			}
			
			/* This adapter has already isolated */
			if (test_bit ( HFC_HWISOL, (ulong *)&mpap->status)) {
				hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x07, ap, NULL, NULL, 0, 0, 0);
//				printk(KERN_ERR " This adapter has already isolated \n");	
				return 0;
			}
			
			/* This port has already isolated, but the adapter has not isolated yet */
			if (test_bit ( HFC_ISOL, (ulong *)&ap->status)) {
				hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x08, ap, NULL, NULL, 0, 0, 0);
//				printk(KERN_ERR " This adapter has already isolated \n");
				return EINVAL;
			}
		}

//		printk(KERN_ERR " isol detail = %x c_err = %d.\n", ap->isol_detail, ap->c_err);
		hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, (int)(0x00000000));
		hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_RST,( char )0x4, (int)(0xffffffff));

		hfc_reset_all_timer(ap);

		clear_bit(HFC_ONLINE, (ulong *)&ap->status );	

		if (ap->pkg.port <= 1) {
			hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x20, ap, NULL, NULL, 0, 0, 0);
//			printk(KERN_ERR "hfcldd%d :Issue Hardwware isolate request .\n",ap->dev_minor);

			/* Issue forced machine check request */
			hfc_write_reg(ap,( uint )HFC_IOSPACE_CMDCTL,( char )0x1, 0x08 );
			
			/* Set HFC_HWWISOL to mp_adap_info */
			set_bit(HFC_HWISOL,(ulong *)&mpap->status);
			set_bit(HFC_ISOL, (ulong *)&ap->status);
			
			if (proc == TRUE)
				ap->isol_detail = HFC_ISOLATE_PORT_C;
			
			/* Wait until DMA transfer ends */
			udelay(1000);
		}
		else {
			hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x30, ap, NULL, NULL, 0, 0, 0);
//			printk(KERN_ERR "hfcldd%d :Issue firmware isolate request .\n",ap->dev_minor);

			/* Issue firmware isolate request */
			hfc_write_reg(ap,( uint )HFC_IOSPACE_FRAMEA,( char )0x4, HFC_FRAMEA_FW_ISOL_PORT );
			set_bit(HFC_ISOL, (ulong *)&ap->status);

			if (proc == TRUE) {
				ap->isol_detail = HFC_ISOLATE_PORT_C;
			}
			else {
				/* Issue control reset */
				hfc_watchdog_enter(ap, NULL, NULL, 0, HFC_ISOLATE_DELAY_TMR, 0, TRUE);
				hfc_watchdog_enter(ap, NULL, NULL, 0, HFC_ISOLATE_DELAY_TMR, 0, FALSE);

//				printk(KERN_ERR "hfcldd%d : call HFC_ISOLATE_DELAY_TMR.\n",ap->dev_minor);

				hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x31, ap, NULL, NULL, 0, 0, 0);
				return 0;
			}
		}
		
		if (proc == TRUE) {
			hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x41, ap, NULL, NULL, 0, 0, 0);
			return 0;
		};
	}
	
	hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x50, ap, NULL, NULL, 0, 0, 0);
	/* Cancel scsi command and mailbox request */
	HFC_MAILBOX_UNLOCK( ap, HFC_MAILBOX_BUSY );

	/* Cancel scsi command */
//	printk(" cancel scsi command\n");
	for (lp=0 ; lp<MAX_TARGET_PROBE ; lp++) {

//		target = hfc_hash_target_info(ap, lp);
		target = hfc_hash_target_valid(ap, lp);		/* FCLNX-703 */
		if (target != NULL)
		{
			if ( test_bit(HFC_WWN_VALID, (ulong *)&target->flags) ){	/* FCLNX-703 */
				hfc_cancel_scsi_cmd(
					ap, target, 0, NULL, SCS_MCK, HFC_CSCSI_ERROR,
							TRUE, TRUE, HFC_FLASH_TARGET);
			}
			
			if ( hfc_manage_info.hfcldd_mp_mod ) {
				if ((ap->isol_detail == HFC_ISOLATE_PORT_C)
				 || (ap->isol_detail == HFC_ISOLATE_CHKSTP_C)) {
					hfc_manage_info.npubp->hfc_forced_offline_c(target, TRUE);
				}
				hfc_manage_info.npubp->hfc_forced_offline_e(target, TRUE);	/* FCLNX-704 */
			}
		}
	}
	
	/* wake up mailbox request */
//	printk(" wake up mailbox task and cancel all mailbox request \n");

	if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status) ){
		hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
	}

	if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) ||
		 (ap->initialize == 1) )
	{
		unlock_mailbox( ap ) ;
		hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);
	}

	if (ap->pkg.port <= 1) {
		hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x55, ap, NULL, NULL, 0, 0, 0);

		/* Issue control reset */
//		printk(KERN_ERR " issue control reset\n");
		set_bit( HFC_CTLRST_PROCESS, (ulong *)&mpap->status );

		if ( ctlres_flag == 0 ){ /* Issue control reset only once */
//			printk(" control reset\n");	
			hfc_reset_start(ap, HFC_CTLRST);
			ctlres_flag = 1;
		}

		/* wait until completion of CTLRST */
		udelay(1000);
	 	clear_bit( HFC_CTLRST_PROCESS, (ulong *)&mpap->status );
 
		/*-- 1.67 LED 0x44 dummy write(H/W Bug) --*/
		/* Stop optical transmission */
		hfc_write_reg(ap,(uint)HFC_IOSPACE_CMDFCIF,(char)0x4,(char)0x80808080);
			
		/* Turn LED (Yellow and Green) off */
		hfc_write_reg(
			ap, ( uint )HFC_IOSPACE_CMDLED,( char )0x4,HFC_WAKE_UP_FAILURE_FIVE);
	}
	
	memset(logdata, 0, 16);
	logdata[0]=ap->c_err;
	logdata[1]=ap->isol_detail;

	if ((ap->isol_detail == HFC_ISOLATE_PORT_C)
	 || (ap->isol_detail == HFC_ISOLATE_CHKSTP_C)) {
		eno = (ap->pkg.port <= 1) ? 0x8E : 0xD4;

		hfc_errlog(
				ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE,
				ERRID_HFCP_EVNT2, eno, logdata, 16);

		HFC_ERRPRT("hfcldd : Device %02x:%02x.%02x is isolated by user command \n",
				ap->pci_cfginf->bus->number,
				PCI_SLOT(ap->pci_cfginf->devfn),
				PCI_FUNC(ap->pci_cfginf->devfn));
	}
	else {
		eno = (ap->pkg.port <= 1) ? 0x8F : 0xD5;

		hfc_errlog(
				ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE,
				ERRID_HFCP_EVNT2, eno, logdata, 16);
		
		HFC_ERRPRT("hfcldd : Device %02x:%02x.%02x is isolated by error \n",
				ap->pci_cfginf->bus->number,
				PCI_SLOT(ap->pci_cfginf->devfn),
				PCI_FUNC(ap->pci_cfginf->devfn));
	}

	hfc_hand2_trace(HFC_TRC_FORCE_ISOL, 0x10, ap, NULL, NULL, 0, 0, 0);

	HFC_EXIT("hfc_force_linkdown");                  //FCLNX-0488
	return 0;                                      //FCLNX-0488
}


/*
 * Function:    hfc_force_linkdown_recovery
 *
 * Purpose:     Recovery for isolated Aaapter port
 *
 * Arguments:
 *  ap         -
 *  err_no     -
 *  mode       -
 *
 * Returns:
 *
 * Notes:
 */
int hfc_force_linkdown_recovery(struct adap_info *ap){
	struct	mp_adap_info 	*mpap;

	HFC_ENTRY("hfc_force_linkdown_recovery");					/*FCLNX-0506*/
	
	hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC, 0x00, ap, NULL, NULL, 0, 0, 0);
	
	if ( HFC_MMODE_CHECK_MLPF(ap) ) {
		hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC, 0x01, ap, NULL, NULL, 0, 0, 0);
		return EINVAL;
	}

	if (test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) {
		hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC, 0x04, ap, NULL, NULL, 0, 0, 0);	/* FCLNX-0707 */
		return EINVAL;
	}
	
	if (ap->pkg.port > 1) { /* FCLNX-583 */
		hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC, 0x02, ap, NULL, NULL, 0, 0, 0);
		return EINVAL;
	}

	/* This adapter is not an isolated state */
	mpap = ap->mp_adap_info;
	if (!test_bit ( HFC_HWISOL, (ulong *)&mpap->status)){
		hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC, 0x03, ap, NULL, NULL, 0, 0, 0);
		return EINVAL;
	}

	/* Adapter isolation recovery process is in progress */
	clear_bit ( HFC_ISOL, (ulong *)&ap->status);
	ap->isol_detail = HFC_NO_ISOLATE;
	ap->c_err = 0x00;
	set_bit ( HFC_ISOL_RECOVERY, (ulong *)&ap->status);
	
	/* Issue control reset */
	hfc_watchdog_enter(ap, NULL, NULL, 0, HFC_CTLRST_DELAY_TMR, 0, TRUE);   /* FCLNX-0276 */
	hfc_watchdog_enter(ap, NULL, NULL, 0, HFC_CTLRST_DELAY_TMR, 1, FALSE);  /* FCLNX-0276 */
	
//	ap->isol_force = HFC_NO_FRC_ISOL; /* FCLNX-576 */

	hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC, 0x10, ap, NULL, NULL, 0, 0, 0);

	HFC_EXIT("hfc_force_linkdown_recovery");	/*FCLNX-0506*/
	return (0);
}


/*
 * Function:    hfc_force_linkdown_port_recovery
 *
 * Purpose:     Recovery for isolated Aaapter port
 *
 * Arguments:
 *  ap         -
 *  err_no     -
 *  mode       -
 *
 * Returns:
 *
 * Notes:
 */
int hfc_force_linkdown_recovery_port(struct adap_info *ap, int immdt_cmd){
	int 					flags;
	struct	mp_adap_info	*mpap;

	HFC_ENTRY("hfc_force_linkdown_port_recovery");

	mpap = ap->mp_adap_info;
	HFC_ADAPLOCK_IRQSAVE(flags);
	
	hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC_P, 0x00, ap, NULL, NULL, 0, 0, 0);

	if ( HFC_MMODE_CHECK_MLPF(ap) ) {
		hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC_P, 0x01, ap, NULL, NULL, 0, 0, 0);
		HFC_ADAPUNLOCK_IRQRESTORE(flags);
		return EINVAL;
	}

	if (test_bit(HFC_CHK_STOP, (ulong *)&ap->status)) {
		hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC_P, 0x02, ap, NULL, NULL, 0, 0, 0);	/* FCLNX-0707 */
		HFC_ADAPUNLOCK_IRQRESTORE(flags);
		return EINVAL;
	}
	
	HFC_ADAP_LOCK(mpap,HFC_MP_ADAP_BUSY);
	
	hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_RST,( char )0x4, (int)(0xffffffff));
	hfc_write_reg(ap,( uint )HFC_IOSPACE_CA_RSTINFO,( char )0x1,(char)0x00);
	hfc_write_reg(ap,( uint )HFC_IOSPACE_FRAMEA,( char )0x4, HFC_FRAMEA_FW_START_PORT );
	hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_INTA_MASK);

	set_bit(HFC_NEED_LINK_INIT, (ulong *)&ap->status);
	clear_bit(HFC_ISOL, (ulong *)&ap->status);
	ap->isol_detail = HFC_NO_ISOLATE;
	ap->c_err = 0x00;

	hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC_P, 0x01, ap, NULL, NULL, 0, 0, 0);

	HFC_ADAP_UNLOCK( mpap, HFC_MP_ADAP_BUSY );
	HFC_ADAPUNLOCK_IRQRESTORE(flags);

	hfc_initialize(ap, immdt_cmd);
	
	if ( hfc_manage_info.hfcldd_mp_mod ) {
		hfc_manage_info.npubp->hfc_make_lgpath();
	}
	
	HFC_ADAPLOCK_IRQSAVE(flags);
	clear_bit(HFC_NEED_LINK_INIT, (ulong *)&ap->status);

	hfc_errlog(
		ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE,
		ERRID_HFCP_EVNT2, 0xD3, NULL, 0);

	HFC_ERRPRT("hfcldd : Device %02x:%02x.%02x is recovered \n",
		ap->pci_cfginf->bus->number,
		PCI_SLOT(ap->pci_cfginf->devfn),
		PCI_FUNC(ap->pci_cfginf->devfn));

	hfc_hand2_trace(HFC_TRC_FORCE_ISOL_REC_P, 0x10, ap, NULL, NULL, 0, 0, 0);
	HFC_ADAPUNLOCK_IRQRESTORE(flags);

	HFC_EXIT("hfc_force_linkdown_port_recovery");
	return(0);
}


/*
 * Function:    hfc_hand2_trace
 *
 * Purpose:     Collection of trace
 *
 * Arguments:   
 *  id         - 
 *  sub_id     - 
 *  ap         - 
 *  target     - 
 *  hfcp       - 
 *  etc1       - 
 *  etc2       - 
 *  etc3       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_hand2_trace(
	uchar               id,
	uchar               sub_id,
	struct adap_info    *ap,
	struct target_info  *target,
	struct hfc_pkt		*hfcp,
	uint64_t            etc1,
	uint64_t            etc2,
	uint64_t            etc3)
{
	uchar               trc_wk[128] ;
	struct err_trc1    *trc1 ;
	struct err_trc2    *trc2 ;
	struct err_trc3    *trc3 ;
	struct err_trc4    *trc4 ;
	struct err_trc5    *trc5 ;
	struct err_trc6    *trc6 ;
	struct err_trc7    *trc7 ;
	struct scsi_cmnd    		*cmnd = NULL;
	uint				etc1_wk=0, etc2_wk=0;
	ushort				etc2_uwk, cmd_xob=0;
	uint64_t			cmnd64=0;
	uint				xob_no;
	uint				ser_num;
	

	memset(trc_wk,0,128) ;
	
/*----------------------------------------------*/
/*                  TRACE1                      */
/* etc1 = int_a_reg                             */
/* etc2 = NULL                                  */
/* etc3 = NULL                                  */
/*----------------------------------------------*/
	etc1_wk = (uint)etc1;
	etc2_wk = (uint)etc2;
	etc2_uwk = (ushort)etc2;
	if(hfcp != NULL ) cmnd = hfcp->cmd_pkt;
	if( id == HFC_TRC_HANDLER )
	{/*-- trace format 1 --*/
		trc1 = (struct err_trc1 *)trc_wk ;
		
		trc1->id = id ;
		trc1->sub_id = sub_id ;
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc1->a_status, ap->status);
			HFC_8L_TO_8B(trc1->a_scsi_id, ap->scsi_id);
		}
		if( etc1 != 0)
			HFC_4L_TO_4B(trc1->int_a_reg, etc1_wk);
	}/*-- trace format 1 --*/
	
/*----------------------------------------------*/
/*                  TRACE2                      */
/* etc1 = NULL                                  */
/* etc2 = NULL                                  */
/* etc3 = NULL                                  */
/*----------------------------------------------*/

	if( (id == HFC_TRC_MBRESP) ||
		(id == HFC_TRC_LINKRSP) ||
		(id == HFC_TRC_PDISCRSP)||
		(id == HFC_TRC_GIDFTRSP) ||
		(id == HFC_TRC_MIHLGRSP) ||
		(id == HFC_TRC_GPNIDRSP) ||
		(id == HFC_TRC_LGINRSP ) ||
		(id == HFC_TRC_GIDPNRSP))
	{/*-- trace format 2 --*/
		trc2 = (struct err_trc2 *)trc_wk ;
		
		trc2->id = id ;
		trc2->sub_id = sub_id ;
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc2->a_status, ap->status) ;
			HFC_8L_TO_8B(trc2->a_scsi_id, ap->scsi_id);
			trc2->mb_status = ap->mb_status ;
			HFC_2L_TO_2B(trc2->mb_retry_cnt, ap->mb_retry_cnt);
			HFC_LP_TO_BP(trc2->a_next_tstart, ap->next_tstart);
			HFC_MEMCPY(trc2->mb_resp,(uchar *)&(ap->mb->mb_resp.flag),68);
		}
		if( target != NULL )
		{
			HFC_4L_TO_4B(trc2->t_status, target->status);
			trc2->t_flags = target->flags ;
			trc2->t_pseq = target->pseq ;
			HFC_2L_TO_2B(trc2->t_id, target->device_flags);	/* FCLNX-0659 */
			HFC_8L_TO_8B(trc2->t_ww_name, target->ww_name);
			HFC_8L_TO_8B(trc2->t_node_name, target->node_name);
		}
		if( etc1 != 0)
			HFC_4L_TO_4B(trc2->passthrough_rsp, etc1_wk);
	}/*-- trace format 2 --*/
	
	if( (id == HFC_TRC_MBINT) ||
		(id == HFC_TRC_LINKDOWN_INT) ||
		(id == HFC_TRC_LINKUP_INT)||
		(id == HFC_TRC_PLOGI_INT) ||
		(id == HFC_TRC_LOGO_INT) ||
		(id == HFC_TRC_SCN_INT) ||
		(id == HFC_TRC_RSCN_INT))
	{/*-- trace format 2 --*/
		trc2 = (struct err_trc2 *)trc_wk ;
		
		trc2->id = id ;
		trc2->sub_id = sub_id ;
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc2->a_status, ap->status) ;
			HFC_8L_TO_8B(trc2->a_scsi_id, ap->scsi_id);
			trc2->mb_status = ap->mb_status ;
			HFC_2L_TO_2B(trc2->mb_retry_cnt, ap->mb_retry_cnt);
			HFC_LP_TO_BP(trc2->a_next_tstart, ap->next_tstart);
			HFC_MEMCPY(trc2->mb_resp,(uchar *)&(ap->mb->mb_intreq.type.fwintreq0.int_code),32) ;
			HFC_MEMCPY(&trc2->mb_resp[32],(uchar *)&(ap->mb->mb_intreq.type.fwintreq0.un.scn.els_type),36) ;
		}
		if( target != NULL )
		{
			HFC_4L_TO_4B(trc2->t_status, target->status);
			trc2->t_flags = target->flags ;
			trc2->t_pseq = target->pseq ;
			HFC_2L_TO_2B(trc2->t_id, target->device_flags);	/* FCLNX-0659 */
			HFC_8L_TO_8B(trc2->t_ww_name, target->ww_name);
			HFC_8L_TO_8B(trc2->t_node_name, target->node_name);
		}
	}/*-- trace format 2 --*/

/*----------------------------------------------*/
/*                  TRACE3                      */
/* etc1 = xrb_cnt                               */
/* etc2 = NULL                                  */
/* etc3 = NULL                                  */
/*----------------------------------------------*/

	if( id == HFC_TRC_XRBRSP )
	{/*-- trace format 3 --*/
		trc3 = (struct err_trc3 *)trc_wk ;
		
		trc3->id = id ;
		trc3->sub_id = sub_id ;
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc3->a_status, ap->status);
			HFC_8L_TO_8B(trc3->a_scsi_id, ap->scsi_id);
			trc3->xrb_outp=ap->fw_init_p->xrb_outp;
			trc3->xrb_inp=ap->fw_init_p->xrb_inp;
			HFC_2L_TO_2B(trc3->xrb_no,ap->xrb_no);	/* FCLNX-0659 */
			if(hfcp != NULL)
			{
				xob_no = (uint)hfcp->cmd_xob;
				HFC_4L_TO_4B(trc3->xob_no, xob_no);
			}
			HFC_MEMCPY(trc3->xrb,(char *)&(ap->xrb[ap->xrb_no].resp_iu.fcp_status2),6);
			HFC_MEMCPY(&trc3->xrb[6],(char*)&(ap->xrb[ap->xrb_no].resp_iu.sns_len),32);
			HFC_MEMCPY(&trc3->xrb[38],(char*)&(ap->xrb[ap->xrb_no].flag),32);
		}
		if( etc2 != 0 )
		{
			HFC_2L_TO_2B(trc3->xrb_no, etc2_uwk);		/* FCLNX-0659 */
		}
		if( hfcp != NULL ){ /*  */
			trc3->adap_status = hfcp->adap_status;
		}
	}/*-- trace format 3 --*/
	
/*----------------------------------------------*/
/*                  TRACE4                      */
/* etc1 = cmnd                                  */
/* etc2 = hfc_pkt                               */
/* etc3 = NULL                                  */
/*----------------------------------------------*/

	if( (id == HFC_TRC_SCSI_CHK) ||
		(id == HFC_TRC_MGM_CHK) ||
		(id == HFC_TRC_LINK_CHK) ||
		(id == HFC_TRC_DEQ_WE))
	{/*-- trace format 4 --*/
		trc4 = (struct err_trc4 *)trc_wk ;
		
		trc4->id = id ;
		trc4->sub_id = sub_id ;
		if( hfcp != NULL)
		{
			cmnd = (struct scsi_cmnd *)hfcp->cmd_pkt;
		}
		
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc4->a_status, ap->status);
			HFC_8L_TO_8B(trc4->a_scsi_id, ap->scsi_id);
			trc4->fcp_status = ap->xrb[ap->xrb_no].resp_iu.fcp_status2;
			trc4->scsi_status = ap->xrb[ap->xrb_no].resp_iu.scsi_status;
		}
		if( target != NULL )
		{
			HFC_4L_TO_4B(trc4->t_status, target->status);
			trc4->t_flags = target->flags ;
			trc4->t_pseq = target->pseq ;
			HFC_4L_TO_4B(trc4->t_we_que_cnt, target->we_que_cnt);
			trc4->s_target = (uchar)target->target_id;
		}
		if( cmnd != NULL )
		{
			cmnd64 = (ulong)cmnd;
			HFC_LP_TO_BP(trc4->srb, cmnd64);
			trc4->s_function = cmnd->cmnd[0];
//			trc4->s_target = CMND_TARGET(cmnd);
//			trc4->s_lun = CMND_LUN(cmnd);
			trc4->s_quetag = 0;
//			trc4->s_queaction = CMND_CHANNEL(cmnd);
			trc4->s_cdblen = cmnd->cmd_len;
			trc4->s_senselen = cmnd->request_bufflen;

			HFC_4L_TO_4B(trc4->s_datatransferlength, cmnd->request_bufflen);
			HFC_4L_TO_4B(trc4->result, cmnd->result);
			HFC_4L_TO_4B(trc4->resid, cmnd->resid);
			ser_num = (uint)cmnd->serial_number;
			HFC_4L_TO_4B(trc4->serial_number, ser_num);
		}
		if( hfcp != NULL)
		{
			HFC_LP_TO_BP(trc4->hfcp, hfcp);
			HFC_4L_TO_4B(trc4->h_adap_status, hfcp->adap_status);
			HFC_4L_TO_4B(trc4->h_iov_no, hfcp->iov_no);
			HFC_4L_TO_4B(trc4->h_iov_cnt, hfcp->iov_cnt);
			trc4->s_lun = (uchar)hfcp->lun_id;
		}
	}/*-- trace format 4 --*/
	
/*----------------------------------------------*/
/*                  TRACE5                      */
/* etc1 = cmnd                                  */
/* etc2 = timer_id                              */
/* etc3 = NULL                                  */
/*----------------------------------------------*/

	if( id == HFC_TRC_WDOG )
	{/*-- trace format 5 --*/
		trc5 = (struct err_trc5 *)trc_wk ;
		trc5->id = id ;
		trc5->sub_id = sub_id ;
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc5->a_status, ap->status);
			HFC_8L_TO_8B(trc5->a_scsi_id, ap->scsi_id);
			HFC_2L_TO_2B(trc5->xrb_no, ap->xrb_no);
		}
		if( target != NULL )
		{
			HFC_4L_TO_4B(trc5->t_status, target->status);
			trc5->t_flags = target->flags ;
			trc5->t_pseq = target->pseq ;
			trc5->s_target = (uchar)target->target_id;
		}
		if( hfcp != NULL)
		{
			cmnd = (struct scsi_cmnd *)hfcp->cmd_pkt;
			cmd_xob = (ushort)hfcp->cmd_xob;		/* FCLNX-0659 */
			HFC_2L_TO_2B(trc5->xrb_no, cmd_xob);	/* FCLNX-0659 */
			trc5->s_lun = (uchar)hfcp->lun_id;
		}
		if( cmnd != NULL )
		{
			cmnd64 = (ulong)cmnd;
			HFC_LP_TO_BP(trc5->srb, cmnd64);
			trc5->s_function = cmnd->cmnd[0];
//			trc5->s_target = CMND_TARGET(cmnd);
//			trc5->s_lun = CMND_LUN(cmnd);
		}
		if( etc2 != 0 )
		{
			HFC_2L_TO_2B(trc5->timer_id, etc2_uwk);
		}
	}/*-- trace format 5 --*/
	
/*----------------------------------------------*/
/*                  TRACE6                      */
/* etc1 = int_a_reg                             */
/* etc2 = status_reg                            */
/* etc3 = detail_reg                            */
/*----------------------------------------------*/

	if( (id == HFC_TRC_CHKSTP) ||
		(id == HFC_TRC_ABEND) ||
		(id == HFC_TRC_MCKREC) ||
		(id == HFC_TRC_HWERR) )
	{/*-- trace format 6 --*/
		trc6 = (struct err_trc6 *)trc_wk ;
		
		trc6->id = id ;
		trc6->sub_id = sub_id ;
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc6->a_status, ap->status);
			HFC_8L_TO_8B(trc6->a_scsi_id, ap->scsi_id);
			HFC_4L_TO_4B(trc6->link_dead_cnt, ap->link_dead_cnt);
			HFC_4L_TO_4B(trc6->pci_err_cnt, ap->pci_err_cnt);
			HFC_4L_TO_4B(trc6->mck_err_cnt, ap->mp_adap_info->mck_err_cnt);	/* FCLNX-GPL-057 */
		}
		if( etc1 != 0)
			trc6->int_a_reg = (uint)etc1;
		if( etc2 != 0)
			trc6->status_reg = etc2 ;
		if( etc3 != 0)
			trc6->detail_reg = (uint)etc3;
	}
	
/*----------------------------------------------*/
/*                  TRACE7                      */
/* etc1 =                                       */
/* etc2 =                                       */
/* etc3 =                                       */
/*----------------------------------------------*/

	if( (id == HFC_TRC_FORCE_ISOL)  ||
		(id == HFC_TRC_FORCE_ISOL_REC)  ||
		(id == HFC_TRC_FORCE_ISOL_REC_P) )
	{/*-- trace format 7 --*/
		trc7 = (struct err_trc7 *)trc_wk ;
		
		trc7->id = id ;
		trc7->sub_id = sub_id ;
		if( ap != NULL )
		{
			HFC_4L_TO_4B(trc7->a_status, ap->status);
			HFC_8L_TO_8B(trc7->a_scsi_id, ap->scsi_id);
			HFC_4L_TO_4B(trc7->link_dead_cnt, ap->link_dead_cnt);
			HFC_4L_TO_4B(trc7->pci_err_cnt, ap->pci_err_cnt);
			if (ap->mp_adap_info != NULL) {
				HFC_4L_TO_4B(trc7->mck_err_cnt, ap->mp_adap_info->mck_err_cnt);
				HFC_4L_TO_4B(trc7->mpap_sts, ap->mp_adap_info->status);
			}
			else {
				trc7->mck_err_cnt = 0xffffffff;
				trc7->mpap_sts    = 0xffffffff;
			}
			trc7->isol_detail = ap->isol_detail;
			trc7->c_err = ap->c_err;
		}
		trc7->etc1 = (uint)etc1;
		trc7->etc2 = (uint)etc2;
		trc7->etc3 = (uint)etc3;
	}
	
	hfc_trace(ap,id,&trc_wk[1],0);
}
